@@ -73,6 +73,8 @@ public:
 	static void RequestRestart(void);
 	static void RequestReopenLogs(void);
 
+	static bool IsShuttingDown(void);
+
 	static void SetDebuggingSeverity(LogSeverity severity);
 	static LogSeverity GetDebuggingSeverity(void);
 
@@ -130,6 +132,7 @@ public:
 	static ThreadPool& GetTP(void);
 
 	static String GetAppVersion(void);
+	static String GetAppSpecVersion(void);
 
 	static double GetStartTime(void);
 	static void SetStartTime(double ts);
