@@ -50,12 +50,18 @@ public:
 	inline Array(void)
 	{ }
 
+	inline Array(std::initializer_list<Value> init)
+	    : m_Data(init)
+	{ }
+
 	inline ~Array(void)
 	{ }
 
 	Value Get(unsigned int index) const;
 	void Set(unsigned int index, const Value& value);
+	void Set(unsigned int index, Value&& value);
 	void Add(const Value& value);
+	void Add(Value&& value);
 
 	/**
 	 * Returns an iterator to the beginning of the array.
@@ -118,43 +124,40 @@ public:
 		return std::set<T>(Begin(), End());
 	}
 
+	template<typename T>
+	static Array::Ptr FromSet(const std::set<T>& v)
+	{
+		Array::Ptr result = new Array();
+		ObjectLock olock(result);
+		std::copy(v.begin(), v.end(), std::back_inserter(result->m_Data));
+		return result;
+	}
+
 	virtual Object::Ptr Clone(void) const override;
 
 	Array::Ptr Reverse(void) const;
 
+	void Sort(void);
+
 	virtual String ToString(void) const override;
 
+	virtual Value GetFieldByName(const String& field, bool sandboxed, const DebugInfo& debugInfo) const override;
+	virtual void SetFieldByName(const String& field, const Value& value, const DebugInfo& debugInfo) override;
+
 private:
 	std::vector<Value> m_Data; /**< The data for the array. */
 };
 
-inline Array::Iterator range_begin(Array::Ptr x)
+inline Array::Iterator begin(Array::Ptr x)
 {
 	return x->Begin();
 }
 
-inline Array::Iterator range_end(Array::Ptr x)
+inline Array::Iterator end(Array::Ptr x)
 {
 	return x->End();
 }
 
 }
 
-namespace boost
-{
-
-template<>
-struct range_mutable_iterator<icinga::Array::Ptr>
-{
-	typedef icinga::Array::Iterator type;
-};
-
-template<>
-struct range_const_iterator<icinga::Array::Ptr>
-{
-	typedef icinga::Array::Iterator type;
-};
-
-}
-
 #endif /* ARRAY_H */
