@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -33,9 +33,7 @@
 #include "base/workqueue.hpp"
 #include "base/context.hpp"
 #include "base/application.hpp"
-#include "config/configitem.hpp"
 #include <fstream>
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/classification.hpp>
 #include <boost/exception/errinfo_api_function.hpp>
@@ -51,11 +49,6 @@ boost::signals2::signal<void (const ConfigObject::Ptr&)> ConfigObject::OnStateCh
 ConfigObject::ConfigObject(void)
 { }
 
-ConfigType::Ptr ConfigObject::GetType(void) const
-{
-	return ConfigType::GetByName(GetReflectionType()->GetName());
-}
-
 bool ConfigObject::IsActive(void) const
 {
 	return GetActive();
@@ -103,7 +96,8 @@ class ModAttrValidationUtils : public ValidationUtils
 public:
 	virtual bool ValidateName(const String& type, const String& name) const override
 	{
-		ConfigType::Ptr dtype = ConfigType::GetByName(type);
+		Type::Ptr ptype = Type::GetByName(type);
+		ConfigType *dtype = dynamic_cast<ConfigType *>(ptype.get());
 
 		if (!dtype)
 			return false;
@@ -163,11 +157,9 @@ void ConfigObject::ModifyAttribute(const String& attr, const Value& value, bool
 			const String& key = tokens[i];
 			prefix += "." + key;
 
-			if (!dict->Contains(key)) {
+			if (!dict->Get(key, &current)) {
 				current = new Dictionary();
 				dict->Set(key, current);
-			} else {
-				current = dict->Get(key);
 			}
 		}
 
@@ -188,7 +180,7 @@ void ConfigObject::ModifyAttribute(const String& attr, const Value& value, bool
 			if (oldValue.IsObjectType<Dictionary>()) {
 				Dictionary::Ptr oldDict = oldValue;
 				ObjectLock olock(oldDict);
-				BOOST_FOREACH(const Dictionary::Pair& kv, oldDict) {
+				for (const auto& kv : oldDict) {
 					String key = prefix + "." + kv.first;
 					if (!original_attributes->Contains(key))
 						original_attributes->Set(key, kv.second);
@@ -198,7 +190,7 @@ void ConfigObject::ModifyAttribute(const String& attr, const Value& value, bool
 				if (value.IsObjectType<Dictionary>()) {
 					Dictionary::Ptr valueDict = value;
 					ObjectLock olock(valueDict);
-					BOOST_FOREACH(const Dictionary::Pair& kv, valueDict) {
+					for (const auto& kv : valueDict) {
 						String key = attr + "." + kv.first;
 						if (!original_attributes->Contains(key))
 							original_attributes->Set(key, Empty);
@@ -242,7 +234,6 @@ void ConfigObject::RestoreAttribute(const String& attr, bool updateVersion)
 	String fieldName = tokens[0];
 
 	int fid = type->GetFieldId(fieldName);
-	Field field = type->GetFieldInfo(fid);
 
 	Value currentValue = GetField(fid);
 
@@ -259,7 +250,7 @@ void ConfigObject::RestoreAttribute(const String& attr, bool updateVersion)
 		Value current = newValue;
 
 		if (current.IsEmpty())
-			BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot restore non-existing object attribute"));
+			BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot restore non-existent object attribute"));
 
 		String prefix = tokens[0];
 
@@ -273,7 +264,7 @@ void ConfigObject::RestoreAttribute(const String& attr, bool updateVersion)
 			prefix += "." + key;
 
 			if (!dict->Contains(key))
-				BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot restore non-existing object attribute"));
+				BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot restore non-existent object attribute"));
 
 			current = dict->Get(key);
 		}
@@ -290,7 +281,7 @@ void ConfigObject::RestoreAttribute(const String& attr, bool updateVersion)
 
 		{
 			ObjectLock olock(original_attributes);
-			BOOST_FOREACH(const Dictionary::Pair& kv, original_attributes) {
+			for (const auto& kv : original_attributes) {
 				std::vector<String> originalTokens;
 				boost::algorithm::split(originalTokens, kv.first, boost::is_any_of("."));
 
@@ -333,7 +324,7 @@ void ConfigObject::RestoreAttribute(const String& attr, bool updateVersion)
 			}
 		}
 
-		BOOST_FOREACH(const String& attr, restoredAttrs)
+		for (const String& attr : restoredAttrs)
 			original_attributes->Remove(attr);
 
 
@@ -362,16 +353,16 @@ void ConfigObject::Register(void)
 {
 	ASSERT(!OwnsLock());
 
-	ConfigType::Ptr dtype = GetType();
-	dtype->RegisterObject(this);
+	TypeImpl<ConfigObject>::Ptr type = static_pointer_cast<TypeImpl<ConfigObject> >(GetReflectionType());
+	type->RegisterObject(this);
 }
 
 void ConfigObject::Unregister(void)
 {
 	ASSERT(!OwnsLock());
 
-	ConfigType::Ptr dtype = GetType();
-	dtype->UnregisterObject(this);
+	TypeImpl<ConfigObject>::Ptr type = static_pointer_cast<TypeImpl<ConfigObject> >(GetReflectionType());
+	type->UnregisterObject(this);
 }
 
 void ConfigObject::Start(bool runtimeCreated)
@@ -385,19 +376,20 @@ void ConfigObject::Start(bool runtimeCreated)
 
 void ConfigObject::Activate(bool runtimeCreated)
 {
-	CONTEXT("Activating object '" + GetName() + "' of type '" + GetType()->GetName() + "'");
-
-	Start(runtimeCreated);
-
-	ASSERT(GetStartCalled());
+	CONTEXT("Activating object '" + GetName() + "' of type '" + GetReflectionType()->GetName() + "'");
 
 	{
 		ObjectLock olock(this);
+
+		Start(runtimeCreated);
+
+		ASSERT(GetStartCalled());
 		ASSERT(!IsActive());
 		SetActive(true, true);
-	}
 
-	SetAuthority(true);
+		if (GetHAMode() == HARunEverywhere)
+			SetAuthority(true);
+	}
 
 	NotifyActive();
 }
@@ -413,9 +405,7 @@ void ConfigObject::Stop(bool runtimeRemoved)
 
 void ConfigObject::Deactivate(bool runtimeRemoved)
 {
-	CONTEXT("Deactivating object '" + GetName() + "' of type '" + GetType()->GetName() + "'");
-
-	SetAuthority(false);
+	CONTEXT("Deactivating object '" + GetName() + "' of type '" + GetReflectionType()->GetName() + "'");
 
 	{
 		ObjectLock olock(this);
@@ -424,9 +414,11 @@ void ConfigObject::Deactivate(bool runtimeRemoved)
 			return;
 
 		SetActive(false, true);
-	}
 
-	Stop(runtimeRemoved);
+		SetAuthority(false);
+
+		Stop(runtimeRemoved);
+	}
 
 	ASSERT(GetStopCalled());
 
@@ -440,7 +432,17 @@ void ConfigObject::OnConfigLoaded(void)
 
 void ConfigObject::OnAllConfigLoaded(void)
 {
-	/* Nothing to do here. */
+	static ConfigType *ctype;
+
+	if (!ctype) {
+		Type::Ptr type = Type::GetByName("Zone");
+		ctype = dynamic_cast<ConfigType *>(type.get());
+	}
+
+	String zoneName = GetZoneName();
+
+	if (!zoneName.IsEmpty())
+		m_Zone = ctype->GetObject(zoneName);
 }
 
 void ConfigObject::CreateChildObjects(const Type::Ptr& childType)
@@ -465,16 +467,18 @@ void ConfigObject::Resume(void)
 
 void ConfigObject::SetAuthority(bool authority)
 {
+	ObjectLock olock(this);
+
 	if (authority && GetPaused()) {
 		SetResumeCalled(false);
 		Resume();
 		ASSERT(GetResumeCalled());
 		SetPaused(false);
 	} else if (!authority && !GetPaused()) {
+		SetPaused(true);
 		SetPauseCalled(false);
 		Pause();
 		ASSERT(GetPauseCalled());
-		SetPaused(true);
 	}
 }
 
@@ -483,18 +487,22 @@ void ConfigObject::DumpObjects(const String& filename, int attributeTypes)
 	Log(LogInformation, "ConfigObject")
 	    << "Dumping program state to file '" << filename << "'";
 
-	String tempFilename = filename + ".tmp";
-
 	std::fstream fp;
-	fp.open(tempFilename.CStr(), std::ios_base::out);
+	String tempFilename = Utility::CreateTempFile(filename + ".XXXXXX", 0600, fp);
+	fp.exceptions(std::ofstream::failbit | std::ofstream::badbit);
 
 	if (!fp)
 		BOOST_THROW_EXCEPTION(std::runtime_error("Could not open '" + tempFilename + "' file"));
 
 	StdioStream::Ptr sfp = new StdioStream(&fp, false);
 
-	BOOST_FOREACH(const ConfigType::Ptr& type, ConfigType::GetTypes()) {
-		BOOST_FOREACH(const ConfigObject::Ptr& object, type->GetObjects()) {
+	for (const Type::Ptr& type : Type::GetAllTypes()) {
+		ConfigType *dtype = dynamic_cast<ConfigType *>(type.get());
+
+		if (!dtype)
+			continue;
+
+		for (const ConfigObject::Ptr& object : dtype->GetObjects()) {
 			Dictionary::Ptr persistentObject = new Dictionary();
 
 			persistentObject->Set("type", type->GetName());
@@ -534,20 +542,13 @@ void ConfigObject::RestoreObject(const String& message, int attributeTypes)
 	Dictionary::Ptr persistentObject = JsonDecode(message);
 
 	String type = persistentObject->Get("type");
-
-	ConfigType::Ptr dt = ConfigType::GetByName(type);
-
-	if (!dt)
-		return;
-
 	String name = persistentObject->Get("name");
 
-	ConfigObject::Ptr object = dt->GetObject(name);
+	ConfigObject::Ptr object = GetObject(type, name);
 
 	if (!object)
 		return;
 
-	ASSERT(!object->IsActive());
 #ifdef I2_DEBUG
 	Log(LogDebug, "ConfigObject")
 	    << "Restoring object '" << name << "' of type '" << type << "'.";
@@ -574,6 +575,7 @@ void ConfigObject::RestoreObjects(const String& filename, int attributeTypes)
 	unsigned long restored = 0;
 
 	WorkQueue upq(25000, Application::GetConcurrency());
+	upq.SetName("ConfigObject::RestoreObjects");
 
 	String message;
 	StreamReadContext src;
@@ -596,8 +598,13 @@ void ConfigObject::RestoreObjects(const String& filename, int attributeTypes)
 
 	unsigned long no_state = 0;
 
-	BOOST_FOREACH(const ConfigType::Ptr& type, ConfigType::GetTypes()) {
-		BOOST_FOREACH(const ConfigObject::Ptr& object, type->GetObjects()) {
+	for (const Type::Ptr& type : Type::GetAllTypes()) {
+		ConfigType *dtype = dynamic_cast<ConfigType *>(type.get());
+
+		if (!dtype)
+			continue;
+
+		for (const ConfigObject::Ptr& object : dtype->GetObjects()) {
 			if (!object->GetStateLoaded()) {
 				object->OnStateLoaded();
 				object->SetStateLoaded(true);
@@ -613,8 +620,13 @@ void ConfigObject::RestoreObjects(const String& filename, int attributeTypes)
 
 void ConfigObject::StopObjects(void)
 {
-	BOOST_FOREACH(const ConfigType::Ptr& dt, ConfigType::GetTypes()) {
-		BOOST_FOREACH(const ConfigObject::Ptr& object, dt->GetObjects()) {
+	for (const Type::Ptr& type : Type::GetAllTypes()) {
+		ConfigType *dtype = dynamic_cast<ConfigType *>(type.get());
+
+		if (!dtype)
+			continue;
+
+		for (const ConfigObject::Ptr& object : dtype->GetObjects()) {
 			object->Deactivate();
 		}
 	}
@@ -622,15 +634,20 @@ void ConfigObject::StopObjects(void)
 
 void ConfigObject::DumpModifiedAttributes(const boost::function<void(const ConfigObject::Ptr&, const String&, const Value&)>& callback)
 {
-	BOOST_FOREACH(const ConfigType::Ptr& dt, ConfigType::GetTypes()) {
-		BOOST_FOREACH(const ConfigObject::Ptr& object, dt->GetObjects()) {
+	for (const Type::Ptr& type : Type::GetAllTypes()) {
+		ConfigType *dtype = dynamic_cast<ConfigType *>(type.get());
+
+		if (!dtype)
+			continue;
+
+		for (const ConfigObject::Ptr& object : dtype->GetObjects()) {
 			Dictionary::Ptr originalAttributes = object->GetOriginalAttributes();
 
 			if (!originalAttributes)
 				continue;
 
 			ObjectLock olock(originalAttributes);
-			BOOST_FOREACH(const Dictionary::Pair& kv, originalAttributes) {
+			for (const Dictionary::Pair& kv : originalAttributes) {
 				String key = kv.first;
 
 				Type::Ptr type = object->GetReflectionType();
@@ -679,8 +696,19 @@ void ConfigObject::DumpModifiedAttributes(const boost::function<void(const Confi
 
 ConfigObject::Ptr ConfigObject::GetObject(const String& type, const String& name)
 {
-	ConfigType::Ptr dtype = ConfigType::GetByName(type);
-	if (!dtype)
+	Type::Ptr ptype = Type::GetByName(type);
+	ConfigType *ctype = dynamic_cast<ConfigType *>(ptype.get());
+
+	if (!ctype)
 		return ConfigObject::Ptr();
-	return dtype->GetObject(name);
+
+	return ctype->GetObject(name);
 }
+
+ConfigObject::Ptr ConfigObject::GetZone(void) const
+{
+	return m_Zone;
+}
+
+NameComposer::~NameComposer(void)
+{ }
