@@ -17,75 +17,25 @@
  * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.             *
  ******************************************************************************/
 
-#include "base/configtype.hpp"
-#include "base/serializer.hpp"
-#include "base/debug.hpp"
-#include "base/objectlock.hpp"
+#include "base/configobject.hpp"
 #include "base/convert.hpp"
 #include "base/exception.hpp"
 
 using namespace icinga;
 
-ConfigType::ConfigType(const String& name)
-	: m_Name(name)
+ConfigType::~ConfigType(void)
 { }
 
-ConfigType::Ptr ConfigType::GetByName(const String& name)
+ConfigObject::Ptr ConfigType::GetObject(const String& name) const
 {
-	boost::mutex::scoped_lock lock(GetStaticMutex());
+	boost::mutex::scoped_lock lock(m_Mutex);
 
-	ConfigType::TypeMap::const_iterator tt = InternalGetTypeMap().find(name);
+	auto nt = m_ObjectMap.find(name);
 
-	if (tt == InternalGetTypeMap().end()) {
-		Type::Ptr type = Type::GetByName(name);
+	if (nt == m_ObjectMap.end())
+		return ConfigObject::Ptr();
 
-		if (!type || !ConfigObject::TypeInstance->IsAssignableFrom(type)
-		    || type->IsAbstract())
-			return ConfigType::Ptr();
-
-		ConfigType::Ptr dtype = new ConfigType(name);
-
-		InternalGetTypeMap()[type->GetName()] = dtype;
-		InternalGetTypeVector().push_back(dtype);
-
-		return dtype;
-	}
-
-	return tt->second;
-}
-
-/**
- * Note: Caller must hold ConfigType::GetStaticMutex() while using the map.
- */
-ConfigType::TypeMap& ConfigType::InternalGetTypeMap(void)
-{
-	static ConfigType::TypeMap typemap;
-	return typemap;
-}
-
-ConfigType::TypeVector& ConfigType::InternalGetTypeVector(void)
-{
-	static ConfigType::TypeVector typevector;
-	return typevector;
-}
-
-ConfigType::TypeVector ConfigType::GetTypes(void)
-{
-	boost::mutex::scoped_lock lock(GetStaticMutex());
-	return InternalGetTypeVector(); /* Making a copy of the vector here. */
-}
-
-std::pair<ConfigTypeIterator<ConfigObject>, ConfigTypeIterator<ConfigObject> > ConfigType::GetObjects(void)
-{
-	return std::make_pair(
-	    ConfigTypeIterator<ConfigObject>(this, 0),
-	    ConfigTypeIterator<ConfigObject>(this, -1)
-	);
-}
-
-String ConfigType::GetName(void) const
-{
-	return m_Name;
+	return nt->second;
 }
 
 void ConfigType::RegisterObject(const ConfigObject::Ptr& object)
@@ -93,15 +43,17 @@ void ConfigType::RegisterObject(const ConfigObject::Ptr& object)
 	String name = object->GetName();
 
 	{
-		ObjectLock olock(this);
+		boost::mutex::scoped_lock lock(m_Mutex);
 
-		ObjectMap::iterator it = m_ObjectMap.find(name);
+		auto it = m_ObjectMap.find(name);
 
 		if (it != m_ObjectMap.end()) {
 			if (it->second == object)
 				return;
 
-			BOOST_THROW_EXCEPTION(ScriptError("An object with type '" + m_Name + "' and name '" + name + "' already exists (" +
+			Type *type = dynamic_cast<Type *>(this);
+
+			BOOST_THROW_EXCEPTION(ScriptError("An object with type '" + type->GetName() + "' and name '" + name + "' already exists (" +
 			    Convert::ToString(it->second->GetDebugInfo()) + "), new declaration: " + Convert::ToString(object->GetDebugInfo()),
 			    object->GetDebugInfo()));
 		}
@@ -116,28 +68,26 @@ void ConfigType::UnregisterObject(const ConfigObject::Ptr& object)
 	String name = object->GetName();
 
 	{
-		ObjectLock olock(this);
+		boost::mutex::scoped_lock lock(m_Mutex);
 
 		m_ObjectMap.erase(name);
 		m_ObjectVector.erase(std::remove(m_ObjectVector.begin(), m_ObjectVector.end(), object), m_ObjectVector.end());
 	}
 }
 
-ConfigObject::Ptr ConfigType::GetObject(const String& name) const
+std::vector<ConfigObject::Ptr> ConfigType::GetObjects(void) const
 {
-	ObjectLock olock(this);
-
-	ConfigType::ObjectMap::const_iterator nt = m_ObjectMap.find(name);
-
-	if (nt == m_ObjectMap.end())
-		return ConfigObject::Ptr();
-
-	return nt->second;
+	boost::mutex::scoped_lock lock(m_Mutex);
+	return m_ObjectVector;
 }
 
-boost::mutex& ConfigType::GetStaticMutex(void)
+std::vector<ConfigObject::Ptr> ConfigType::GetObjectsHelper(Type *type)
 {
-	static boost::mutex mutex;
-	return mutex;
+	return static_cast<TypeImpl<ConfigObject> *>(type)->GetObjects();
 }
 
+int ConfigType::GetObjectCount(void) const
+{
+	boost::mutex::scoped_lock lock(m_Mutex);
+	return m_ObjectVector.size();
+}
