@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -18,9 +18,7 @@
  ******************************************************************************/
 
 #include "base/configwriter.hpp"
-#include "config/configcompiler.hpp"
 #include "base/exception.hpp"
-#include <boost/foreach.hpp>
 #include <boost/regex.hpp>
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/algorithm/string/split.hpp>
@@ -64,7 +62,7 @@ void ConfigWriter::EmitArrayItems(std::ostream& fp, int indentLevel, const Array
 	bool first = true;
 
 	ObjectLock olock(val);
-	BOOST_FOREACH(const Value& item, val) {
+	for (const Value& item : val) {
 		if (first)
 			first = false;
 		else
@@ -74,13 +72,14 @@ void ConfigWriter::EmitArrayItems(std::ostream& fp, int indentLevel, const Array
 	}
 }
 
-void ConfigWriter::EmitScope(std::ostream& fp, int indentLevel, const Dictionary::Ptr& val, const Array::Ptr& imports)
+void ConfigWriter::EmitScope(std::ostream& fp, int indentLevel, const Dictionary::Ptr& val,
+    const Array::Ptr& imports, bool splitDot)
 {
 	fp << "{";
 
 	if (imports && imports->GetLength() > 0) {
 		ObjectLock xlock(imports);
-		BOOST_FOREACH(const Value& import, imports) {
+		for (const Value& import : imports) {
 			fp << "\n";
 			EmitIndent(fp, indentLevel);
 			fp << "import \"" << import << "\"";
@@ -91,21 +90,24 @@ void ConfigWriter::EmitScope(std::ostream& fp, int indentLevel, const Dictionary
 
 	if (val) {
 		ObjectLock olock(val);
-		BOOST_FOREACH(const Dictionary::Pair& kv, val) {
+		for (const Dictionary::Pair& kv : val) {
 			fp << "\n";
 			EmitIndent(fp, indentLevel);
-			
-			std::vector<String> tokens;
-			boost::algorithm::split(tokens, kv.first, boost::is_any_of("."));
-			
-			EmitIdentifier(fp, tokens[0], true);
-			
-			for (std::vector<String>::size_type i = 1; i < tokens.size(); i++) {
-				fp << "[";
-				EmitString(fp, tokens[i]);
-				fp << "]";
-			}
-			
+
+			if (splitDot) {
+				std::vector<String> tokens;
+				boost::algorithm::split(tokens, kv.first, boost::is_any_of("."));
+
+				EmitIdentifier(fp, tokens[0], true);
+
+				for (std::vector<String>::size_type i = 1; i < tokens.size(); i++) {
+					fp << "[";
+					EmitString(fp, tokens[i]);
+					fp << "]";
+				}
+			} else
+				EmitIdentifier(fp, kv.first, true);
+
 			fp << " = ";
 			EmitValue(fp, indentLevel + 1, kv.second);
 		}
@@ -189,7 +191,7 @@ void ConfigWriter::EmitConfigItem(std::ostream& fp, const String& type, const St
 		fp << " ignore_on_error";
 
 	fp << " ";
-	EmitScope(fp, 1, attrs, imports);
+	EmitScope(fp, 1, attrs, imports, true);
 }
 
 void ConfigWriter::EmitComment(std::ostream& fp, const String& text)
@@ -240,6 +242,8 @@ const std::vector<String>& ConfigWriter::GetKeywords(void)
 		keywords.push_back("globals");
 		keywords.push_back("locals");
 		keywords.push_back("use");
+		keywords.push_back("__using");
+		keywords.push_back("default");
 		keywords.push_back("ignore_on_error");
 		keywords.push_back("current_filename");
 		keywords.push_back("current_line");
