@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -60,7 +60,8 @@ public:
 	static void EmitEmpty(std::ostream& fp);
 	static void EmitArray(std::ostream& fp, int indentLevel, const Array::Ptr& val);
 	static void EmitArrayItems(std::ostream& fp, int indentLevel, const Array::Ptr& val);
-	static void EmitScope(std::ostream& fp, int indentLevel, const Dictionary::Ptr& val, const Array::Ptr& imports = Array::Ptr());
+	static void EmitScope(std::ostream& fp, int indentLevel, const Dictionary::Ptr& val,
+	    const Array::Ptr& imports = Array::Ptr(), bool splitDot = false);
 	static void EmitValue(std::ostream& fp, int indentLevel, const Value& val);
 	static void EmitRaw(std::ostream& fp, const String& val);
 	static void EmitIndent(std::ostream& fp, int indentLevel);
@@ -74,9 +75,8 @@ public:
 
 	static const std::vector<String>& GetKeywords(void);
 private:
-	ConfigWriter(void);
-
 	static String EscapeIcingaString(const String& str);
+	ConfigWriter(void);
 };
 
 }
