@@ -18,7 +18,6 @@
  ******************************************************************************/
 
 #include "base/dependencygraph.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -35,8 +34,8 @@ void DependencyGraph::RemoveDependency(Object *parent, Object *child)
 {
 	boost::mutex::scoped_lock lock(m_Mutex);
 
-	std::map<Object *, int>& refs = m_Dependencies[child];
-	std::map<Object *, int>::iterator it = refs.find(parent);
+	auto& refs = m_Dependencies[child];
+	auto it = refs.find(parent);
 
 	if (it == refs.end())
 		return;
@@ -55,11 +54,11 @@ std::vector<Object::Ptr> DependencyGraph::GetParents(const Object::Ptr& child)
 	std::vector<Object::Ptr> objects;
 
 	boost::mutex::scoped_lock lock(m_Mutex);
-	std::map<Object *, std::map<Object *, int> >::const_iterator it = m_Dependencies.find(child.get());
+	auto it = m_Dependencies.find(child.get());
 
 	if (it != m_Dependencies.end()) {
 		typedef std::pair<Object *, int> kv_pair;
-		BOOST_FOREACH(const kv_pair& kv, it->second) {
+		for (const kv_pair& kv : it->second) {
 			objects.push_back(kv.first);
 		}
 	}
