@@ -91,4 +91,13 @@
 #	define I2_BASE_API I2_IMPORT
 #endif /* I2_BASE_BUILD */
 
+#if defined(__GNUC__)
+#	define likely(x) __builtin_expect(!!(x), 1)
+#	define unlikely(x) __builtin_expect(!!(x), 0)
+#else
+#	define likely(x) (x)
+#	define unlikely(x) (x)
+#endif
+
+
 #endif /* I2BASE_H */
