@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -22,13 +22,17 @@
 
 #include "base/object.hpp"
 
+#define I2MUTEX_UNLOCKED 0
+#define I2MUTEX_LOCKED 1
+
 namespace icinga
 {
 
 /**
  * A scoped lock for Objects.
  */
-struct I2_BASE_API ObjectLock {
+struct I2_BASE_API ObjectLock
+{
 public:
 	inline ObjectLock(void)
 		: m_Object(NULL), m_Locked(false)
@@ -53,11 +57,49 @@ public:
 			Lock();
 	}
 
+	inline static void LockMutex(const Object *object)
+	{
+		unsigned int it = 0;
+
+#ifdef _WIN32
+#	ifdef _WIN64
+		while (likely(InterlockedCompareExchange64((LONGLONG *)&object->m_Mutex, I2MUTEX_LOCKED, I2MUTEX_UNLOCKED) != I2MUTEX_UNLOCKED)) {
+#	else /* _WIN64 */
+		while (likely(InterlockedCompareExchange(&object->m_Mutex, I2MUTEX_LOCKED, I2MUTEX_UNLOCKED) != I2MUTEX_UNLOCKED)) {
+#	endif /* _WIN64 */
+#else /* _WIN32 */
+		while (likely(!__sync_bool_compare_and_swap(&object->m_Mutex, I2MUTEX_UNLOCKED, I2MUTEX_LOCKED))) {
+#endif /* _WIN32 */
+			if (likely(object->m_Mutex > I2MUTEX_LOCKED)) {
+				boost::recursive_mutex *mtx = reinterpret_cast<boost::recursive_mutex *>(object->m_Mutex);
+				mtx->lock();
+
+				return;
+			}
+
+			Spin(it);
+			it++;
+		}
+
+		boost::recursive_mutex *mtx = new boost::recursive_mutex();
+		mtx->lock();
+#ifdef _WIN32
+#	ifdef _WIN64
+		InterlockedCompareExchange64((LONGLONG *)&object->m_Mutex, reinterpret_cast<LONGLONG>(mtx), I2MUTEX_LOCKED);
+#	else /* _WIN64 */
+		InterlockedCompareExchange(&object->m_Mutex, reinterpret_cast<LONG>(mtx), I2MUTEX_LOCKED);
+#	endif /* _WIN64 */
+#else /* _WIN32 */
+		__sync_bool_compare_and_swap(&object->m_Mutex, I2MUTEX_LOCKED, reinterpret_cast<uintptr_t>(mtx));
+#endif /* _WIN32 */
+	}
+
 	inline void Lock(void)
 	{
 		ASSERT(!m_Locked && m_Object != NULL);
 
-		m_Object->m_Mutex.lock();
+		LockMutex(m_Object);
+
 		m_Locked = true;
 
 #ifdef I2_DEBUG
@@ -69,6 +111,25 @@ public:
 #endif /* I2_DEBUG */
 	}
 
+	inline static void Spin(unsigned int it)
+	{
+		if (it < 8) {
+			/* Do nothing. */
+		}
+#ifdef SPIN_PAUSE
+		else if (it < 16) {
+			SPIN_PAUSE();
+		}
+#endif /* SPIN_PAUSE */
+		else {
+#ifdef _WIN32
+			Sleep(0);
+#else /* _WIN32 */
+			sched_yield();
+#endif /* _WIN32 */
+		}
+	}
+
 	inline void Unlock(void)
 	{
 #ifdef I2_DEBUG
@@ -82,7 +143,7 @@ public:
 #endif /* I2_DEBUG */
 
 		if (m_Locked) {
-			m_Object->m_Mutex.unlock();
+			reinterpret_cast<boost::recursive_mutex *>(m_Object->m_Mutex)->unlock();
 			m_Locked = false;
 		}
 	}
