@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -20,8 +20,9 @@
 #ifndef SCRIPTFRAME_H
 #define SCRIPTFRAME_H
 
-#include "config/i2-config.hpp"
+#include "base/i2-base.hpp"
 #include "base/dictionary.hpp"
+#include "base/array.hpp"
 #include <boost/thread/tss.hpp>
 #include <stack>
 
@@ -33,18 +34,28 @@ struct I2_BASE_API ScriptFrame
 	Dictionary::Ptr Locals;
 	Value Self;
 	bool Sandboxed;
+	int Depth;
 
 	ScriptFrame(void);
 	ScriptFrame(const Value& self);
 	~ScriptFrame(void);
 
+	void IncreaseStackDepth(void);
+	void DecreaseStackDepth(void);
+
 	static ScriptFrame *GetCurrentFrame(void);
 
+	static Array::Ptr GetImports(void);
+	static void AddImport(const Object::Ptr& import);
+
 private:
 	static boost::thread_specific_ptr<std::stack<ScriptFrame *> > m_ScriptFrames;
+	static Array::Ptr m_Imports;
 
 	inline static void PushFrame(ScriptFrame *frame);
 	inline static ScriptFrame *PopFrame(void);
+
+	void InitializeFrame(void);
 };
 
 }
