@@ -26,7 +26,7 @@
 #include "base/convert.hpp"
 #include "base/objectlock.hpp"
 #include "base/exception.hpp"
-#include <boost/foreach.hpp>
+#include <boost/algorithm/string/split.hpp>
 #include <fstream>
 
 using namespace icinga;
@@ -35,19 +35,49 @@ Dictionary::Ptr ScriptGlobal::m_Globals = new Dictionary();
 
 Value ScriptGlobal::Get(const String& name, const Value *defaultValue)
 {
-	if (!m_Globals->Contains(name)) {
+	Value result;
+
+	if (!m_Globals->Get(name, &result)) {
 		if (defaultValue)
 			return *defaultValue;
 
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Tried to access undefined script variable '" + name + "'"));
 	}
 
-	return m_Globals->Get(name);
+	return result;
 }
 
 void ScriptGlobal::Set(const String& name, const Value& value)
 {
-	m_Globals->Set(name, value);
+	std::vector<String> tokens;
+	boost::algorithm::split(tokens, name, boost::is_any_of("."));
+
+	if (tokens.empty())
+		BOOST_THROW_EXCEPTION(std::invalid_argument("Name must not be empty"));
+
+	{
+		ObjectLock olock(m_Globals);
+
+		Dictionary::Ptr parent = m_Globals;
+
+		for (std::vector<String>::size_type i = 0; i < tokens.size(); i++) {
+			const String& token = tokens[i];
+
+			if (i + 1 != tokens.size()) {
+				Value vparent;
+
+				if (!parent->Get(token, &vparent)) {
+					Dictionary::Ptr dict = new Dictionary();
+					parent->Set(token, dict);
+					parent = dict;
+				} else {
+					parent = vparent;
+				}
+			}
+		}
+
+		parent->Set(tokens[tokens.size() - 1], value);
+	}
 }
 
 bool ScriptGlobal::Exists(const String& name)
@@ -74,7 +104,7 @@ void ScriptGlobal::WriteToFile(const String& filename)
 	StdioStream::Ptr sfp = new StdioStream(&fp, false);
 
 	ObjectLock olock(m_Globals);
-	BOOST_FOREACH(const Dictionary::Pair& kv, m_Globals) {
+	for (const Dictionary::Pair& kv : m_Globals) {
 		Dictionary::Ptr persistentVariable = new Dictionary();
 
 		persistentVariable->Set("name", kv.first);
