@@ -27,7 +27,6 @@
 #include "base/configtype.hpp"
 #include "base/application.hpp"
 #include "base/dependencygraph.hpp"
-#include <boost/foreach.hpp>
 #include <boost/regex.hpp>
 #include <algorithm>
 #include <set>
@@ -37,34 +36,36 @@
 
 using namespace icinga;
 
-REGISTER_SAFE_SCRIPTFUNCTION(regex, &ScriptUtils::Regex);
-REGISTER_SAFE_SCRIPTFUNCTION(match, &Utility::Match);
-REGISTER_SAFE_SCRIPTFUNCTION(cidr_match, &Utility::CidrMatch);
-REGISTER_SAFE_SCRIPTFUNCTION(len, &ScriptUtils::Len);
-REGISTER_SAFE_SCRIPTFUNCTION(union, &ScriptUtils::Union);
-REGISTER_SAFE_SCRIPTFUNCTION(intersection, &ScriptUtils::Intersection);
-REGISTER_SCRIPTFUNCTION(log, &ScriptUtils::Log);
-REGISTER_SCRIPTFUNCTION(range, &ScriptUtils::Range);
-REGISTER_SCRIPTFUNCTION(exit, &Application::Exit);
-REGISTER_SAFE_SCRIPTFUNCTION(typeof, &ScriptUtils::TypeOf);
-REGISTER_SAFE_SCRIPTFUNCTION(keys, &ScriptUtils::Keys);
-REGISTER_SAFE_SCRIPTFUNCTION(random, &Utility::Random);
-REGISTER_SAFE_SCRIPTFUNCTION(get_object, &ScriptUtils::GetObject);
-REGISTER_SAFE_SCRIPTFUNCTION(get_objects, &ScriptUtils::GetObjects);
-REGISTER_SCRIPTFUNCTION(assert, &ScriptUtils::Assert);
-REGISTER_SAFE_SCRIPTFUNCTION(string, &ScriptUtils::CastString);
-REGISTER_SAFE_SCRIPTFUNCTION(number, &ScriptUtils::CastNumber);
-REGISTER_SAFE_SCRIPTFUNCTION(bool, &ScriptUtils::CastBool);
-REGISTER_SAFE_SCRIPTFUNCTION(get_time, &Utility::GetTime);
-REGISTER_SAFE_SCRIPTFUNCTION(basename, &Utility::BaseName);
-REGISTER_SAFE_SCRIPTFUNCTION(dirname, &Utility::DirName);
-REGISTER_SAFE_SCRIPTFUNCTION(msi_get_component_path, &ScriptUtils::MsiGetComponentPathShim);
-REGISTER_SAFE_SCRIPTFUNCTION(track_parents, &ScriptUtils::TrackParents);
-REGISTER_SAFE_SCRIPTFUNCTION(escape_shell_cmd, &Utility::EscapeShellCmd);
-REGISTER_SAFE_SCRIPTFUNCTION(escape_shell_arg, &Utility::EscapeShellArg);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, regex, &ScriptUtils::Regex);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, match, &Utility::Match);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, cidr_match, &Utility::CidrMatch);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, len, &ScriptUtils::Len);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, union, &ScriptUtils::Union);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, intersection, &ScriptUtils::Intersection);
+REGISTER_SCRIPTFUNCTION_NS(System, log, &ScriptUtils::Log);
+REGISTER_SCRIPTFUNCTION_NS(System, range, &ScriptUtils::Range);
+REGISTER_SCRIPTFUNCTION_NS(System, exit, &Application::Exit);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, typeof, &ScriptUtils::TypeOf);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, keys, &ScriptUtils::Keys);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, random, &Utility::Random);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, get_object, &ScriptUtils::GetObject);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, get_objects, &ScriptUtils::GetObjects);
+REGISTER_SCRIPTFUNCTION_NS(System, assert, &ScriptUtils::Assert);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, string, &ScriptUtils::CastString);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, number, &ScriptUtils::CastNumber);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, bool, &ScriptUtils::CastBool);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, get_time, &Utility::GetTime);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, basename, &Utility::BaseName);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, dirname, &Utility::DirName);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, msi_get_component_path, &ScriptUtils::MsiGetComponentPathShim);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, track_parents, &ScriptUtils::TrackParents);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, escape_shell_cmd, &Utility::EscapeShellCmd);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, escape_shell_arg, &Utility::EscapeShellArg);
 #ifdef _WIN32
-REGISTER_SAFE_SCRIPTFUNCTION(escape_create_process_arg, &Utility::EscapeCreateProcessArg);
+REGISTER_SAFE_SCRIPTFUNCTION_NS(System, escape_create_process_arg, &Utility::EscapeCreateProcessArg);
 #endif /* _WIN32 */
+REGISTER_SCRIPTFUNCTION_NS(System, ptr, &ScriptUtils::Ptr);
+REGISTER_SCRIPTFUNCTION_NS(System, sleep, &Utility::Sleep);
 
 String ScriptUtils::CastString(const Value& value)
 {
@@ -113,19 +114,19 @@ Array::Ptr ScriptUtils::Union(const std::vector<Value>& arguments)
 {
 	std::set<Value> values;
 
-	BOOST_FOREACH(const Value& varr, arguments) {
+	for (const Value& varr : arguments) {
 		Array::Ptr arr = varr;
 
 		if (arr) {
 			ObjectLock olock(arr);
-			BOOST_FOREACH(const Value& value, arr) {
+			for (const Value& value : arr) {
 				values.insert(value);
 			}
 		}
 	}
 
 	Array::Ptr result = new Array();
-	BOOST_FOREACH(const Value& value, values) {
+	for (const Value& value : values) {
 		result->Add(value);
 	}
 
@@ -250,7 +251,7 @@ Array::Ptr ScriptUtils::Keys(const Dictionary::Ptr& dict)
 
 	if (dict) {
 		ObjectLock olock(dict);
-		BOOST_FOREACH(const Dictionary::Pair& kv, dict) {
+		for (const Dictionary::Pair& kv : dict) {
 			result->Add(kv.first);
 		}
 	}
@@ -260,31 +261,34 @@ Array::Ptr ScriptUtils::Keys(const Dictionary::Ptr& dict)
 
 ConfigObject::Ptr ScriptUtils::GetObject(const Value& vtype, const String& name)
 {
-	String typeName;
+	Type::Ptr ptype;
 
 	if (vtype.IsObjectType<Type>())
-		typeName = static_cast<Type::Ptr>(vtype)->GetName();
+		ptype = vtype;
 	else
-		typeName = vtype;
+		ptype = Type::GetByName(vtype);
 
-	ConfigType::Ptr dtype = ConfigType::GetByName(typeName);
+	ConfigType *ctype = dynamic_cast<ConfigType *>(ptype.get());
 
-	if (!dtype)
+	if (!ctype)
 		return ConfigObject::Ptr();
 
-	return dtype->GetObject(name);
+	return ctype->GetObject(name);
 }
 
 Array::Ptr ScriptUtils::GetObjects(const Type::Ptr& type)
 {
-	ConfigType::Ptr dtype = ConfigType::GetByName(type->GetName());
+	if (!type)
+		BOOST_THROW_EXCEPTION(std::invalid_argument("Invalid type: Must not be null"));
 
-	if (!dtype)
-		BOOST_THROW_EXCEPTION(std::invalid_argument("Invalid type name"));
+	ConfigType *ctype = dynamic_cast<ConfigType *>(type.get());
+
+	if (!ctype)
+		BOOST_THROW_EXCEPTION(std::invalid_argument("Invalid type: Type must inherit from 'ConfigObject'"));
 
 	Array::Ptr result = new Array();
 
-	BOOST_FOREACH(const ConfigObject::Ptr& object, dtype->GetObjects())
+	for (const ConfigObject::Ptr& object : ctype->GetObjects())
 		result->Add(object);
 
 	return result;
@@ -317,3 +321,7 @@ Array::Ptr ScriptUtils::TrackParents(const Object::Ptr& child)
 	return Array::FromVector(DependencyGraph::GetParents(child));
 }
 
+double ScriptUtils::Ptr(const Object::Ptr& object)
+{
+	return reinterpret_cast<intptr_t>(object.get());
+}
