@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -24,7 +24,6 @@
 #include "base/scriptframe.hpp"
 #include "base/exception.hpp"
 #include <boost/algorithm/string.hpp>
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -80,13 +79,13 @@ static Array::Ptr StringSplit(const String& delims)
 	boost::algorithm::split(tokens, self, boost::is_any_of(delims));
 
 	Array::Ptr result = new Array();
-	BOOST_FOREACH(const String& token, tokens) {
+	for (const String& token : tokens) {
 		result->Add(token);
 	}
 	return result;
 }
 
-static Value StringFind(const std::vector<Value>& args)
+static int StringFind(const std::vector<Value>& args)
 {
 	ScriptFrame *vframe = ScriptFrame::GetCurrentFrame();
 	String self = vframe->Self;
@@ -133,22 +132,30 @@ static String StringReverse(void)
 	return self.Reverse();
 }
 
+static String StringTrim(void)
+{
+	ScriptFrame *vframe = ScriptFrame::GetCurrentFrame();
+	String self = vframe->Self;
+	return self.Trim();
+}
+
 Object::Ptr String::GetPrototype(void)
 {
 	static Dictionary::Ptr prototype;
 
 	if (!prototype) {
 		prototype = new Dictionary();
-		prototype->Set("len", new Function(WrapFunction(StringLen), true));
-		prototype->Set("to_string", new Function(WrapFunction(StringToString), true));
-		prototype->Set("substr", new Function(WrapFunction(StringSubstr), true));
-		prototype->Set("upper", new Function(WrapFunction(StringUpper), true));
-		prototype->Set("lower", new Function(WrapFunction(StringLower), true));
-		prototype->Set("split", new Function(WrapFunction(StringSplit), true));
-		prototype->Set("find", new Function(WrapFunction(StringFind), true));
-		prototype->Set("contains", new Function(WrapFunction(StringContains), true));
-		prototype->Set("replace", new Function(WrapFunction(StringReplace), true));
-		prototype->Set("reverse", new Function(WrapFunction(StringReverse), true));
+		prototype->Set("len", new Function("String#len", WrapFunction(StringLen), true));
+		prototype->Set("to_string", new Function("String#to_string", WrapFunction(StringToString), true));
+		prototype->Set("substr", new Function("String#substr", WrapFunction(StringSubstr), true));
+		prototype->Set("upper", new Function("String#upper", WrapFunction(StringUpper), true));
+		prototype->Set("lower", new Function("String#lower", WrapFunction(StringLower), true));
+		prototype->Set("split", new Function("String#split", WrapFunction(StringSplit), true));
+		prototype->Set("find", new Function("String#find", WrapFunction(StringFind), true));
+		prototype->Set("contains", new Function("String#contains", WrapFunction(StringContains), true));
+		prototype->Set("replace", new Function("String#replace", WrapFunction(StringReplace), true));
+		prototype->Set("reverse", new Function("String#reverse", WrapFunction(StringReverse), true));
+		prototype->Set("trim", new Function("String#trim", WrapFunction(StringTrim), true));
 	}
 
 	return prototype;
