@@ -86,7 +86,7 @@ public:
 
 	String GetPluralName(void) const;
 
-	Object::Ptr Instantiate(void) const;
+	Object::Ptr Instantiate(const std::vector<Value>& args) const;
 
 	bool IsAssignableFrom(const Type::Ptr& other) const;
 
@@ -97,6 +97,7 @@ public:
 
 	static void Register(const Type::Ptr& type);
 	static Type::Ptr GetByName(const String& name);
+	static std::vector<Type::Ptr> GetAllTypes(void);
 
 	virtual void SetField(int id, const Value& value, bool suppress_events = false, const Value& cookie = Empty) override;
 	virtual Value GetField(int id) const override;
@@ -132,35 +133,25 @@ protected:
 };
 
 template<typename T>
-class TypeImpl
+class I2_BASE_API TypeImpl
 {
 };
 
 #define REGISTER_TYPE(type) \
-	namespace { namespace UNIQUE_NAME(rt) { \
-		void RegisterType ## type(void) \
-		{ \
-			icinga::Type::Ptr t = new TypeImpl<type>(); \
-			type::TypeInstance = t; \
-			icinga::Type::Register(t); \
-		} \
-		\
-		INITIALIZE_ONCE_WITH_PRIORITY(RegisterType ## type, 10); \
-	} } \
+	INITIALIZE_ONCE_WITH_PRIORITY([]() { \
+		icinga::Type::Ptr t = new TypeImpl<type>(); \
+		type::TypeInstance = t; \
+		icinga::Type::Register(t); \
+	}, 10); \
 	DEFINE_TYPE_INSTANCE(type)
 
 #define REGISTER_TYPE_WITH_PROTOTYPE(type, prototype) \
-	namespace { namespace UNIQUE_NAME(rt) { \
-		void RegisterType ## type(void) \
-		{ \
-			icinga::Type::Ptr t = new TypeImpl<type>(); \
-			t->SetPrototype(prototype); \
-			type::TypeInstance = t; \
-			icinga::Type::Register(t); \
-		} \
-		\
-		INITIALIZE_ONCE_WITH_PRIORITY(RegisterType ## type, 10); \
-	} } \
+	INITIALIZE_ONCE_WITH_PRIORITY([]() { \
+		icinga::Type::Ptr t = new TypeImpl<type>(); \
+		t->SetPrototype(prototype); \
+		type::TypeInstance = t; \
+		icinga::Type::Register(t); \
+	}, 10); \
 	DEFINE_TYPE_INSTANCE(type)
 
 #define DEFINE_TYPE_INSTANCE(type) \
