@@ -28,7 +28,6 @@
 #include "base/objectlock.hpp"
 #include <mmatch.h>
 #include <boost/lexical_cast.hpp>
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/classification.hpp>
 #include <boost/algorithm/string/trim.hpp>
@@ -64,6 +63,10 @@ using namespace icinga;
 boost::thread_specific_ptr<String> Utility::m_ThreadName;
 boost::thread_specific_ptr<unsigned int> Utility::m_RandSeed;
 
+#ifdef I2_DEBUG
+double Utility::m_DebugTime = -1;
+#endif /* I2_DEBUG */
+
 /**
  * Demangles a symbol name.
  *
@@ -260,7 +263,7 @@ String Utility::DirName(const String& path)
 	String dupPath = path;
 
 	/* PathRemoveFileSpec doesn't properly handle forward slashes. */
-	BOOST_FOREACH(char& ch, dupPath) {
+	for (char& ch : dupPath) {
 		if (ch == '/')
 			ch = '\\';
 	}
@@ -332,6 +335,29 @@ void Utility::NullDeleter(void *)
 	/* Nothing to do here. */
 }
 
+#ifdef I2_DEBUG
+/**
+ * (DEBUG / TESTING ONLY) Sets the current system time to a static value,
+ * that will be be retrieved by any component of Icinga, when using GetTime().
+ *
+ * This should be only used for testing purposes, e.g. unit tests and debugging of certain functionalities.
+ */
+void Utility::SetTime(double time)
+{
+	m_DebugTime = time;
+}
+
+/**
+ * (DEBUG / TESTING ONLY) Increases the set debug system time by X seconds.
+ *
+ * This should be only used for testing purposes, e.g. unit tests and debugging of certain functionalities.
+ */
+void Utility::IncrementTime(double diff)
+{
+	m_DebugTime += diff;
+}
+#endif /* I2_DEBUG */
+
 /**
  * Returns the current UNIX timestamp including fractions of seconds.
  *
@@ -339,6 +365,12 @@ void Utility::NullDeleter(void *)
  */
 double Utility::GetTime(void)
 {
+#ifdef I2_DEBUG
+	if (m_DebugTime >= 0) {
+		// (DEBUG / TESTING ONLY) this will return a *STATIC* system time, if the value has been set!
+		return m_DebugTime;
+	}
+#endif /* I2_DEBUG */
 #ifdef _WIN32
 	FILETIME cft;
 	GetSystemTimeAsFileTime(&cft);
@@ -388,7 +420,11 @@ pid_t Utility::GetPid(void)
 void Utility::Sleep(double timeout)
 {
 #ifndef _WIN32
-	usleep(timeout * 1000 * 1000);
+	unsigned long micros = timeout * 1000000u;
+	if (timeout >= 1.0)
+		sleep((unsigned)timeout);
+
+	usleep(micros % 1000000u);
 #else /* _WIN32 */
 	::Sleep(timeout * 1000);
 #endif /* _WIN32 */
@@ -506,7 +542,7 @@ bool Utility::Glob(const String& pathSpec, const boost::function<void (const Str
 			if (!GlobHelper(part1, GlobDirectory, files2, dirs2))
 				return false;
 
-			BOOST_FOREACH(const String& dir, dirs2) {
+			for (const String& dir : dirs2) {
 				if (!Utility::Glob(dir + "/" + part2, callback, type))
 					return false;
 			}
@@ -558,12 +594,12 @@ bool Utility::Glob(const String& pathSpec, const boost::function<void (const Str
 #endif /* _WIN32 */
 
 	std::sort(files.begin(), files.end());
-	BOOST_FOREACH(const String& cpath, files) {
+	for (const String& cpath : files) {
 		callback(cpath);
 	}
 
 	std::sort(dirs.begin(), dirs.end());
-	BOOST_FOREACH(const String& cpath, dirs) {
+	for (const String& cpath : dirs) {
 		callback(cpath);
 	}
 
@@ -684,17 +720,17 @@ bool Utility::GlobRecursive(const String& path, const String& pattern, const boo
 #endif /* _WIN32 */
 
 	std::sort(files.begin(), files.end());
-	BOOST_FOREACH(const String& cpath, files) {
+	for (const String& cpath : files) {
 		callback(cpath);
 	}
 
 	std::sort(dirs.begin(), dirs.end());
-	BOOST_FOREACH(const String& cpath, dirs) {
+	for (const String& cpath : dirs) {
 		callback(cpath);
 	}
 
 	std::sort(alldirs.begin(), alldirs.end());
-	BOOST_FOREACH(const String& cpath, alldirs) {
+	for (const String& cpath : alldirs) {
 		GlobRecursive(cpath, pattern, callback, type);
 	}
 
@@ -745,7 +781,7 @@ void Utility::RemoveDirRecursive(const String& path)
 	   first before recursing into subdirectories. */
 	std::reverse(paths.begin(), paths.end());
 
-	BOOST_FOREACH(const String& path, paths) {
+	for (const String& path : paths) {
 		if (remove(path.CStr()) < 0)
 			BOOST_THROW_EXCEPTION(posix_error()
 			    << boost::errinfo_api_function("remove")
@@ -902,23 +938,26 @@ String Utility::NaturalJoin(const std::vector<String>& tokens)
 	return result;
 }
 
-String Utility::Join(const Array::Ptr& tokens, char separator)
+String Utility::Join(const Array::Ptr& tokens, char separator, bool escapeSeparator)
 {
 	String result;
 	bool first = true;
 
 	ObjectLock olock(tokens);
-	BOOST_FOREACH(const Value& vtoken, tokens) {
+	for (const Value& vtoken : tokens) {
 		String token = Convert::ToString(vtoken);
-		boost::algorithm::replace_all(token, "\\", "\\\\");
 
-		char sep_before[2], sep_after[3];
-		sep_before[0] = separator;
-		sep_before[1] = '\0';
-		sep_after[0] = '\\';
-		sep_after[1] = separator;
-		sep_after[2] = '\0';
-		boost::algorithm::replace_all(token, sep_before, sep_after);
+		if (escapeSeparator) {
+			boost::algorithm::replace_all(token, "\\", "\\\\");
+
+			char sep_before[2], sep_after[3];
+			sep_before[0] = separator;
+			sep_before[1] = '\0';
+			sep_after[0] = '\\';
+			sep_after[1] = separator;
+			sep_after[2] = '\0';
+			boost::algorithm::replace_all(token, sep_before, sep_after);
+		}
 
 		if (first)
 			first = false;
@@ -1031,7 +1070,7 @@ String Utility::EscapeShellCmd(const String& s)
 	size_t prev_quote = String::NPos;
 	int index = -1;
 
-	BOOST_FOREACH(char ch, s) {
+	for (char ch : s) {
 		bool escape = false;
 
 		index++;
@@ -1081,7 +1120,7 @@ String Utility::EscapeShellArg(const String& s)
 	result = "'";
 #endif /* _WIN32 */
 
-	BOOST_FOREACH(char ch, s) {
+	for (char ch : s) {
 #ifdef _WIN32
 		if (ch == '"' || ch == '%') {
 			result += ' ';
@@ -1196,7 +1235,7 @@ unsigned long Utility::SDBM(const String& str, size_t len)
 	unsigned long hash = 0;
 	size_t current = 0;
 
-	BOOST_FOREACH(char c, str) {
+	for (char c : str) {
 		if (current >= len)
 			break;
 
@@ -1384,7 +1423,7 @@ String Utility::EscapeString(const String& s, const String& chars, const bool il
 {
 	std::ostringstream result;
 	if (illegal) {
-		BOOST_FOREACH(char ch, s) {
+		for (char ch : s) {
 			if (chars.FindFirstOf(ch) != String::NPos || ch == '%') {
 				result << '%';
 				HexEncode(ch, result);
@@ -1392,7 +1431,7 @@ String Utility::EscapeString(const String& s, const String& chars, const bool il
 				result << ch;
 		}
 	} else {
-		BOOST_FOREACH(char ch, s) {
+		for (char ch : s) {
 			if (chars.FindFirstOf(ch) == String::NPos || ch == '%') {
 				result << '%';
 				HexEncode(ch, result);
