@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -28,6 +28,8 @@
 namespace icinga
 {
 
+typedef double Timestamp;
+
 /**
  * The type of a Value.
  *
@@ -69,6 +71,14 @@ public:
 		: m_Value(double(value))
 	{ }
 
+	inline Value(long long value)
+		: m_Value(double(value))
+	{ }
+
+	inline Value(unsigned long long value)
+		: m_Value(double(value))
+	{ }
+
 	inline Value(double value)
 		: m_Value(value)
 	{ }
@@ -81,14 +91,27 @@ public:
 		: m_Value(value)
 	{ }
 
+	inline Value(String&& value)
+		: m_Value(value)
+	{ }
+
 	inline Value(const char *value)
 		: m_Value(String(value))
 	{ }
 
-	inline Value(const Value& other)
+	Value(const Value& other)
 		: m_Value(other.m_Value)
 	{ }
 
+	Value(Value&& other)
+	{
+#if BOOST_VERSION >= 105400
+		m_Value = std::move(other.m_Value);
+#else /* BOOST_VERSION */
+		m_Value.swap(other.m_Value);
+#endif /* BOOST_VERSION */
+	}
+
 	inline Value(Object *value)
 	{
 		if (!value)
@@ -111,7 +134,22 @@ public:
 	operator double(void) const;
 	operator String(void) const;
 
-	Value& operator=(const Value& other);
+	Value& operator=(const Value& other)
+	{
+		m_Value = other.m_Value;
+		return *this;
+	}
+
+	Value& operator=(Value&& other)
+	{
+#if BOOST_VERSION >= 105400
+		m_Value = std::move(other.m_Value);
+#else /* BOOST_VERSION */
+		m_Value.swap(other.m_Value);
+#endif /* BOOST_VERSION */
+
+		return *this;
+	}
 
 	bool operator==(bool rhs) const;
 	bool operator!=(bool rhs) const;
@@ -134,13 +172,13 @@ public:
 	template<typename T>
 	operator intrusive_ptr<T>(void) const
 	{
-		if (IsEmpty())
+		if (IsEmpty() && !IsString())
 			return intrusive_ptr<T>();
 
 		if (!IsObject())
 			BOOST_THROW_EXCEPTION(std::runtime_error("Cannot convert value of type '" + GetTypeName() + "' to an object."));
 
-		Object::Ptr object = boost::get<Object::Ptr>(m_Value);
+		const Object::Ptr& object = boost::get<Object::Ptr>(m_Value);
 
 		ASSERT(object);
 
@@ -218,7 +256,7 @@ public:
 		if (!IsObject())
 			return false;
 
-		return (dynamic_pointer_cast<T>(boost::get<Object::Ptr>(m_Value)) != NULL);
+		return (dynamic_cast<T *>(boost::get<Object::Ptr>(m_Value).get()) != NULL);
 	}
 
 	/**
@@ -226,25 +264,30 @@ public:
 	*
 	* @returns The type.
 	*/
-	ValueType GetType(void) const
+	inline ValueType GetType(void) const
 	{
 		return static_cast<ValueType>(m_Value.which());
 	}
 
+	inline void Swap(Value& other)
+	{
+		m_Value.swap(other.m_Value);
+	}
+
 	String GetTypeName(void) const;
 
 	Type::Ptr GetReflectionType(void) const;
 	
 	Value Clone(void) const;
 
-private:
-	boost::variant<boost::blank, double, bool, String, Object::Ptr> m_Value;
-
 	template<typename T>
 	const T& Get(void) const
 	{
 		return boost::get<T>(m_Value);
 	}
+
+private:
+	boost::variant<boost::blank, double, bool, String, Object::Ptr> m_Value;
 };
 
 extern I2_BASE_API Value Empty;
