@@ -46,8 +46,8 @@ struct Task
 	    : Priority(PriorityNormal), ID(-1)
 	{ }
 
-	Task(const boost::function<void (void)>& function, WorkQueuePriority priority, int id)
-	    : Function(function), Priority(priority), ID(id)
+	Task(boost::function<void (void)>&& function, WorkQueuePriority priority, int id)
+	    : Function(std::move(function)), Priority(priority), ID(id)
 	{ }
 
 	boost::function<void (void)> Function;
@@ -83,7 +83,10 @@ public:
 	WorkQueue(size_t maxItems = 0, int threadCount = 1);
 	~WorkQueue(void);
 
-	void Enqueue(const boost::function<void (void)>& function, WorkQueuePriority priority = PriorityNormal,
+	void SetName(const String& name);
+	String GetName(void) const;
+
+	void Enqueue(boost::function<void (void)>&& function, WorkQueuePriority priority = PriorityNormal,
 	    bool allowInterleaved = false);
 	void Join(bool stop = false);
 
@@ -99,6 +102,7 @@ public:
 
 private:
 	int m_ID;
+	String m_Name;
 	static int m_NextID;
 	int m_ThreadCount;
 	bool m_Spawned;
