@@ -30,7 +30,6 @@
 #include "base/exception.hpp"
 #include "base/convert.hpp"
 #include "base/statsfunction.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -42,7 +41,7 @@ void CheckerComponent::StatsFunc(const Dictionary::Ptr& status, const Array::Ptr
 {
 	Dictionary::Ptr nodes = new Dictionary();
 
-	BOOST_FOREACH(const CheckerComponent::Ptr& checker, ConfigType::GetObjectsByType<CheckerComponent>()) {
+	for (const CheckerComponent::Ptr& checker : ConfigType::GetObjectsByType<CheckerComponent>()) {
 		unsigned long idle = checker->GetIdleCheckables();
 		unsigned long pending = checker->GetPendingCheckables();
 
@@ -116,10 +115,10 @@ void CheckerComponent::CheckThreadProc(void)
 		if (m_Stopped)
 			break;
 
-		CheckTimeView::iterator it = idx.begin();
-		Checkable::Ptr checkable = *it;
+		auto it = idx.begin();
+		CheckableScheduleInfo csi = *it;
 
-		double wait = checkable->GetNextCheck() - Utility::GetTime();
+		double wait = csi.NextCheck - Utility::GetTime();
 
 		if (Checkable::GetPendingChecks() >= GetConcurrentChecks())
 			wait = 0.5;
@@ -131,6 +130,8 @@ void CheckerComponent::CheckThreadProc(void)
 			continue;
 		}
 
+		Checkable::Ptr checkable = csi.Object;
+
 		m_IdleCheckables.erase(checkable);
 
 		bool forced = checkable->GetForceNextCheck();
@@ -162,14 +163,15 @@ void CheckerComponent::CheckThreadProc(void)
 
 			if (tp && !tp->IsInside(Utility::GetTime())) {
 				Log(LogNotice, "CheckerComponent")
-				    << "Skipping check for object '" << checkable->GetName() << "': not in check_period";
+				    << "Skipping check for object '" << checkable->GetName()
+				    << "': not in check period '" << tp->GetName() << "'";
 				check = false;
 			}
 		}
 
 		/* reschedule the checkable if checks are disabled */
 		if (!check) {
-			m_IdleCheckables.insert(checkable);
+			m_IdleCheckables.insert(GetCheckableScheduleInfo(checkable));
 			lock.unlock();
 
 			checkable->UpdateNextCheck();
@@ -179,7 +181,7 @@ void CheckerComponent::CheckThreadProc(void)
 			continue;
 		}
 
-		m_PendingCheckables.insert(checkable);
+		m_PendingCheckables.insert(GetCheckableScheduleInfo(checkable));
 
 		lock.unlock();
 
@@ -191,6 +193,8 @@ void CheckerComponent::CheckThreadProc(void)
 		Log(LogDebug, "CheckerComponent")
 		    << "Executing check for '" << checkable->GetName() << "'";
 
+		Checkable::IncreasePendingChecks();
+
 		Utility::QueueAsyncCallback(boost::bind(&CheckerComponent::ExecuteCheckHelper, CheckerComponent::Ptr(this), checkable));
 
 		lock.lock();
@@ -219,19 +223,21 @@ void CheckerComponent::ExecuteCheckHelper(const Checkable::Ptr& checkable)
 		Log(LogCritical, "checker", output);
 	}
 
+	Checkable::DecreasePendingChecks();
+
 	{
 		boost::mutex::scoped_lock lock(m_Mutex);
 
 		/* remove the object from the list of pending objects; if it's not in the
 		 * list this was a manual (i.e. forced) check and we must not re-add the
 		 * object to the list because it's already there. */
-		CheckerComponent::CheckableSet::iterator it;
-		it = m_PendingCheckables.find(checkable);
+		auto it = m_PendingCheckables.find(checkable);
+
 		if (it != m_PendingCheckables.end()) {
 			m_PendingCheckables.erase(it);
 
 			if (checkable->IsActive())
-				m_IdleCheckables.insert(checkable);
+				m_IdleCheckables.insert(GetCheckableScheduleInfo(checkable));
 
 			m_CV.notify_all();
 		}
@@ -249,7 +255,7 @@ void CheckerComponent::ResultTimerHandler(void)
 		boost::mutex::scoped_lock lock(m_Mutex);
 
 		msgbuf << "Pending checkables: " << m_PendingCheckables.size() << "; Idle checkables: " << m_IdleCheckables.size() << "; Checks/s: "
-		    << (CIB::GetActiveHostChecksStatistics(5) + CIB::GetActiveServiceChecksStatistics(5)) / 5.0;
+		    << (CIB::GetActiveHostChecksStatistics(60) + CIB::GetActiveServiceChecksStatistics(60)) / 60.0;
 	}
 
 	Log(LogNotice, "CheckerComponent", msgbuf.str());
@@ -272,7 +278,7 @@ void CheckerComponent::ObjectHandler(const ConfigObject::Ptr& object)
 			if (m_PendingCheckables.find(checkable) != m_PendingCheckables.end())
 				return;
 
-			m_IdleCheckables.insert(checkable);
+			m_IdleCheckables.insert(GetCheckableScheduleInfo(checkable));
 		} else {
 			m_IdleCheckables.erase(checkable);
 			m_PendingCheckables.erase(checkable);
@@ -282,6 +288,14 @@ void CheckerComponent::ObjectHandler(const ConfigObject::Ptr& object)
 	}
 }
 
+CheckableScheduleInfo CheckerComponent::GetCheckableScheduleInfo(const Checkable::Ptr& checkable)
+{
+	CheckableScheduleInfo csi;
+	csi.Object = checkable;
+	csi.NextCheck = checkable->GetNextCheck();
+	return csi;
+}
+
 void CheckerComponent::NextCheckChangedHandler(const Checkable::Ptr& checkable)
 {
 	boost::mutex::scoped_lock lock(m_Mutex);
@@ -290,12 +304,16 @@ void CheckerComponent::NextCheckChangedHandler(const Checkable::Ptr& checkable)
 	typedef boost::multi_index::nth_index<CheckableSet, 0>::type CheckableView;
 	CheckableView& idx = boost::get<0>(m_IdleCheckables);
 
-	CheckableView::iterator it = idx.find(checkable);
+	auto it = idx.find(checkable);
+
 	if (it == idx.end())
 		return;
 
 	idx.erase(checkable);
-	idx.insert(checkable);
+
+	CheckableScheduleInfo csi = GetCheckableScheduleInfo(checkable);
+	idx.insert(csi);
+
 	m_CV.notify_all();
 }
 
