@@ -27,7 +27,6 @@
 #include "base/tlsutility.hpp"
 #include "base/scriptglobal.hpp"
 #include "base/exception.hpp"
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/join.hpp>
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/algorithm/string/case_conv.hpp>
@@ -120,7 +119,6 @@ bool ApiSetupUtility::SetupMasterCertificates(const String& cn)
 	String ca_path = PkiUtility::GetLocalCaPath();
 	String ca = ca_path + "/ca.crt";
 	String ca_key = ca_path + "/ca.key";
-	String serial = ca_path + "/serial.txt";
 	String target_ca = pki_path + "/ca.crt";
 
 	Log(LogInformation, "cli")
@@ -137,13 +135,12 @@ bool ApiSetupUtility::SetupMasterCertificates(const String& cn)
 	files.push_back(ca_path);
 	files.push_back(ca);
 	files.push_back(ca_key);
-	files.push_back(serial);
 	files.push_back(target_ca);
 	files.push_back(key);
 	files.push_back(csr);
 	files.push_back(cert);
 
-	BOOST_FOREACH(const String& file, files) {
+	for (const String& file : files) {
 		if (!Utility::SetFileOwnership(file, user, group)) {
 			Log(LogWarning, "cli")
 			    << "Cannot set ownership for user '" << user << "' group '" << group << "' on file '" << file << "'.";
