@@ -48,6 +48,40 @@ REGISTER_CLICOMMAND("console", ConsoleCommand);
 
 INITIALIZE_ONCE(&ConsoleCommand::StaticInitialize);
 
+extern "C" void dbg_spawn_console(void)
+{
+	ScriptFrame frame;
+	ConsoleCommand::RunScriptConsole(frame);
+}
+
+extern "C" void dbg_inspect_value(const Value& value)
+{
+	ConfigWriter::EmitValue(std::cout, 1, Serialize(value, 0));
+	std::cout << std::endl;
+}
+
+extern "C" void dbg_inspect_object(Object *obj)
+{
+	Object::Ptr objr = obj;
+	dbg_inspect_value(objr);
+}
+
+extern "C" void dbg_eval(const char *text)
+{
+	Expression *expr = NULL;
+
+	try {
+		ScriptFrame frame;
+		expr = ConfigCompiler::CompileText("<dbg>", text);
+		Value result = Serialize(expr->Evaluate(frame), 0);
+		dbg_inspect_value(result);
+	} catch (const std::exception& ex) {
+		std::cout << "Error: " << DiagnosticInformation(ex) << "\n";
+	}
+
+	delete expr;
+}
+
 void ConsoleCommand::BreakpointHandler(ScriptFrame& frame, ScriptError *ex, const DebugInfo& di)
 {
 	static boost::mutex mutex;
@@ -104,6 +138,8 @@ void ConsoleCommand::InitParameters(boost::program_options::options_description&
 	visibleDesc.add_options()
 		("connect,c", po::value<std::string>(), "connect to an Icinga 2 instance")
 		("eval,e", po::value<std::string>(), "evaluate expression and terminate")
+		("file,r", po::value<std::string>(), "evaluate a file and terminate")
+		("syntax-only", "only validate syntax (requires --eval or --file)")
 		("sandbox", "enable sandbox mode")
 	;
 }
@@ -141,7 +177,7 @@ char *ConsoleCommand::ConsoleCompleteHelper(const char *word, int state)
 		}
 	}
 
-	if (state >= matches.size())
+	if (state >= static_cast<int>(matches.size()))
 		return NULL;
 
 	return strdup(matches[state].CStr());
@@ -170,7 +206,7 @@ int ConsoleCommand::Run(const po::variables_map& vm, const std::vector<std::stri
 
 	scriptFrame.Self = scriptFrame.Locals;
 
-	if (!vm.count("eval"))
+	if (!vm.count("eval") && !vm.count("file"))
 		std::cout << "Icinga 2 (version: " << Application::GetAppVersion() << ")\n";
 
 	const char *addrEnv = getenv("ICINGA2_API_URL");
@@ -181,18 +217,54 @@ int ConsoleCommand::Run(const po::variables_map& vm, const std::vector<std::stri
 		addr = vm["connect"].as<std::string>();
 
 	String command;
+	bool syntaxOnly = false;
+
+	if (vm.count("syntax-only")) {
+		if (vm.count("eval") || vm.count("file"))
+			syntaxOnly = true;
+		else {
+			std::cerr << "The option --syntax-only can only be used in combination with --eval or --file." << std::endl;
+			return EXIT_FAILURE;
+		}
+	}
 
 	if (vm.count("eval"))
 		command = vm["eval"].as<std::string>();
+	else if (vm.count("file")) {
+		std::string fname = vm["file"].as<std::string>();
 
-	return RunScriptConsole(scriptFrame, addr, session, command);;
+		try {
+			std::ifstream fp(fname.c_str());
+			fp.exceptions(std::ifstream::failbit | std::ifstream::badbit);
+			command = String(std::istreambuf_iterator<char>(fp), std::istreambuf_iterator<char>());
+		} catch (const std::exception&) {
+			std::cerr << "Could not read file '" << fname << "'." << std::endl;
+			return EXIT_FAILURE;
+		}
+	}
+
+	return RunScriptConsole(scriptFrame, addr, session, command, syntaxOnly);
 }
 
-int ConsoleCommand::RunScriptConsole(ScriptFrame& scriptFrame, const String& addr, const String& session, const String& commandOnce)
+int ConsoleCommand::RunScriptConsole(ScriptFrame& scriptFrame, const String& addr, const String& session, const String& commandOnce, bool syntaxOnly)
 {
 	std::map<String, String> lines;
 	int next_line = 1;
 
+#ifdef HAVE_EDITLINE
+	String homeEnv = getenv("HOME");
+	String historyPath = homeEnv + "/.icinga2_history";
+
+	std::fstream historyfp;
+	historyfp.open(historyPath.CStr(), std::fstream::in);
+
+	String line;
+	while (std::getline(historyfp, line.GetData()))
+		add_history(line.CStr());
+
+	historyfp.close();
+#endif /* HAVE_EDITLINE */
+
 	l_ScriptFrame = &scriptFrame;
 	l_Session = session;
 
@@ -254,7 +326,13 @@ incomplete:
 			if (!cline)
 				break;
 
-			add_history(cline);
+			if (commandOnce.IsEmpty() && cline[0] != '\0') {
+				add_history(cline);
+
+				historyfp.open(historyPath.CStr(), std::fstream::out | std::fstream::app);
+				historyfp << cline << "\n";
+				historyfp.close();
+			}
 
 			line = cline;
 
@@ -287,7 +365,15 @@ incomplete:
 
 			if (!l_ApiClient) {
 				expr = ConfigCompiler::CompileText(fileName, command);
-				result = Serialize(expr->Evaluate(scriptFrame), 0);
+
+				/* This relies on the fact that - for syntax errors - CompileText()
+				 * returns an AST where the top-level expression is a 'throw'. */
+				if (!syntaxOnly || dynamic_cast<ThrowExpression *>(expr)) {
+					if (syntaxOnly)
+						std::cerr << "    => " << command << std::endl;
+					result = Serialize(expr->Evaluate(scriptFrame), 0);
+				} else
+					result = true;
 			} else {
 				boost::mutex mutex;
 				boost::condition_variable cv;
@@ -319,7 +405,7 @@ incomplete:
 				break;
 			}
 		} catch (const ScriptError& ex) {
-			if (ex.IsIncompleteExpression()) {
+			if (ex.IsIncompleteExpression() && commandOnce.IsEmpty()) {
 				continuation = true;
 				goto incomplete;
 			}
