@@ -24,7 +24,6 @@
 #include "config/configcompiler.hpp"
 #include "config/configcompilercontext.hpp"
 #include "config/configitembuilder.hpp"
-#include "remote/configobjectutility.hpp"
 
 
 using namespace icinga;
@@ -106,7 +105,7 @@ bool DaemonUtility::ValidateConfigFiles(const std::vector<std::string>& configs,
 		ConfigCompilerContext::GetInstance()->OpenObjectsFile(objectsFile);
 
 	if (!configs.empty()) {
-		BOOST_FOREACH(const String& configPath, configs) {
+		for (const String& configPath : configs) {
 			Expression *expression = ConfigCompiler::CompileFile(configPath, String(), "_etc");
 			success = ExecuteExpression(expression);
 			delete expression;
@@ -150,6 +149,7 @@ bool DaemonUtility::ValidateConfigFiles(const std::vector<std::string>& configs,
 		ConfigItemBuilder::Ptr builder = new ConfigItemBuilder();
 		builder->SetType(appType);
 		builder->SetName("app");
+		builder->AddExpression(new ImportDefaultTemplatesExpression());
 		ConfigItem::Ptr item = builder->Compile();
 		item->Register();
 	}
@@ -163,17 +163,19 @@ bool DaemonUtility::LoadConfigFiles(const std::vector<std::string>& configs,
 {
 	ActivationScope ascope;
 
-	if (!DaemonUtility::ValidateConfigFiles(configs, objectsFile))
+	if (!DaemonUtility::ValidateConfigFiles(configs, objectsFile)) {
+		ConfigCompilerContext::GetInstance()->CancelObjectsFile();
 		return false;
+	}
 
 	WorkQueue upq(25000, Application::GetConcurrency());
+	upq.SetName("DaemonUtility::LoadConfigFiles");
 	bool result = ConfigItem::CommitItems(ascope.GetContext(), upq, newItems);
 
-	/* Remove ignored Downtime/Comment objects. */
-	ConfigItem::RemoveIgnoredItems(ConfigObjectUtility::GetConfigDir());
-
-	if (!result)
+	if (!result) {
+		ConfigCompilerContext::GetInstance()->CancelObjectsFile();
 		return false;
+	}
 
 	ConfigCompilerContext::GetInstance()->FinishObjectsFile();
 	ScriptGlobal::WriteToFile(varsfile);
