@@ -21,7 +21,6 @@
 #include "cli/nodeutility.hpp"
 #include "base/logger.hpp"
 #include "base/application.hpp"
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/join.hpp>
 #include <boost/algorithm/string/replace.hpp>
 #include <iostream>
@@ -48,6 +47,11 @@ int NodeAddCommand::GetMinArguments(void) const
 	return 1;
 }
 
+bool NodeAddCommand::IsDeprecated(void) const
+{
+	return true;
+}
+
 /**
  * The entry point for the "node add" CLI command.
  *
