@@ -19,7 +19,6 @@
 
 #include "config/applyrule.hpp"
 #include "base/logger.hpp"
-#include <boost/foreach.hpp>
 #include <set>
 
 using namespace icinga;
@@ -113,7 +112,7 @@ bool ApplyRule::IsValidSourceType(const String& sourceType)
 
 bool ApplyRule::IsValidTargetType(const String& sourceType, const String& targetType)
 {
-	TypeMap::const_iterator it = m_Types.find(sourceType);
+	auto it = m_Types.find(sourceType);
 
 	if (it == m_Types.end())
 		return false;
@@ -121,7 +120,7 @@ bool ApplyRule::IsValidTargetType(const String& sourceType, const String& target
 	if (it->second.size() == 1 && targetType == "")
 		return true;
 
-	BOOST_FOREACH(const String& type, it->second) {
+	for (const String& type : it->second) {
 		if (type == targetType)
 			return true;
 	}
@@ -131,7 +130,7 @@ bool ApplyRule::IsValidTargetType(const String& sourceType, const String& target
 
 std::vector<String> ApplyRule::GetTargetTypes(const String& sourceType)
 {
-	TypeMap::const_iterator it = m_Types.find(sourceType);
+	auto it = m_Types.find(sourceType);
 
 	if (it == m_Types.end())
 		return std::vector<String>();
@@ -151,7 +150,7 @@ bool ApplyRule::HasMatches(void) const
 
 std::vector<ApplyRule>& ApplyRule::GetRules(const String& type)
 {
-	RuleMap::iterator it = m_Rules.find(type);
+	auto it = m_Rules.find(type);
 	if (it == m_Rules.end()) {
 		static std::vector<ApplyRule> emptyList;
 		return emptyList;
@@ -161,17 +160,11 @@ std::vector<ApplyRule>& ApplyRule::GetRules(const String& type)
 
 void ApplyRule::CheckMatches(void)
 {
-	BOOST_FOREACH(const RuleMap::value_type& kv, m_Rules) {
-		BOOST_FOREACH(const ApplyRule& rule, kv.second) {
+	for (const RuleMap::value_type& kv : m_Rules) {
+		for (const ApplyRule& rule : kv.second) {
 			if (!rule.HasMatches())
 				Log(LogWarning, "ApplyRule")
 				    << "Apply rule '" << rule.GetName() << "' (" << rule.GetDebugInfo() << ") for type '" << kv.first << "' does not match anywhere!";
 		}
 	}
 }
-
-void ApplyRule::DiscardRules(void)
-{
-	m_Rules.clear();
-}
-
