@@ -43,14 +43,15 @@ public:
 	ConfigItem(const String& type, const String& name, bool abstract,
 	    const boost::shared_ptr<Expression>& exprl,
 	    const boost::shared_ptr<Expression>& filter,
-	    bool ignoreOnError, const DebugInfo& debuginfo,
+	    bool defaultTmpl, bool ignoreOnError, const DebugInfo& debuginfo,
 	    const Dictionary::Ptr& scope, const String& zone,
 	    const String& package);
 
 	String GetType(void) const;
 	String GetName(void) const;
 	bool IsAbstract(void) const;
-	bool GetIgnoreOnError(void) const;
+	bool IsDefaultTemplate(void) const;
+	bool IsIgnoreOnError(void) const;
 
 	std::vector<ConfigItem::Ptr> GetParents(void) const;
 
@@ -68,12 +69,13 @@ public:
 	static ConfigItem::Ptr GetByTypeAndName(const String& type,
 	    const String& name);
 
-	static bool CommitItems(const ActivationContext::Ptr& context, WorkQueue& upq, std::vector<ConfigItem::Ptr>& newItems);
-	static bool ActivateItems(WorkQueue& upq, const std::vector<ConfigItem::Ptr>& newItems, bool runtimeCreated = false);
+	static bool CommitItems(const ActivationContext::Ptr& context, WorkQueue& upq, std::vector<ConfigItem::Ptr>& newItems, bool silent = false);
+	static bool ActivateItems(WorkQueue& upq, const std::vector<ConfigItem::Ptr>& newItems, bool runtimeCreated = false, bool silent = false);
 
 	static bool RunWithActivationContext(const Function::Ptr& function);
 
 	static std::vector<ConfigItem::Ptr> GetItems(const String& type);
+	static std::vector<ConfigItem::Ptr> GetDefaultTemplates(const String& type);
 
 	static void RemoveIgnoredItems(const String& allowedConfigPath);
 
@@ -84,6 +86,7 @@ private:
 
 	boost::shared_ptr<Expression> m_Expression;
 	boost::shared_ptr<Expression> m_Filter;
+	bool m_DefaultTmpl;
 	bool m_IgnoreOnError;
 	DebugInfo m_DebugInfo; /**< Debug information. */
 	Dictionary::Ptr m_Scope; /**< variable scope. */
@@ -98,6 +101,7 @@ private:
 	typedef std::map<String, ConfigItem::Ptr> ItemMap;
 	typedef std::map<String, ItemMap> TypeMap;
 	static TypeMap m_Items; /**< All registered configuration items. */
+	static TypeMap m_DefaultTemplates;
 
 	typedef std::vector<ConfigItem::Ptr> ItemList;
 	static ItemList m_UnnamedItems;
@@ -111,9 +115,6 @@ private:
 	ConfigObject::Ptr Commit(bool discard = true);
 
 	static bool CommitNewItems(const ActivationContext::Ptr& context, WorkQueue& upq, std::vector<ConfigItem::Ptr>& newItems);
-
-	void OnAllConfigLoadedHelper(void);
-	void CreateChildObjectsHelper(const Type::Ptr& type);
 };
 
 }
