@@ -20,13 +20,12 @@
 #include "config/configitembuilder.hpp"
 #include "base/configtype.hpp"
 #include <sstream>
-#include <boost/foreach.hpp>
 #include <boost/smart_ptr/make_shared.hpp>
 
 using namespace icinga;
 
 ConfigItemBuilder::ConfigItemBuilder(void)
-	: m_Abstract(false)
+	: m_Abstract(false), m_DefaultTmpl(false), m_IgnoreOnError(false)
 {
 	m_DebugInfo.FirstLine = 0;
 	m_DebugInfo.FirstColumn = 0;
@@ -35,7 +34,7 @@ ConfigItemBuilder::ConfigItemBuilder(void)
 }
 
 ConfigItemBuilder::ConfigItemBuilder(const DebugInfo& debugInfo)
-	: m_Abstract(false)
+	: m_Abstract(false), m_DefaultTmpl(false), m_IgnoreOnError(false)
 {
 	m_DebugInfo = debugInfo;
 }
@@ -80,6 +79,11 @@ void ConfigItemBuilder::SetFilter(const boost::shared_ptr<Expression>& filter)
 	m_Filter = filter;
 }
 
+void ConfigItemBuilder::SetDefaultTemplate(bool defaultTmpl)
+{
+	m_DefaultTmpl = defaultTmpl;
+}
+
 void ConfigItemBuilder::SetIgnoreOnError(bool ignoreOnError)
 {
 	m_IgnoreOnError = ignoreOnError;
@@ -93,7 +97,10 @@ ConfigItem::Ptr ConfigItemBuilder::Compile(void)
 		BOOST_THROW_EXCEPTION(ScriptError(msgbuf.str(), m_DebugInfo));
 	}
 
-	if (!ConfigType::GetByName(m_Type)) {
+	Type::Ptr ptype = Type::GetByName(m_Type);
+	ConfigType *ctype = dynamic_cast<ConfigType *>(ptype.get());
+
+	if (!ctype) {
 		std::ostringstream msgbuf;
 		msgbuf << "The type '" + m_Type + "' is unknown";
 		BOOST_THROW_EXCEPTION(ScriptError(msgbuf.str(), m_DebugInfo));
@@ -117,10 +124,25 @@ ConfigItem::Ptr ConfigItemBuilder::Compile(void)
 	dexpr->MakeInline();
 	exprs.push_back(dexpr);
 
+#ifdef I2_DEBUG
+	if (!m_Abstract) {
+		bool foundDefaultImport = false;
+
+		for (Expression *expr : m_Expressions) {
+			if (dynamic_cast<ImportDefaultTemplatesExpression *>(expr)) {
+				foundDefaultImport = true;
+				break;
+			}
+		}
+
+		ASSERT(foundDefaultImport);
+	}
+#endif /* I2_DEBUG */
+
 	boost::shared_ptr<DictExpression> exprl = boost::make_shared<DictExpression>(exprs, m_DebugInfo);
 	exprl->MakeInline();
 
 	return new ConfigItem(m_Type, m_Name, m_Abstract, exprl, m_Filter,
-	    m_IgnoreOnError, m_DebugInfo, m_Scope, m_Zone, m_Package);
+	    m_DefaultTmpl, m_IgnoreOnError, m_DebugInfo, m_Scope, m_Zone, m_Package);
 }
 
