@@ -28,7 +28,6 @@
 #include "base/exception.hpp"
 #include "base/scriptglobal.hpp"
 #include "base/loader.hpp"
-#include <boost/foreach.hpp>
 #include <boost/exception_ptr.hpp>
 #include <boost/exception/errinfo_nested_exception.hpp>
 
@@ -122,8 +121,10 @@ ExpressionResult VariableExpression::DoEvaluate(ScriptFrame& frame, DebugHint *d
 
 	if (frame.Locals && frame.Locals->Get(m_Variable, &value))
 		return value;
-	else if (frame.Self.IsObject() && frame.Locals != static_cast<Object::Ptr>(frame.Self) && VMOps::HasField(frame.Self, m_Variable))
-		return VMOps::GetField(frame.Self, m_Variable, frame.Sandboxed, m_DebugInfo);
+	else if (frame.Self.IsObject() && frame.Locals != frame.Self.Get<Object::Ptr>() && frame.Self.Get<Object::Ptr>()->GetOwnField(m_Variable, &value))
+		return value;
+	else if (VMOps::FindVarImport(frame, m_Variable, &value, m_DebugInfo))
+		return value;
 	else
 		return ScriptGlobal::Get(m_Variable);
 }
@@ -137,11 +138,13 @@ bool VariableExpression::GetReference(ScriptFrame& frame, bool init_dict, Value
 
 		if (dhint)
 			*dhint = NULL;
-	} else if (frame.Self.IsObject() && frame.Locals != static_cast<Object::Ptr>(frame.Self) && VMOps::HasField(frame.Self, m_Variable)) {
+	} else if (frame.Self.IsObject() && frame.Locals != frame.Self.Get<Object::Ptr>() && frame.Self.Get<Object::Ptr>()->HasOwnField(m_Variable)) {
 		*parent = frame.Self;
 
 		if (dhint && *dhint)
 			*dhint = new DebugHint((*dhint)->GetChild(m_Variable));
+	} else if (VMOps::FindVarImportRef(frame, m_Variable, parent, m_DebugInfo)) {
+		return true;
 	} else if (ScriptGlobal::Exists(m_Variable)) {
 		*parent = ScriptGlobal::GetGlobals();
 
@@ -424,15 +427,15 @@ ExpressionResult FunctionCallExpression::DoEvaluate(ScriptFrame& frame, DebugHin
 	}
 
 	if (vfunc.IsObjectType<Type>()) {
-		if (m_Args.empty())
-			return VMOps::ConstructorCall(vfunc, m_DebugInfo);
-		else if (m_Args.size() == 1) {
-			ExpressionResult argres = m_Args[0]->Evaluate(frame);
+		std::vector<Value> arguments;
+		for (Expression *arg : m_Args) {
+			ExpressionResult argres = arg->Evaluate(frame);
 			CHECK_RESULT(argres);
 
-			return VMOps::CopyConstructorCall(vfunc, argres.GetValue(), m_DebugInfo);
-		} else
-			BOOST_THROW_EXCEPTION(ScriptError("Too many arguments for constructor.", m_DebugInfo));
+			arguments.push_back(argres.GetValue());
+		}
+
+		return VMOps::ConstructorCall(vfunc, arguments, m_DebugInfo);
 	}
 
 	if (!vfunc.IsObjectType<Function>())
@@ -444,7 +447,7 @@ ExpressionResult FunctionCallExpression::DoEvaluate(ScriptFrame& frame, DebugHin
 		BOOST_THROW_EXCEPTION(ScriptError("Function is not marked as safe for sandbox mode.", m_DebugInfo));
 
 	std::vector<Value> arguments;
-	BOOST_FOREACH(Expression *arg, m_Args) {
+	for (Expression *arg : m_Args) {
 		ExpressionResult argres = arg->Evaluate(frame);
 		CHECK_RESULT(argres);
 
@@ -457,8 +460,9 @@ ExpressionResult FunctionCallExpression::DoEvaluate(ScriptFrame& frame, DebugHin
 ExpressionResult ArrayExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhint) const
 {
 	Array::Ptr result = new Array();
+	result->Reserve(m_Expressions.size());
 
-	BOOST_FOREACH(Expression *aexpr, m_Expressions) {
+	for (Expression *aexpr : m_Expressions) {
 		ExpressionResult element = aexpr->Evaluate(frame);
 		CHECK_RESULT(element);
 
@@ -480,8 +484,8 @@ ExpressionResult DictExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhint
 	Value result;
 
 	try {
-		BOOST_FOREACH(Expression *aexpr, m_Expressions) {
-			ExpressionResult element = aexpr->Evaluate(frame, dhint);
+		for (Expression *aexpr : m_Expressions) {
+			ExpressionResult element = aexpr->Evaluate(frame, m_Inline ? dhint : NULL);
 			CHECK_RESULT(element);
 			result = element.GetValue();
 		}
@@ -682,7 +686,7 @@ void icinga::BindToScope(Expression *& expr, ScopeSpecifier scopeSpec)
 	DictExpression *dexpr = dynamic_cast<DictExpression *>(expr);
 
 	if (dexpr) {
-		BOOST_FOREACH(Expression *& expr, dexpr->m_Expressions)
+		for (Expression *& expr : dexpr->m_Expressions)
 			BindToScope(expr, scopeSpec);
 
 		return;
@@ -704,9 +708,8 @@ void icinga::BindToScope(Expression *& expr, ScopeSpecifier scopeSpec)
 	}
 
 	LiteralExpression *lexpr = dynamic_cast<LiteralExpression *>(expr);
-	ScriptFrame frame;
 
-	if (lexpr && lexpr->Evaluate(frame).GetValue().IsString()) {
+	if (lexpr && lexpr->GetValue().IsString()) {
 		Expression *scope = new GetScopeExpression(scopeSpec);
 		expr = new IndexerExpression(scope, lexpr, lexpr->GetDebugInfo());
 	}
@@ -747,15 +750,40 @@ ExpressionResult ImportExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhi
 	if (!item)
 		BOOST_THROW_EXCEPTION(ScriptError("Import references unknown template: '" + name + "'", m_DebugInfo));
 
+	Dictionary::Ptr scope = item->GetScope();
+
+	if (scope)
+		scope->CopyTo(frame.Locals);
+
 	ExpressionResult result = item->GetExpression()->Evaluate(frame, dhint);
 	CHECK_RESULT(result);
 
 	return Empty;
 }
 
+ExpressionResult ImportDefaultTemplatesExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhint) const
+{
+	if (frame.Sandboxed)
+		BOOST_THROW_EXCEPTION(ScriptError("Imports are not allowed in sandbox mode.", m_DebugInfo));
+
+	String type = VMOps::GetField(frame.Self, "type", frame.Sandboxed, m_DebugInfo);
+
+	for (const ConfigItem::Ptr& item : ConfigItem::GetDefaultTemplates(type)) {
+		Dictionary::Ptr scope = item->GetScope();
+
+		if (scope)
+			scope->CopyTo(frame.Locals);
+
+		ExpressionResult result = item->GetExpression()->Evaluate(frame, dhint);
+		CHECK_RESULT(result);
+	}
+
+	return Empty;
+}
+
 ExpressionResult FunctionExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhint) const
 {
-	return VMOps::NewFunction(frame, m_Args, m_ClosedVars, m_Expression);
+	return VMOps::NewFunction(frame, m_Name, m_Args, m_ClosedVars, m_Expression);
 }
 
 ExpressionResult ApplyExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhint) const
@@ -785,7 +813,7 @@ ExpressionResult ObjectExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhi
 	}
 
 	return VMOps::NewObject(frame, m_Abstract, m_Type, name, m_Filter, m_Zone,
-	    m_Package, m_IgnoreOnError, m_ClosedVars, m_Expression, m_DebugInfo);
+	    m_Package, m_DefaultTmpl, m_IgnoreOnError, m_ClosedVars, m_Expression, m_DebugInfo);
 }
 
 ExpressionResult ForExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhint) const
@@ -891,3 +919,20 @@ ExpressionResult BreakpointExpression::DoEvaluate(ScriptFrame& frame, DebugHint
 	return Empty;
 }
 
+ExpressionResult UsingExpression::DoEvaluate(ScriptFrame& frame, DebugHint *dhint) const
+{
+	if (frame.Sandboxed)
+		BOOST_THROW_EXCEPTION(ScriptError("Using directives are not allowed in sandbox mode.", m_DebugInfo));
+
+	ExpressionResult importres = m_Name->Evaluate(frame);
+	CHECK_RESULT(importres);
+	Value import = importres.GetValue();
+
+	if (!import.IsObjectType<Dictionary>())
+		BOOST_THROW_EXCEPTION(ScriptError("The parameter must resolve to an object of type 'Dictionary'", m_DebugInfo));
+
+	ScriptFrame::AddImport(import);
+
+	return Empty;
+}
+
