@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -33,7 +33,6 @@
 #include "base/exception.hpp"
 #include "base/convert.hpp"
 #include "base/objectlock.hpp"
-#include <boost/foreach.hpp>
 #include <boost/smart_ptr/make_shared.hpp>
 #include <map>
 #include <vector>
@@ -44,63 +43,76 @@ namespace icinga
 class VMOps
 {
 public:
-	static inline Value Variable(ScriptFrame& frame, const String& name, const DebugInfo& debugInfo = DebugInfo())
+	static inline bool FindVarImportRef(ScriptFrame& frame, const String& name, Value *result, const DebugInfo& debugInfo = DebugInfo())
 	{
-		Value value;
-		if (frame.Locals && frame.Locals->Get(name, &value))
-			return value;
-		else if (frame.Self.IsObject() && frame.Locals != static_cast<Object::Ptr>(frame.Self) && HasField(frame.Self, name))
-			return GetField(frame.Self, name, frame.Sandboxed, debugInfo);
-		else
-			return ScriptGlobal::Get(name);
-	}
+		Array::Ptr imports = ScriptFrame::GetImports();
 
-	static inline Value CopyConstructorCall(const Type::Ptr& type, const Value& value, const DebugInfo& debugInfo = DebugInfo())
-	{
-		if (type->GetName() == "String")
-			return Convert::ToString(value);
-		else if (type->GetName() == "Number")
-			return Convert::ToDouble(value);
-		else if (type->GetName() == "Boolean")
-			return Convert::ToBool(value);
-		else if (!value.IsEmpty() && !type->IsAssignableFrom(value.GetReflectionType()))
-			BOOST_THROW_EXCEPTION(ScriptError("Invalid cast: Tried to cast object of type '" + value.GetReflectionType()->GetName() + "' to type '" + type->GetName() + "'", debugInfo));
-		else
-			return value;
-	}
-
-	static inline Value ConstructorCall(const Type::Ptr& type, const DebugInfo& debugInfo = DebugInfo())
-	{
-		if (type->GetName() == "String")
-			return "";
-		else if (type->GetName() == "Number")
-			return 0;
-		else if (type->GetName() == "Boolean")
-			return false;
-		else {
-			Object::Ptr object = type->Instantiate();
-
-			if (!object)
-				BOOST_THROW_EXCEPTION(ScriptError("Failed to instantiate object of type '" + type->GetName() + "'", debugInfo));
-
-			return object;
+		ObjectLock olock(imports);
+		for (const Value& import : imports) {
+			Object::Ptr obj = import;
+			if (obj->HasOwnField(name)) {
+				*result = import;
+				return true;
+			}
 		}
+
+		return false;
+	}
+
+	static inline bool FindVarImport(ScriptFrame& frame, const String& name, Value *result, const DebugInfo& debugInfo = DebugInfo())
+	{
+		Value parent;
+
+		if (FindVarImportRef(frame, name, &parent, debugInfo)) {
+			*result = GetField(parent, name, frame.Sandboxed, debugInfo);
+			return true;
+		}
+
+		return false;
+	}
+
+	static inline Value ConstructorCall(const Type::Ptr& type, const std::vector<Value>& args, const DebugInfo& debugInfo = DebugInfo())
+	{
+		if (type->GetName() == "String") {
+			if (args.empty())
+				return "";
+			else if (args.size() == 1)
+				return Convert::ToString(args[0]);
+			else
+				BOOST_THROW_EXCEPTION(ScriptError("Too many arguments for constructor."));
+		} else if (type->GetName() == "Number") {
+			if (args.empty())
+				return 0;
+			else if (args.size() == 1)
+				return Convert::ToDouble(args[0]);
+			else
+				BOOST_THROW_EXCEPTION(ScriptError("Too many arguments for constructor."));
+		} else if (type->GetName() == "Boolean") {
+			if (args.empty())
+				return 0;
+			else if (args.size() == 1)
+				return Convert::ToBool(args[0]);
+			else
+				BOOST_THROW_EXCEPTION(ScriptError("Too many arguments for constructor."));
+		} else if (args.size() == 1 && type->IsAssignableFrom(args[0].GetReflectionType()))
+			return args[0];
+		else
+			return type->Instantiate(args);
 	}
 
 	static inline Value FunctionCall(ScriptFrame& frame, const Value& self, const Function::Ptr& func, const std::vector<Value>& arguments)
 	{
-		ScriptFrame vframe;
-		
 		if (!self.IsEmpty() || self.IsString())
-			vframe.Self = self;
+			return func->Invoke(self, arguments);
+		else
+			return func->Invoke(arguments);
 
-		return func->Invoke(arguments);
 	}
 
-	static inline Value NewFunction(ScriptFrame& frame, const std::vector<String>& args,
+	static inline Value NewFunction(ScriptFrame& frame, const String& name, const std::vector<String>& args,
 	    std::map<String, Expression *> *closedVars, const boost::shared_ptr<Expression>& expression)
 	{
-		return new Function(boost::bind(&FunctionWrapper, _1, args,
+		return new Function(name, boost::bind(&FunctionWrapper, _1, args,
 		    EvaluateClosedVars(frame, closedVars), expression));
 	}
 
@@ -115,7 +127,7 @@ public:
 	}
 
 	static inline Value NewObject(ScriptFrame& frame, bool abstract, const String& type, const String& name, const boost::shared_ptr<Expression>& filter,
-		const String& zone, const String& package, bool ignoreOnError, std::map<String, Expression *> *closedVars, const boost::shared_ptr<Expression>& expression, const DebugInfo& debugInfo = DebugInfo())
+		const String& zone, const String& package, bool defaultTmpl, bool ignoreOnError, std::map<String, Expression *> *closedVars, const boost::shared_ptr<Expression>& expression, const DebugInfo& debugInfo = DebugInfo())
 	{
 		ConfigItemBuilder::Ptr item = new ConfigItemBuilder(debugInfo);
 
@@ -143,12 +155,16 @@ public:
 		item->SetType(type);
 		item->SetName(name);
 
+		if (!abstract)
+			item->AddExpression(new ImportDefaultTemplatesExpression());
+
 		item->AddExpression(new OwnedExpression(expression));
 		item->SetAbstract(abstract);
 		item->SetScope(EvaluateClosedVars(frame, closedVars));
 		item->SetZone(zone);
 		item->SetPackage(package);
 		item->SetFilter(filter);
+		item->SetDefaultTemplate(defaultTmpl);
 		item->SetIgnoreOnError(ignoreOnError);
 		item->Compile()->Register();
 
@@ -177,12 +193,12 @@ public:
 
 			{
 				ObjectLock olock(dict);
-				BOOST_FOREACH(const Dictionary::Pair& kv, dict) {
+				for (const Dictionary::Pair& kv : dict) {
 					keys.push_back(kv.first);
 				}
 			}
 
-			BOOST_FOREACH(const String& key, keys) {
+			for (const String& key : keys) {
 				frame.Locals->Set(fkvar, key);
 				frame.Locals->Set(fvvar, dict->Get(key));
 				ExpressionResult res = expression->Evaluate(frame);
@@ -194,97 +210,17 @@ public:
 		return Empty;
 	}
 
-	static inline bool HasField(const Object::Ptr& context, const String& field)
-	{
-		Dictionary::Ptr dict = dynamic_pointer_cast<Dictionary>(context);
-
-		if (dict)
-			return dict->Contains(field);
-		else {
-			Type::Ptr type = context->GetReflectionType();
-
-			if (!type)
-				return false;
-
-			return type->GetFieldId(field) != -1;
-		}
-	}
-
-	static inline Value GetPrototypeField(const Value& context, const String& field, bool not_found_error = true, const DebugInfo& debugInfo = DebugInfo())
-	{
-		Type::Ptr ctype = context.GetReflectionType();
-		Type::Ptr type = ctype;
-
-		do {
-			Object::Ptr object = type->GetPrototype();
-
-			if (object && HasField(object, field))
-				return GetField(object, field, false, debugInfo);
-
-			type = type->GetBaseType();
-		} while (type);
-
-		if (not_found_error)
-			BOOST_THROW_EXCEPTION(ScriptError("Invalid field access (for value of type '" + ctype->GetName() + "'): '" + field + "'", debugInfo));
-		else
-			return Empty;
-	}
-
 	static inline Value GetField(const Value& context, const String& field, bool sandboxed = false, const DebugInfo& debugInfo = DebugInfo())
 	{
-		if (context.IsEmpty() && !context.IsString())
+		if (unlikely(context.IsEmpty() && !context.IsString()))
 			return Empty;
 
-		if (!context.IsObject())
+		if (unlikely(!context.IsObject()))
 			return GetPrototypeField(context, field, true, debugInfo);
 
 		Object::Ptr object = context;
 
-		Dictionary::Ptr dict = dynamic_pointer_cast<Dictionary>(object);
-
-		if (dict) {
-			Value value;
-			if (dict->Get(field, &value))
-				return value;
-			else
-				return GetPrototypeField(context, field, false, debugInfo);
-		}
-
-		Array::Ptr arr = dynamic_pointer_cast<Array>(object);
-
-		if (arr) {
-			int index;
-
-			try {
-				index = Convert::ToLong(field);
-			} catch (...) {
-				return GetPrototypeField(context, field, true, debugInfo);
-			}
-
-			if (index < 0 || index >= arr->GetLength())
-				BOOST_THROW_EXCEPTION(ScriptError("Array index '" + Convert::ToString(index) + "' is out of bounds.", debugInfo));
-
-			return arr->Get(index);
-		}
-
-		Type::Ptr type = object->GetReflectionType();
-
-		if (!type)
-			return Empty;
-
-		int fid = type->GetFieldId(field);
-
-		if (fid == -1)
-			return GetPrototypeField(context, field, true, debugInfo);
-
-		if (sandboxed) {
-			Field fieldInfo = type->GetFieldInfo(fid);
-
-			if (fieldInfo.Attributes & FANoUserView)
-				BOOST_THROW_EXCEPTION(ScriptError("Accessing the field '" + field + "' for type '" + type->GetName() + "' is not allowed in sandbox mode."));
-		}
-
-		return object->GetField(fid);
+		return object->GetFieldByName(field, sandboxed, debugInfo);
 	}
 
 	static inline void SetField(const Object::Ptr& context, const String& field, const Value& value, const DebugInfo& debugInfo = DebugInfo())
@@ -292,44 +228,7 @@ public:
 		if (!context)
 			BOOST_THROW_EXCEPTION(ScriptError("Cannot set field '" + field + "' on a value that is not an object.", debugInfo));
 
-		Dictionary::Ptr dict = dynamic_pointer_cast<Dictionary>(context);
-
-		if (dict) {
-			dict->Set(field, value);
-			return;
-		}
-
-		Array::Ptr arr = dynamic_pointer_cast<Array>(context);
-
-		if (arr) {
-			int index = Convert::ToLong(field);
-			if (index >= arr->GetLength())
-				arr->Resize(index + 1);
-			arr->Set(index, value);
-			return;
-		}
-
-		Type::Ptr type = context->GetReflectionType();
-
-		if (!type)
-			BOOST_THROW_EXCEPTION(ScriptError("Cannot set field on object.", debugInfo));
-
-		int fid = type->GetFieldId(field);
-
-		if (fid == -1)
-			BOOST_THROW_EXCEPTION(ScriptError("Attribute '" + field + "' does not exist.", debugInfo));
-
-		try {
-			context->SetField(fid, value);
-		} catch (const boost::bad_lexical_cast&) {
-			Field fieldInfo = type->GetFieldInfo(fid);
-			Type::Ptr ftype = Type::GetByName(fieldInfo.TypeName);
-			BOOST_THROW_EXCEPTION(ScriptError("Attribute '" + field + "' cannot be set to value of type '" + value.GetTypeName() + "', expected '" + ftype->GetName() + "'", debugInfo));
-		} catch (const std::bad_cast&) {
-			Field fieldInfo = type->GetFieldInfo(fid);
-			Type::Ptr ftype = Type::GetByName(fieldInfo.TypeName);
-			BOOST_THROW_EXCEPTION(ScriptError("Attribute '" + field + "' cannot be set to value of type '" + value.GetTypeName() + "', expected '" + ftype->GetName() + "'", debugInfo));
-		}
+		return context->SetFieldByName(field, value, debugInfo);
 	}
 
 private:
@@ -358,7 +257,7 @@ private:
 			locals = new Dictionary();
 
 			typedef std::pair<String, Expression *> ClosedVar;
-			BOOST_FOREACH(const ClosedVar& cvar, *closedVars) {
+			for (const ClosedVar& cvar : *closedVars) {
 				locals->Set(cvar.first, cvar.second->Evaluate(frame));
 			}
 		}
