@@ -24,13 +24,10 @@
 #include "icinga/compatutility.hpp"
 #include "base/objectlock.hpp"
 #include "base/convert.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
-REGISTER_DBTYPE(CheckCommand, "command", DbObjectTypeCommand, "object_id", CommandDbObject);
-REGISTER_DBTYPE(EventCommand, "command", DbObjectTypeCommand, "object_id", CommandDbObject);
-REGISTER_DBTYPE(NotificationCommand, "command", DbObjectTypeCommand, "object_id", CommandDbObject);
+REGISTER_DBTYPE(Command, "command", DbObjectTypeCommand, "object_id", CommandDbObject);
 
 CommandDbObject::CommandDbObject(const DbType::Ptr& type, const String& name1, const String& name2)
 	: DbObject(type, name1, name2)
@@ -48,10 +45,5 @@ Dictionary::Ptr CommandDbObject::GetConfigFields(void) const
 
 Dictionary::Ptr CommandDbObject::GetStatusFields(void) const
 {
-	return Empty;
-}
-
-void CommandDbObject::OnConfigUpdate(void)
-{
-	return;
+	return Dictionary::Ptr();
 }
