@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -29,7 +29,6 @@
 #include "base/utility.hpp"
 #include "base/logger.hpp"
 #include "base/exception.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -39,13 +38,26 @@ Timer::Ptr DbConnection::m_ProgramStatusTimer;
 boost::once_flag DbConnection::m_OnceFlag = BOOST_ONCE_INIT;
 
 DbConnection::DbConnection(void)
-	: m_QueryStats(15 * 60)
+	: m_IDCacheValid(false), m_QueryStats(15 * 60), m_PendingQueries(0),
+	  m_PendingQueriesTimestamp(0), m_ActiveChangedHandler(false)
 { }
 
 void DbConnection::OnConfigLoaded(void)
 {
 	ConfigObject::OnConfigLoaded();
 
+	Value categories = GetCategories();
+
+	//TODO: Remove 'cat1 | cat2' notation in 2.6
+	if (categories.IsNumber()) {
+		SetCategoryFilter(categories);
+		Log(LogWarning, "DbConnection")
+		    << "Specifying flags using '|' for 'categories' for object '" << GetName()
+		    << "' of type '" << GetReflectionType()->GetName() << "'"
+		    << " is deprecated. This functionality will be removed in 2.6.0. Please use an array.";
+	} else
+		SetCategoryFilter(FilterArrayToInt(categories, DbQuery::GetCategoryFilterMap(), DbCatEverything));
+
 	if (!GetEnableHa()) {
 		Log(LogDebug, "DbConnection")
 		    << "HA functionality disabled. Won't pause IDO connection: " << GetName();
@@ -61,7 +73,46 @@ void DbConnection::Start(bool runtimeCreated)
 	ObjectImpl<DbConnection>::Start(runtimeCreated);
 
 	DbObject::OnQuery.connect(boost::bind(&DbConnection::ExecuteQuery, this, _1));
-	ConfigObject::OnActiveChanged.connect(boost::bind(&DbConnection::UpdateObject, this, _1));
+	DbObject::OnMultipleQueries.connect(boost::bind(&DbConnection::ExecuteMultipleQueries, this, _1));
+}
+
+void DbConnection::EnableActiveChangedHandler(void)
+{
+	if (!m_ActiveChangedHandler) {
+		ConfigObject::OnActiveChanged.connect(boost::bind(&DbConnection::UpdateObject, this, _1));
+		m_ActiveChangedHandler = true;
+	}
+}
+
+void DbConnection::StatsLoggerTimerHandler(void)
+{
+	if (!GetConnected())
+		return;
+
+	int pending = GetPendingQueryCount();
+
+	double now = Utility::GetTime();
+	double gradient = (pending - m_PendingQueries) / (now - m_PendingQueriesTimestamp);
+	double timeToZero = -pending / gradient;
+
+	String timeInfo;
+
+	if (pending > GetQueryCount(5)) {
+		timeInfo = " empty in ";
+		if (timeToZero < 0)
+			timeInfo += "infinite time, your database isn't able to keep up";
+		else
+			timeInfo += Utility::FormatDuration(timeToZero);
+	}
+
+	m_PendingQueries = pending;
+	m_PendingQueriesTimestamp = now;
+
+	Log(LogInformation, GetReflectionType()->GetName())
+	    << "Query queue items: " << pending
+	    << ", query rate: " << std::setw(2) << GetQueryCount(60) / 60.0 << "/s"
+	    << " (" << GetQueryCount(60) << "/min " << GetQueryCount(5 * 60) << "/5min " << GetQueryCount(15 * 60) << "/15min);"
+	    << timeInfo;
 }
 
 void DbConnection::Resume(void)
@@ -75,6 +126,11 @@ void DbConnection::Resume(void)
 	m_CleanUpTimer->SetInterval(60);
 	m_CleanUpTimer->OnTimerExpired.connect(boost::bind(&DbConnection::CleanUpHandler, this));
 	m_CleanUpTimer->Start();
+
+	m_StatsLoggerTimer = new Timer();
+	m_StatsLoggerTimer->SetInterval(15);
+	m_StatsLoggerTimer->OnTimerExpired.connect(boost::bind(&DbConnection::StatsLoggerTimerHandler, this));
+	m_StatsLoggerTimer->Start();
 }
 
 void DbConnection::Pause(void)
@@ -97,6 +153,9 @@ void DbConnection::Pause(void)
 	query1.Fields = new Dictionary();
 	query1.Fields->Set("instance_id", 0); /* DbConnection class fills in real ID */
 	query1.Fields->Set("program_end_time", DbValue::FromTimestamp(Utility::GetTime()));
+
+	query1.Priority = PriorityHigh;
+
 	ExecuteQuery(query1);
 
 	NewTransaction();
@@ -106,7 +165,7 @@ void DbConnection::InitializeDbTimer(void)
 {
 	m_ProgramStatusTimer = new Timer();
 	m_ProgramStatusTimer->SetInterval(10);
-	m_ProgramStatusTimer->OnTimerExpired.connect(boost::bind(&DbConnection::ProgramStatusHandler));
+	m_ProgramStatusTimer->OnTimerExpired.connect(boost::bind(&DbConnection::UpdateProgramStatus));
 	m_ProgramStatusTimer->Start();
 }
 
@@ -123,44 +182,48 @@ void DbConnection::InsertRuntimeVariable(const String& key, const Value& value)
 	DbObject::OnQuery(query);
 }
 
-void DbConnection::ProgramStatusHandler(void)
+void DbConnection::UpdateProgramStatus(void)
 {
 	Log(LogNotice, "DbConnection")
 	     << "Updating programstatus table.";
 
+	std::vector<DbQuery> queries;
+
 	DbQuery query1;
 	query1.Table = "programstatus";
-	query1.Type = DbQueryDelete;
+	query1.IdColumn = "programstatus_id";
+	query1.Type = DbQueryInsert | DbQueryUpdate;
 	query1.Category = DbCatProgramStatus;
+
+	query1.Fields = new Dictionary();
+	query1.Fields->Set("instance_id", 0); /* DbConnection class fills in real ID */
+	query1.Fields->Set("program_version", Application::GetAppVersion());
+	query1.Fields->Set("status_update_time", DbValue::FromTimestamp(Utility::GetTime()));
+	query1.Fields->Set("program_start_time", DbValue::FromTimestamp(Application::GetStartTime()));
+	query1.Fields->Set("is_currently_running", 1);
+	query1.Fields->Set("endpoint_name", IcingaApplication::GetInstance()->GetNodeName());
+	query1.Fields->Set("process_id", Utility::GetPid());
+	query1.Fields->Set("daemon_mode", 1);
+	query1.Fields->Set("last_command_check", DbValue::FromTimestamp(Utility::GetTime()));
+	query1.Fields->Set("notifications_enabled", (IcingaApplication::GetInstance()->GetEnableNotifications() ? 1 : 0));
+	query1.Fields->Set("active_host_checks_enabled", (IcingaApplication::GetInstance()->GetEnableHostChecks() ? 1 : 0));
+	query1.Fields->Set("passive_host_checks_enabled", 1);
+	query1.Fields->Set("active_service_checks_enabled", (IcingaApplication::GetInstance()->GetEnableServiceChecks() ? 1 : 0));
+	query1.Fields->Set("passive_service_checks_enabled", 1);
+	query1.Fields->Set("event_handlers_enabled", (IcingaApplication::GetInstance()->GetEnableEventHandlers() ? 1 : 0));
+	query1.Fields->Set("flap_detection_enabled", (IcingaApplication::GetInstance()->GetEnableFlapping() ? 1 : 0));
+	query1.Fields->Set("process_performance_data", (IcingaApplication::GetInstance()->GetEnablePerfdata() ? 1 : 0));
 	query1.WhereCriteria = new Dictionary();
 	query1.WhereCriteria->Set("instance_id", 0);  /* DbConnection class fills in real ID */
-	DbObject::OnQuery(query1);
+
+	query1.Priority = PriorityHigh;
+	queries.push_back(query1);
 
 	DbQuery query2;
-	query2.Table = "programstatus";
-	query2.IdColumn = "programstatus_id";
-	query2.Type = DbQueryInsert;
-	query2.Category = DbCatProgramStatus;
+	query2.Type = DbQueryNewTransaction;
+	queries.push_back(query2);
 
-	query2.Fields = new Dictionary();
-	query2.Fields->Set("instance_id", 0); /* DbConnection class fills in real ID */
-	query2.Fields->Set("program_version", Application::GetAppVersion());
-	query2.Fields->Set("status_update_time", DbValue::FromTimestamp(Utility::GetTime()));
-	query2.Fields->Set("program_start_time", DbValue::FromTimestamp(Application::GetStartTime()));
-	query2.Fields->Set("is_currently_running", 1);
-	query2.Fields->Set("endpoint_name", IcingaApplication::GetInstance()->GetNodeName());
-	query2.Fields->Set("process_id", Utility::GetPid());
-	query2.Fields->Set("daemon_mode", 1);
-	query2.Fields->Set("last_command_check", DbValue::FromTimestamp(Utility::GetTime()));
-	query2.Fields->Set("notifications_enabled", (IcingaApplication::GetInstance()->GetEnableNotifications() ? 1 : 0));
-	query2.Fields->Set("active_host_checks_enabled", (IcingaApplication::GetInstance()->GetEnableHostChecks() ? 1 : 0));
-	query2.Fields->Set("passive_host_checks_enabled", 1);
-	query2.Fields->Set("active_service_checks_enabled", (IcingaApplication::GetInstance()->GetEnableServiceChecks() ? 1 : 0));
-	query2.Fields->Set("passive_service_checks_enabled", 1);
-	query2.Fields->Set("event_handlers_enabled", (IcingaApplication::GetInstance()->GetEnableEventHandlers() ? 1 : 0));
-	query2.Fields->Set("flap_detection_enabled", (IcingaApplication::GetInstance()->GetEnableFlapping() ? 1 : 0));
-	query2.Fields->Set("process_performance_data", (IcingaApplication::GetInstance()->GetEnablePerfdata() ? 1 : 0));
-	DbObject::OnQuery(query2);
+	DbObject::OnMultipleQueries(queries);
 
 	DbQuery query3;
 	query3.Table = "runtimevariables";
@@ -170,10 +233,10 @@ void DbConnection::ProgramStatusHandler(void)
 	query3.WhereCriteria->Set("instance_id", 0);  /* DbConnection class fills in real ID */
 	DbObject::OnQuery(query3);
 
-	InsertRuntimeVariable("total_services", std::distance(ConfigType::GetObjectsByType<Service>().first, ConfigType::GetObjectsByType<Service>().second));
-	InsertRuntimeVariable("total_scheduled_services", std::distance(ConfigType::GetObjectsByType<Service>().first, ConfigType::GetObjectsByType<Service>().second));
-	InsertRuntimeVariable("total_hosts", std::distance(ConfigType::GetObjectsByType<Host>().first, ConfigType::GetObjectsByType<Host>().second));
-	InsertRuntimeVariable("total_scheduled_hosts", std::distance(ConfigType::GetObjectsByType<Host>().first, ConfigType::GetObjectsByType<Host>().second));
+	InsertRuntimeVariable("total_services", ConfigType::Get<Service>()->GetObjectCount());
+	InsertRuntimeVariable("total_scheduled_services", ConfigType::Get<Service>()->GetObjectCount());
+	InsertRuntimeVariable("total_hosts", ConfigType::Get<Host>()->GetObjectCount());
+	InsertRuntimeVariable("total_scheduled_hosts", ConfigType::Get<Host>()->GetObjectCount());
 }
 
 void DbConnection::CleanUpHandler(void)
@@ -191,7 +254,7 @@ void DbConnection::CleanUpHandler(void)
 		{ "downtimehistory", "entry_time" },
 		{ "eventhandlers", "start_time" },
 		{ "externalcommands", "entry_time" },
-		{ "flappinghistory" "event_time" },
+		{ "flappinghistory", "event_time" },
 		{ "hostchecks", "start_time" },
 		{ "logentries", "logentry_time" },
 		{ "notifications", "start_time" },
@@ -221,6 +284,40 @@ void DbConnection::CleanUpExecuteQuery(const String&, const String&, double)
 	/* Default handler does nothing. */
 }
 
+void DbConnection::SetConfigHash(const DbObject::Ptr& dbobj, const String& hash)
+{
+	SetConfigHash(dbobj->GetType(), GetObjectID(dbobj), hash);
+}
+
+void DbConnection::SetConfigHash(const DbType::Ptr& type, const DbReference& objid, const String& hash)
+{
+	if (!objid.IsValid())
+		return;
+
+	if (!hash.IsEmpty())
+		m_ConfigHashes[std::make_pair(type, objid)] = hash;
+	else
+		m_ConfigHashes.erase(std::make_pair(type, objid));
+}
+
+String DbConnection::GetConfigHash(const DbObject::Ptr& dbobj) const
+{
+	return GetConfigHash(dbobj->GetType(), GetObjectID(dbobj));
+}
+
+String DbConnection::GetConfigHash(const DbType::Ptr& type, const DbReference& objid) const
+{
+	if (!objid.IsValid())
+		return String();
+
+	auto it = m_ConfigHashes.find(std::make_pair(type, objid));
+
+	if (it == m_ConfigHashes.end())
+		return String();
+
+	return it->second;
+}
+
 void DbConnection::SetObjectID(const DbObject::Ptr& dbobj, const DbReference& dbref)
 {
 	if (dbref.IsValid())
@@ -231,9 +328,7 @@ void DbConnection::SetObjectID(const DbObject::Ptr& dbobj, const DbReference& db
 
 DbReference DbConnection::GetObjectID(const DbObject::Ptr& dbobj) const
 {
-	std::map<DbObject::Ptr, DbReference>::const_iterator it;
-
-	it = m_ObjectIDs.find(dbobj);
+	auto it = m_ObjectIDs.find(dbobj);
 
 	if (it == m_ObjectIDs.end())
 		return DbReference();
@@ -267,9 +362,7 @@ DbReference DbConnection::GetInsertID(const DbType::Ptr& type, const DbReference
 	if (!objid.IsValid())
 		return DbReference();
 
-	std::map<std::pair<DbType::Ptr, DbReference>, DbReference>::const_iterator it;
-
-	it = m_InsertIDs.find(std::make_pair(type, objid));
+	auto it = m_InsertIDs.find(std::make_pair(type, objid));
 
 	if (it == m_InsertIDs.end())
 		return DbReference();
@@ -277,26 +370,6 @@ DbReference DbConnection::GetInsertID(const DbType::Ptr& type, const DbReference
 	return it->second;
 }
 
-void DbConnection::SetNotificationInsertID(const CustomVarObject::Ptr& obj, const DbReference& dbref)
-{
-	if (dbref.IsValid())
-		m_NotificationInsertIDs[obj] = dbref;
-	else
-		m_NotificationInsertIDs.erase(obj);
-}
-
-DbReference DbConnection::GetNotificationInsertID(const CustomVarObject::Ptr& obj) const
-{
-	std::map<CustomVarObject::Ptr, DbReference>::const_iterator it;
-
-	it = m_NotificationInsertIDs.find(obj);
-
-	if (it == m_NotificationInsertIDs.end())
-		return DbReference();
-
-	return it->second;
-}
-
 void DbConnection::SetObjectActive(const DbObject::Ptr& dbobj, bool active)
 {
 	if (active)
@@ -312,12 +385,14 @@ bool DbConnection::GetObjectActive(const DbObject::Ptr& dbobj) const
 
 void DbConnection::ClearIDCache(void)
 {
+	SetIDCacheValid(false);
+
 	m_ObjectIDs.clear();
 	m_InsertIDs.clear();
-	m_NotificationInsertIDs.clear();
 	m_ActiveObjects.clear();
 	m_ConfigUpdates.clear();
 	m_StatusUpdates.clear();
+	m_ConfigHashes.clear();
 }
 
 void DbConnection::SetConfigUpdate(const DbObject::Ptr& dbobj, bool hasupdate)
@@ -346,36 +421,52 @@ bool DbConnection::GetStatusUpdate(const DbObject::Ptr& dbobj) const
 	return (m_StatusUpdates.find(dbobj) != m_StatusUpdates.end());
 }
 
-void DbConnection::ExecuteQuery(const DbQuery&)
-{
-	/* Default handler does nothing. */
-}
-
 void DbConnection::UpdateObject(const ConfigObject::Ptr& object)
 {
-	if (!GetConnected())
+	if (!GetConnected() || Application::IsShuttingDown())
 		return;
 
 	DbObject::Ptr dbobj = DbObject::GetOrCreateByObject(object);
 
 	if (dbobj) {
+		bool dbActive = GetObjectActive(dbobj);
 		bool active = object->IsActive();
 
 		if (active) {
-			ActivateObject(dbobj);
+			if (!dbActive)
+				ActivateObject(dbobj);
 
-			dbobj->SendConfigUpdate();
-			dbobj->SendStatusUpdate();
-		} else
+			Dictionary::Ptr configFields = dbobj->GetConfigFields();
+			String configHash = dbobj->CalculateConfigHash(configFields);
+			ASSERT(configHash.GetLength() <= 64);
+			configFields->Set("config_hash", configHash);
+
+			String cachedHash = GetConfigHash(dbobj);
+
+			if (cachedHash != configHash) {
+				dbobj->SendConfigUpdateHeavy(configFields);
+				dbobj->SendStatusUpdate();
+			} else {
+				dbobj->SendConfigUpdateLight();
+			}
+		} else if (!active) {
+			/* Deactivate the deleted object no matter
+			 * which state it had in the database.
+			 */
 			DeactivateObject(dbobj);
+		}
 	}
 }
 
 void DbConnection::UpdateAllObjects(void)
 {
-	ConfigType::Ptr type;
-	BOOST_FOREACH(const ConfigType::Ptr& dt, ConfigType::GetTypes()) {
-		BOOST_FOREACH(const ConfigObject::Ptr& object, dt->GetObjects()) {
+	for (const Type::Ptr& type : Type::GetAllTypes()) {
+		ConfigType *dtype = dynamic_cast<ConfigType *>(type.get());
+
+		if (!dtype)
+			continue;
+
+		for (const ConfigObject::Ptr& object : dtype->GetObjects()) {
 			UpdateObject(object);
 		}
 	}
@@ -383,44 +474,7 @@ void DbConnection::UpdateAllObjects(void)
 
 void DbConnection::PrepareDatabase(void)
 {
-	/*
-	 * only clear tables on reconnect which
-	 * cannot be updated by their existing ids
-	 * for details check https://dev.icinga.org/issues/5565
-	 */
-
-	//ClearConfigTable("commands");
-	ClearConfigTable("comments");
-	ClearConfigTable("contact_addresses");
-	ClearConfigTable("contact_notificationcommands");
-	ClearConfigTable("contactgroup_members");
-	//ClearConfigTable("contactgroups");
-	//ClearConfigTable("contacts");
-	//ClearConfigTable("contactstatus");
-	//ClearConfigTable("customvariables");
-	//ClearConfigTable("customvariablestatus");
-	//ClearConfigTable("endpoints");
-	//ClearConfigTable("endpointstatus");
-	ClearConfigTable("host_contactgroups");
-	ClearConfigTable("host_contacts");
-	ClearConfigTable("host_parenthosts");
-	ClearConfigTable("hostdependencies");
-	ClearConfigTable("hostgroup_members");
-	//ClearConfigTable("hostgroups");
-	//ClearConfigTable("hosts");
-	//ClearConfigTable("hoststatus");
-	ClearConfigTable("scheduleddowntime");
-	ClearConfigTable("service_contactgroups");
-	ClearConfigTable("service_contacts");
-	ClearConfigTable("servicedependencies");
-	ClearConfigTable("servicegroup_members");
-	//ClearConfigTable("servicegroups");
-	//ClearConfigTable("services");
-	//ClearConfigTable("servicestatus");
-	ClearConfigTable("timeperiod_timeranges");
-	//ClearConfigTable("timeperiods");
-
-	BOOST_FOREACH(const DbType::Ptr& type, DbType::GetAllTypes()) {
+	for (const DbType::Ptr& type : DbType::GetAllTypes()) {
 		FillIDCache(type);
 	}
 }
@@ -446,3 +500,18 @@ int DbConnection::GetQueryCount(RingBuffer::SizeType span) const
 	boost::mutex::scoped_lock lock(m_StatsMutex);
 	return m_QueryStats.GetValues(span);
 }
+
+bool DbConnection::IsIDCacheValid(void) const
+{
+	return m_IDCacheValid;
+}
+
+void DbConnection::SetIDCacheValid(bool valid)
+{
+	m_IDCacheValid = valid;
+}
+
+int DbConnection::GetSessionToken(void)
+{
+	return Application::GetStartTime();
+}
