@@ -22,7 +22,6 @@
 #include "base/objectlock.hpp"
 #include "base/debug.hpp"
 #include <boost/thread/once.hpp>
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -58,8 +57,15 @@ void DbType::RegisterType(const DbType::Ptr& type)
 
 DbType::Ptr DbType::GetByName(const String& name)
 {
+	String typeName;
+
+	if (name == "CheckCommand" || name == "NotificationCommand" || name == "EventCommand")
+		typeName = "Command";
+	else
+		typeName = name;
+
 	boost::mutex::scoped_lock lock(GetStaticMutex());
-	DbType::TypeMap::const_iterator it = GetTypes().find(name);
+	auto it = GetTypes().find(typeName);
 
 	if (it == GetTypes().end())
 		return DbType::Ptr();
@@ -71,7 +77,7 @@ DbType::Ptr DbType::GetByID(long tid)
 {
 	boost::mutex::scoped_lock lock(GetStaticMutex());
 
-	BOOST_FOREACH(const TypeMap::value_type& kv, GetTypes()) {
+	for (const TypeMap::value_type& kv : GetTypes()) {
 		if (kv.second->GetTypeID() == tid)
 			return kv.second;
 	}
@@ -83,7 +89,7 @@ DbObject::Ptr DbType::GetOrCreateObjectByName(const String& name1, const String&
 {
 	ObjectLock olock(this);
 
-	DbType::ObjectMap::const_iterator it = m_Objects.find(std::make_pair(name1, name2));
+	auto it = m_Objects.find(std::make_pair(name1, name2));
 
 	if (it != m_Objects.end())
 		return it->second;
@@ -96,7 +102,22 @@ DbObject::Ptr DbType::GetOrCreateObjectByName(const String& name1, const String&
 	if (!name2.IsEmpty())
 		objName += "!" + name2;
 
-	dbobj->SetObject(ConfigObject::GetObject(m_Name, objName));
+	String objType = m_Name;
+
+	if (m_TypeID == DbObjectTypeCommand) {
+		if (objName.SubStr(0, 6) == "check_") {
+			objType = "CheckCommand";
+			objName = objName.SubStr(6);
+		} else if (objName.SubStr(0, 13) == "notification_") {
+			objType = "NotificationCommand";
+			objName = objName.SubStr(13);
+		} else if (objName.SubStr(0, 6) == "event_") {
+			objType = "EventCommand";
+			objName = objName.SubStr(6);
+		}
+	}
+
+	dbobj->SetObject(ConfigObject::GetObject(objType, objName));
 
 	return dbobj;
 }
@@ -120,10 +141,11 @@ std::set<DbType::Ptr> DbType::GetAllTypes(void)
 {
 	std::set<DbType::Ptr> result;
 
-	boost::mutex::scoped_lock lock(GetStaticMutex());
-	std::pair<String, DbType::Ptr> kv;
-	BOOST_FOREACH(kv, GetTypes()) {
-		result.insert(kv.second);
+	{
+		boost::mutex::scoped_lock lock(GetStaticMutex());
+		for (const auto& kv : GetTypes()) {
+			result.insert(kv.second);
+		}
 	}
 
 	return result;
