@@ -23,7 +23,6 @@
 #include "base/objectlock.hpp"
 #include "base/initialize.hpp"
 #include "base/configtype.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -48,5 +47,5 @@ Dictionary::Ptr HostGroupDbObject::GetConfigFields(void) const
 
 Dictionary::Ptr HostGroupDbObject::GetStatusFields(void) const
 {
-	return Empty;
+	return Dictionary::Ptr();
 }
