@@ -22,7 +22,6 @@
 #include "db_ido/dbvalue.hpp"
 #include "base/objectlock.hpp"
 #include "base/initialize.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -47,5 +46,5 @@ Dictionary::Ptr ServiceGroupDbObject::GetConfigFields(void) const
 
 Dictionary::Ptr ServiceGroupDbObject::GetStatusFields(void) const
 {
-	return Empty;
+	return Dictionary::Ptr();
 }
