@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -26,7 +26,6 @@
 #include "base/convert.hpp"
 #include "base/objectlock.hpp"
 #include "base/logger.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -49,16 +48,20 @@ Dictionary::Ptr UserDbObject::GetConfigFields(void) const
 	fields->Set("host_notifications_enabled", user->GetEnableNotifications());
 	fields->Set("service_notifications_enabled", user->GetEnableNotifications());
 	fields->Set("can_submit_commands", 1);
-	fields->Set("notify_service_recovery", (user->GetTypeFilter() & NotificationRecovery) != 0);
-	fields->Set("notify_service_warning", (user->GetStateFilter() & StateFilterWarning) != 0);
-	fields->Set("notify_service_unknown", (user->GetStateFilter() & StateFilterUnknown) != 0);
-	fields->Set("notify_service_critical", (user->GetStateFilter() & StateFilterCritical) != 0);
-	fields->Set("notify_service_flapping", (user->GetTypeFilter() & (NotificationFlappingStart | NotificationFlappingEnd)) != 0);
-	fields->Set("notify_service_downtime", (user->GetTypeFilter() & (NotificationDowntimeStart | NotificationDowntimeEnd | NotificationDowntimeRemoved)) != 0);
-	fields->Set("notify_host_recovery", (user->GetTypeFilter() & NotificationRecovery) != 0);
-	fields->Set("notify_host_down", (user->GetStateFilter() & StateFilterDown) != 0);
-	fields->Set("notify_host_flapping", (user->GetTypeFilter() & (NotificationFlappingStart | NotificationFlappingEnd)) != 0);
-	fields->Set("notify_host_downtime", (user->GetTypeFilter() & (NotificationDowntimeStart | NotificationDowntimeEnd | NotificationDowntimeRemoved)) != 0);
+
+	int typeFilter = user->GetTypeFilter();
+	int stateFilter = user->GetStateFilter();
+
+	fields->Set("notify_service_recovery", (typeFilter & NotificationRecovery) != 0);
+	fields->Set("notify_service_warning", (stateFilter & StateFilterWarning) != 0);
+	fields->Set("notify_service_unknown", (stateFilter & StateFilterUnknown) != 0);
+	fields->Set("notify_service_critical", (stateFilter & StateFilterCritical) != 0);
+	fields->Set("notify_service_flapping", (typeFilter & (NotificationFlappingStart | NotificationFlappingEnd)) != 0);
+	fields->Set("notify_service_downtime", (typeFilter & (NotificationDowntimeStart | NotificationDowntimeEnd | NotificationDowntimeRemoved)) != 0);
+	fields->Set("notify_host_recovery", (typeFilter & NotificationRecovery) != 0);
+	fields->Set("notify_host_down", (stateFilter & StateFilterDown) != 0);
+	fields->Set("notify_host_flapping", (typeFilter & (NotificationFlappingStart | NotificationFlappingEnd)) != 0);
+	fields->Set("notify_host_downtime", (typeFilter & (NotificationDowntimeStart | NotificationDowntimeEnd | NotificationDowntimeRemoved)) != 0);
 
 	return fields;
 }
@@ -76,49 +79,65 @@ Dictionary::Ptr UserDbObject::GetStatusFields(void) const
 	return fields;
 }
 
-void UserDbObject::OnConfigUpdate(void)
+void UserDbObject::OnConfigUpdateHeavy(void)
 {
-	Dictionary::Ptr fields = new Dictionary();
 	User::Ptr user = static_pointer_cast<User>(GetObject());
 
 	/* groups */
 	Array::Ptr groups = user->GetGroups();
 
+	std::vector<DbQuery> queries;
+
+	DbQuery query1;
+	query1.Table = DbType::GetByName("UserGroup")->GetTable() + "_members";
+	query1.Type = DbQueryDelete;
+	query1.Category = DbCatConfig;
+	query1.WhereCriteria = new Dictionary();
+	query1.WhereCriteria->Set("contact_object_id", user);
+
+	queries.push_back(query1);
+
 	if (groups) {
 		ObjectLock olock(groups);
-		BOOST_FOREACH(const String& groupName, groups) {
+		for (const String& groupName : groups) {
 			UserGroup::Ptr group = UserGroup::GetByName(groupName);
 
-			DbQuery query1;
-			query1.Table = DbType::GetByName("UserGroup")->GetTable() + "_members";
-			query1.Type = DbQueryDelete;
-			query1.Category = DbCatConfig;
-			query1.WhereCriteria = new Dictionary();
-			query1.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
-			query1.WhereCriteria->Set("contactgroup_id", DbValue::FromObjectInsertID(group));
-			query1.WhereCriteria->Set("contact_object_id", user);
-			OnQuery(query1);
-
 			DbQuery query2;
 			query2.Table = DbType::GetByName("UserGroup")->GetTable() + "_members";
-			query2.Type = DbQueryInsert;
+			query2.Type = DbQueryInsert | DbQueryUpdate;
 			query2.Category = DbCatConfig;
 			query2.Fields = new Dictionary();
 			query2.Fields->Set("instance_id", 0); /* DbConnection class fills in real ID */
 			query2.Fields->Set("contactgroup_id", DbValue::FromObjectInsertID(group));
 			query2.Fields->Set("contact_object_id", user);
-			OnQuery(query2);
+			query2.WhereCriteria = new Dictionary();
+			query2.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
+			query2.WhereCriteria->Set("contactgroup_id", DbValue::FromObjectInsertID(group));
+			query2.WhereCriteria->Set("contact_object_id", user);
+
+			queries.push_back(query2);
 		}
 	}
 
-	/* contact addresses */
-	Log(LogDebug, "UserDbObject")
-	    << "contact addresses for '" << user->GetName() << "'";
+	DbObject::OnMultipleQueries(queries);
+
+	queries.clear();
+
+	DbQuery query2;
+	query2.Table = "contact_addresses";
+	query2.Type = DbQueryDelete;
+	query2.Category = DbCatConfig;
+	query2.WhereCriteria = new Dictionary();
+	query2.WhereCriteria->Set("contact_id", DbValue::FromObjectInsertID(user));
+
+	queries.push_back(query2);
 
 	Dictionary::Ptr vars = user->GetVars();
 
 	if (vars) { /* This is sparta. */
 		for (int i = 1; i <= 6; i++) {
+			Dictionary::Ptr fields = new Dictionary();
+
 			String key = "address" + Convert::ToString(i);
 
 			if (!vars->Contains(key))
@@ -136,12 +155,24 @@ void UserDbObject::OnConfigUpdate(void)
 			query.Table = "contact_addresses";
 			query.Category = DbCatConfig;
 			query.Fields = fields;
-			OnQuery(query);
+
+			queries.push_back(query);
 		}
 	}
+
+	DbObject::OnMultipleQueries(queries);
 }
 
-bool UserDbObject::IsStatusAttribute(const String& attribute) const
+String UserDbObject::CalculateConfigHash(const Dictionary::Ptr& configFields) const
 {
-	return (attribute == "last_notification");
+	String hashData = DbObject::CalculateConfigHash(configFields);
+
+	User::Ptr user = static_pointer_cast<User>(GetObject());
+
+	Array::Ptr groups = user->GetGroups();
+
+	if (groups)
+		hashData += DbObject::HashValue(groups);
+
+	return SHA256(hashData);
 }
