@@ -38,12 +38,13 @@ public:
 
 	UserDbObject(const DbType::Ptr& type, const String& name1, const String& name2);
 
+protected:
 	virtual Dictionary::Ptr GetConfigFields(void) const override;
 	virtual Dictionary::Ptr GetStatusFields(void) const override;
 
-	virtual void OnConfigUpdate(void) override;
+	virtual void OnConfigUpdateHeavy(void) override;
 
-	virtual bool IsStatusAttribute(const String& attribute) const override;
+	virtual String CalculateConfigHash(const Dictionary::Ptr& configFields) const override;
 };
 
 }
