@@ -23,7 +23,6 @@
 #include "base/objectlock.hpp"
 #include "base/initialize.hpp"
 #include "base/configtype.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -45,5 +44,5 @@ Dictionary::Ptr UserGroupDbObject::GetConfigFields(void) const
 
 Dictionary::Ptr UserGroupDbObject::GetStatusFields(void) const
 {
-	return Empty;
+	return Dictionary::Ptr();
 }
