@@ -57,26 +57,26 @@ public:
 	virtual int GetPendingQueryCount(void) const override;
 
 protected:
+	virtual void OnConfigLoaded(void) override;
 	virtual void Resume(void) override;
 	virtual void Pause(void) override;
 
 	virtual void ActivateObject(const DbObject::Ptr& dbobj) override;
 	virtual void DeactivateObject(const DbObject::Ptr& dbobj) override;
 	virtual void ExecuteQuery(const DbQuery& query) override;
-	virtual void ExecuteMultipleQueries(const std::vector<DbQuery>& queries);
+	virtual void ExecuteMultipleQueries(const std::vector<DbQuery>& queries) override;
 	virtual void CleanUpExecuteQuery(const String& table, const String& time_key, double time_value) override;
 	virtual void FillIDCache(const DbType::Ptr& type) override;
 	virtual void NewTransaction(void) override;
 
 private:
 	DbReference m_InstanceID;
-	int m_SessionToken;
 
 	WorkQueue m_QueryQueue;
 
 	MYSQL m_Connection;
 	int m_AffectedRows;
-	int m_MaxPacketSize;
+	unsigned int m_MaxPacketSize;
 
 	std::vector<IdoAsyncQuery> m_AsyncQueries;
 
@@ -107,15 +107,15 @@ private:
 
 	bool CanExecuteQuery(const DbQuery& query);
 
-	void InternalExecuteQuery(const DbQuery& query, DbQueryType *typeOverride = NULL);
+	void InternalExecuteQuery(const DbQuery& query, int typeOverride = -1);
 	void InternalExecuteMultipleQueries(const std::vector<DbQuery>& queries);
 
 	void FinishExecuteQuery(const DbQuery& query, int type, bool upsert);
 	void InternalCleanUpExecuteQuery(const String& table, const String& time_key, double time_value);
 	void InternalNewTransaction(void);
 
-	virtual void ClearConfigTable(const String& table) override;
-	void ClearCustomVarTable(const String& table);
+	void ClearTableBySession(const String& table);
+	void ClearTablesBySession(void);
 
 	void ExceptionHandler(boost::exception_ptr exp);
 
