@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -32,7 +32,6 @@
 #include "base/context.hpp"
 #include "base/statsfunction.hpp"
 #include <boost/tuple/tuple.hpp>
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -41,14 +40,23 @@ REGISTER_TYPE(IdoPgsqlConnection);
 REGISTER_STATSFUNCTION(IdoPgsqlConnection, &IdoPgsqlConnection::StatsFunc);
 
 IdoPgsqlConnection::IdoPgsqlConnection(void)
-	: m_QueryQueue(500000)
-{ }
+	: m_QueryQueue(1000000)
+{
+	m_QueryQueue.SetName("IdoPgsqlConnection, " + GetName());
+}
+
+void IdoPgsqlConnection::OnConfigLoaded(void)
+{
+	ObjectImpl<IdoPgsqlConnection>::OnConfigLoaded();
+
+	m_QueryQueue.SetName("IdoPgsqlConnection, " + GetName());
+}
 
 void IdoPgsqlConnection::StatsFunc(const Dictionary::Ptr& status, const Array::Ptr& perfdata)
 {
 	Dictionary::Ptr nodes = new Dictionary();
 
-	BOOST_FOREACH(const IdoPgsqlConnection::Ptr& idopgsqlconnection, ConfigType::GetObjectsByType<IdoPgsqlConnection>()) {
+	for (const IdoPgsqlConnection::Ptr& idopgsqlconnection : ConfigType::GetObjectsByType<IdoPgsqlConnection>()) {
 		size_t items = idopgsqlconnection->m_QueryQueue.GetLength();
 
 		Dictionary::Ptr stats = new Dictionary();
@@ -97,7 +105,7 @@ void IdoPgsqlConnection::Pause(void)
 
 	DbConnection::Pause();
 
-	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::Disconnect, this));
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::Disconnect, this), PriorityHigh);
 	m_QueryQueue.Join();
 }
 
@@ -139,7 +147,7 @@ void IdoPgsqlConnection::TxTimerHandler(void)
 
 void IdoPgsqlConnection::NewTransaction(void)
 {
-	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalNewTransaction, this), true);
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalNewTransaction, this), PriorityHigh, true);
 }
 
 void IdoPgsqlConnection::InternalNewTransaction(void)
@@ -155,7 +163,7 @@ void IdoPgsqlConnection::InternalNewTransaction(void)
 
 void IdoPgsqlConnection::ReconnectTimerHandler(void)
 {
-	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::Reconnect, this));
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::Reconnect, this), PriorityLow);
 }
 
 void IdoPgsqlConnection::Reconnect(void)
@@ -164,229 +172,250 @@ void IdoPgsqlConnection::Reconnect(void)
 
 	CONTEXT("Reconnecting to PostgreSQL IDO database '" + GetName() + "'");
 
-	m_SessionToken = static_cast<int>(Utility::GetTime());
+	double startTime = Utility::GetTime();
 
 	SetShouldConnect(true);
 
-	std::vector<DbObject::Ptr> active_dbobjs;
+	bool reconnect = false;
 
-	{
-		bool reconnect = false;
-
-		if (GetConnected()) {
-			/* Check if we're really still connected */
-			try {
-				Query("SELECT 1");
-				return;
-			} catch (const std::exception&) {
-				PQfinish(m_Connection);
-				SetConnected(false);
-				reconnect = true;
-			}
-		}
-
-		ClearIDCache();
-
-		String ihost, iport, iuser, ipasswd, idb;
-		const char *host, *port, *user , *passwd, *db;
-
-		ihost = GetHost();
-		iport = GetPort();
-		iuser = GetUser();
-		ipasswd = GetPassword();
-		idb = GetDatabase();
-
-		host = (!ihost.IsEmpty()) ? ihost.CStr() : NULL;
-		port = (!iport.IsEmpty()) ? iport.CStr() : NULL;
-		user = (!iuser.IsEmpty()) ? iuser.CStr() : NULL;
-		passwd = (!ipasswd.IsEmpty()) ? ipasswd.CStr() : NULL;
-		db = (!idb.IsEmpty()) ? idb.CStr() : NULL;
-
-		m_Connection = PQsetdbLogin(host, port, NULL, NULL, db, user, passwd);
-
-		if (!m_Connection)
+	if (GetConnected()) {
+		/* Check if we're really still connected */
+		try {
+			Query("SELECT 1");
 			return;
-
-		if (PQstatus(m_Connection) != CONNECTION_OK) {
-			String message = PQerrorMessage(m_Connection);
+		} catch (const std::exception&) {
 			PQfinish(m_Connection);
 			SetConnected(false);
-
-			Log(LogCritical, "IdoPgsqlConnection")
-			    << "Connection to database '" << db << "' with user '" << user << "' on '" << host << ":" << port
-			    << "' failed: \"" << message << "\"";
-
-			BOOST_THROW_EXCEPTION(std::runtime_error(message));
+			reconnect = true;
 		}
+	}
 
-		SetConnected(true);
+	ClearIDCache();
 
-		IdoPgsqlResult result;
+	String ihost, iport, iuser, ipasswd, idb;
+	const char *host, *port, *user , *passwd, *db;
 
-		/* explicitely require legacy mode for string escaping in PostgreSQL >= 9.1
-		 * changing standard_conforming_strings to on by default
-		 */
-		if (PQserverVersion(m_Connection) >= 90100)
-			result = Query("SET standard_conforming_strings TO off");
+	ihost = GetHost();
+	iport = GetPort();
+	iuser = GetUser();
+	ipasswd = GetPassword();
+	idb = GetDatabase();
 
-		String dbVersionName = "idoutils";
-		result = Query("SELECT version FROM " + GetTablePrefix() + "dbversion WHERE name=E'" + Escape(dbVersionName) + "'");
+	host = (!ihost.IsEmpty()) ? ihost.CStr() : NULL;
+	port = (!iport.IsEmpty()) ? iport.CStr() : NULL;
+	user = (!iuser.IsEmpty()) ? iuser.CStr() : NULL;
+	passwd = (!ipasswd.IsEmpty()) ? ipasswd.CStr() : NULL;
+	db = (!idb.IsEmpty()) ? idb.CStr() : NULL;
 
-		Dictionary::Ptr row = FetchRow(result, 0);
+	m_Connection = PQsetdbLogin(host, port, NULL, NULL, db, user, passwd);
 
-		if (!row) {
-			PQfinish(m_Connection);
-			SetConnected(false);
+	if (!m_Connection)
+		return;
 
-			Log(LogCritical, "IdoPgsqlConnection", "Schema does not provide any valid version! Verify your schema installation.");
+	if (PQstatus(m_Connection) != CONNECTION_OK) {
+		String message = PQerrorMessage(m_Connection);
+		PQfinish(m_Connection);
+		SetConnected(false);
 
-			Application::Exit(EXIT_FAILURE);
-		}
+		Log(LogCritical, "IdoPgsqlConnection")
+		    << "Connection to database '" << db << "' with user '" << user << "' on '" << host << ":" << port
+		    << "' failed: \"" << message << "\"";
 
-		String version = row->Get("version");
+		BOOST_THROW_EXCEPTION(std::runtime_error(message));
+	}
 
-		SetSchemaVersion(version);
+	SetConnected(true);
 
-		if (Utility::CompareVersion(IDO_COMPAT_SCHEMA_VERSION, version) < 0) {
-			PQfinish(m_Connection);
-			SetConnected(false);
+	IdoPgsqlResult result;
 
-			Log(LogCritical, "IdoPgsqlConnection")
-			    << "Schema version '" << version << "' does not match the required version '"
-			    << IDO_COMPAT_SCHEMA_VERSION << "' (or newer)! Please check the upgrade documentation.";
+	/* explicitely require legacy mode for string escaping in PostgreSQL >= 9.1
+	 * changing standard_conforming_strings to on by default
+	 */
+	if (PQserverVersion(m_Connection) >= 90100)
+		result = Query("SET standard_conforming_strings TO off");
 
-			Application::Exit(EXIT_FAILURE);
-		}
+	String dbVersionName = "idoutils";
+	result = Query("SELECT version FROM " + GetTablePrefix() + "dbversion WHERE name=E'" + Escape(dbVersionName) + "'");
 
-		String instanceName = GetInstanceName();
+	Dictionary::Ptr row = FetchRow(result, 0);
 
-		result = Query("SELECT instance_id FROM " + GetTablePrefix() + "instances WHERE instance_name = E'" + Escape(instanceName) + "'");
+	if (!row) {
+		PQfinish(m_Connection);
+		SetConnected(false);
+
+		Log(LogCritical, "IdoPgsqlConnection", "Schema does not provide any valid version! Verify your schema installation.");
+
+		Application::Exit(EXIT_FAILURE);
+	}
+
+	String version = row->Get("version");
+
+	SetSchemaVersion(version);
+
+	if (Utility::CompareVersion(IDO_COMPAT_SCHEMA_VERSION, version) < 0) {
+		PQfinish(m_Connection);
+		SetConnected(false);
+
+		Log(LogCritical, "IdoPgsqlConnection")
+		    << "Schema version '" << version << "' does not match the required version '"
+		    << IDO_COMPAT_SCHEMA_VERSION << "' (or newer)! Please check the upgrade documentation.";
+
+		Application::Exit(EXIT_FAILURE);
+	}
+
+	String instanceName = GetInstanceName();
+
+	result = Query("SELECT instance_id FROM " + GetTablePrefix() + "instances WHERE instance_name = E'" + Escape(instanceName) + "'");
+	row = FetchRow(result, 0);
+
+	if (!row) {
+		Query("INSERT INTO " + GetTablePrefix() + "instances (instance_name, instance_description) VALUES (E'" + Escape(instanceName) + "', E'" + Escape(GetInstanceDescription()) + "')");
+		m_InstanceID = GetSequenceValue(GetTablePrefix() + "instances", "instance_id");
+	} else {
+		m_InstanceID = DbReference(row->Get("instance_id"));
+	}
+
+	Endpoint::Ptr my_endpoint = Endpoint::GetLocalEndpoint();
+
+	/* we have an endpoint in a cluster setup, so decide if we can proceed here */
+	if (my_endpoint && GetHAMode() == HARunOnce) {
+		/* get the current endpoint writing to programstatus table */
+		result = Query("SELECT UNIX_TIMESTAMP(status_update_time) AS status_update_time, endpoint_name FROM " +
+		    GetTablePrefix() + "programstatus WHERE instance_id = " + Convert::ToString(m_InstanceID));
 		row = FetchRow(result, 0);
 
-		if (!row) {
-			Query("INSERT INTO " + GetTablePrefix() + "instances (instance_name, instance_description) VALUES (E'" + Escape(instanceName) + "', E'" + Escape(GetInstanceDescription()) + "')");
-			m_InstanceID = GetSequenceValue(GetTablePrefix() + "instances", "instance_id");
-		} else {
-			m_InstanceID = DbReference(row->Get("instance_id"));
-		}
+		String endpoint_name;
 
-		Endpoint::Ptr my_endpoint = Endpoint::GetLocalEndpoint();
+		if (row)
+			endpoint_name = row->Get("endpoint_name");
+		else
+			Log(LogNotice, "IdoPgsqlConnection", "Empty program status table");
 
-		/* we have an endpoint in a cluster setup, so decide if we can proceed here */
-		if (my_endpoint && GetHAMode() == HARunOnce) {
-			/* get the current endpoint writing to programstatus table */
-			result = Query("SELECT UNIX_TIMESTAMP(status_update_time) AS status_update_time, endpoint_name FROM " +
-			    GetTablePrefix() + "programstatus WHERE instance_id = " + Convert::ToString(m_InstanceID));
-			row = FetchRow(result, 0);
-
-			String endpoint_name;
+		/* if we did not write into the database earlier, another instance is active */
+		if (endpoint_name != my_endpoint->GetName()) {
+			double status_update_time;
 
 			if (row)
-				endpoint_name = row->Get("endpoint_name");
+				status_update_time = row->Get("status_update_time");
 			else
-				Log(LogNotice, "IdoPgsqlConnection", "Empty program status table");
+				status_update_time = 0;
 
-			/* if we did not write into the database earlier, another instance is active */
-			if (endpoint_name != my_endpoint->GetName()) {
-				double status_update_time;
+			double status_update_age = Utility::GetTime() - status_update_time;
 
-				if (row)
-					status_update_time = row->Get("status_update_time");
-				else
-					status_update_time = 0;
+			Log(LogNotice, "IdoPgsqlConnection")
+			    << "Last update by '" << endpoint_name << "' was " << status_update_age << "s ago.";
 
-				double status_update_age = Utility::GetTime() - status_update_time;
+			if (status_update_age < GetFailoverTimeout()) {
+				PQfinish(m_Connection);
+				SetConnected(false);
+				SetShouldConnect(false);
 
-				Log(LogNotice, "IdoPgsqlConnection")
-				    << "Last update by '" << endpoint_name << "' was " << status_update_age << "s ago.";
-
-				if (status_update_age < GetFailoverTimeout()) {
-					PQfinish(m_Connection);
-					SetConnected(false);
-					SetShouldConnect(false);
-
-					return;
-				}
-
-				/* activate the IDO only, if we're authoritative in this zone */
-				if (IsPaused()) {
-					Log(LogNotice, "IdoPgsqlConnection")
-					    << "Local endpoint '" << my_endpoint->GetName() << "' is not authoritative, bailing out.";
-
-					PQfinish(m_Connection);
-					SetConnected(false);
-
-					return;
-				}
+				return;
 			}
 
-			Log(LogNotice, "IdoPgsqlConnection", "Enabling IDO connection.");
+			/* activate the IDO only, if we're authoritative in this zone */
+			if (IsPaused()) {
+				Log(LogNotice, "IdoPgsqlConnection")
+				    << "Local endpoint '" << my_endpoint->GetName() << "' is not authoritative, bailing out.";
+
+				PQfinish(m_Connection);
+				SetConnected(false);
+
+				return;
+			}
 		}
 
-		Log(LogInformation, "IdoPgsqlConnection")
-		    << "pgSQL IDO instance id: " << static_cast<long>(m_InstanceID) << " (schema version: '" + version + "')";
+		Log(LogNotice, "IdoPgsqlConnection", "Enabling IDO connection.");
+	}
 
-		/* record connection */
-		Query("INSERT INTO " + GetTablePrefix() + "conninfo " +
-		    "(instance_id, connect_time, last_checkin_time, agent_name, agent_version, connect_type, data_start_time) VALUES ("
-		    + Convert::ToString(static_cast<long>(m_InstanceID)) + ", NOW(), NOW(), E'icinga2 db_ido_pgsql', E'" + Escape(Application::GetAppVersion())
-		    + "', E'" + (reconnect ? "RECONNECT" : "INITIAL") + "', NOW())");
+	Log(LogInformation, "IdoPgsqlConnection")
+	    << "pgSQL IDO instance id: " << static_cast<long>(m_InstanceID) << " (schema version: '" + version + "')";
 
-		/* clear config tables for the initial config dump */
-		PrepareDatabase();
+	Query("BEGIN");
 
-		std::ostringstream q1buf;
-		q1buf << "SELECT object_id, objecttype_id, name1, name2, is_active FROM " + GetTablePrefix() + "objects WHERE instance_id = " << static_cast<long>(m_InstanceID);
-		result = Query(q1buf.str());
+	/* update programstatus table */
+	UpdateProgramStatus();
 
-		int index = 0;
-		while ((row = FetchRow(result, index))) {
-			index++;
+	/* record connection */
+	Query("INSERT INTO " + GetTablePrefix() + "conninfo " +
+	    "(instance_id, connect_time, last_checkin_time, agent_name, agent_version, connect_type, data_start_time) VALUES ("
+	    + Convert::ToString(static_cast<long>(m_InstanceID)) + ", NOW(), NOW(), E'icinga2 db_ido_pgsql', E'" + Escape(Application::GetAppVersion())
+	    + "', E'" + (reconnect ? "RECONNECT" : "INITIAL") + "', NOW())");
 
-			DbType::Ptr dbtype = DbType::GetByID(row->Get("objecttype_id"));
+	/* clear config tables for the initial config dump */
+	PrepareDatabase();
 
-			if (!dbtype)
-				continue;
+	std::ostringstream q1buf;
+	q1buf << "SELECT object_id, objecttype_id, name1, name2, is_active FROM " + GetTablePrefix() + "objects WHERE instance_id = " << static_cast<long>(m_InstanceID);
+	result = Query(q1buf.str());
 
-			DbObject::Ptr dbobj = dbtype->GetOrCreateObjectByName(row->Get("name1"), row->Get("name2"));
-			SetObjectID(dbobj, DbReference(row->Get("object_id")));
-			SetObjectActive(dbobj, row->Get("is_active"));
+	std::vector<DbObject::Ptr> activeDbObjs;
 
-			if (GetObjectActive(dbobj))
-				active_dbobjs.push_back(dbobj);
-		}
+	int index = 0;
+	while ((row = FetchRow(result, index))) {
+		index++;
 
-		Query("BEGIN");
+		DbType::Ptr dbtype = DbType::GetByID(row->Get("objecttype_id"));
+
+		if (!dbtype)
+			continue;
+
+		DbObject::Ptr dbobj = dbtype->GetOrCreateObjectByName(row->Get("name1"), row->Get("name2"));
+		SetObjectID(dbobj, DbReference(row->Get("object_id")));
+		bool active = row->Get("is_active");
+		SetObjectActive(dbobj, active);
+
+		if (active)
+			activeDbObjs.push_back(dbobj);
+	}
+
+	SetIDCacheValid(true);
+
+	EnableActiveChangedHandler();
+
+	for (const DbObject::Ptr& dbobj : activeDbObjs) {
+		if (dbobj->GetObject())
+			continue;
+
+		Log(LogNotice, "IdoPgsqlConnection")
+		    << "Deactivate deleted object name1: '" << dbobj->GetName1()
+		    << "' name2: '" << dbobj->GetName2() + "'.";
+		DeactivateObject(dbobj);
 	}
 
 	UpdateAllObjects();
 
-	/* deactivate all deleted configuration objects */
-	BOOST_FOREACH(const DbObject::Ptr& dbobj, active_dbobjs) {
-		if (dbobj->GetObject() == NULL) {
-			Log(LogNotice, "IdoPgsqlConnection")
-			    << "Deactivate deleted object name1: '" << dbobj->GetName1()
-			    << "' name2: '" << dbobj->GetName2() + "'.";
-			DeactivateObject(dbobj);
-		}
-	}
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::ClearTablesBySession, this), PriorityLow);
 
-	/* delete all customvariables without current session token */
-	ClearCustomVarTable("customvariables");
-	ClearCustomVarTable("customvariablestatus");
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::FinishConnect, this, startTime), PriorityLow);
+}
+
+void IdoPgsqlConnection::FinishConnect(double startTime)
+{
+	AssertOnWorkQueue();
+
+	if (!GetConnected())
+		return;
+
+	Log(LogInformation, "IdoPgsqlConnection")
+	    << "Finished reconnecting to PostgreSQL IDO database in " << std::setw(2) << Utility::GetTime() - startTime << " second(s).";
 
 	Query("COMMIT");
 	Query("BEGIN");
 }
 
-void IdoPgsqlConnection::ClearCustomVarTable(const String& table)
+void IdoPgsqlConnection::ClearTablesBySession(void)
 {
-	Query("DELETE FROM " + GetTablePrefix() + table + " WHERE session_token <> " + Convert::ToString(m_SessionToken));
+	/* delete all comments and downtimes without current session token */
+	ClearTableBySession("comments");
+	ClearTableBySession("scheduleddowntime");
 }
 
-void IdoPgsqlConnection::ClearConfigTable(const String& table)
+void IdoPgsqlConnection::ClearTableBySession(const String& table)
 {
-	Query("DELETE FROM " + GetTablePrefix() + table + " WHERE instance_id = " + Convert::ToString(static_cast<long>(m_InstanceID)));
+	Query("DELETE FROM " + GetTablePrefix() + table + " WHERE instance_id = " +
+	    Convert::ToString(static_cast<long>(m_InstanceID)) + " AND session_token <> " +
+	    Convert::ToString(GetSessionToken()));
 }
 
 IdoPgsqlResult IdoPgsqlConnection::Query(const String& query)
@@ -464,10 +493,12 @@ String IdoPgsqlConnection::Escape(const String& s)
 {
 	AssertOnWorkQueue();
 
-	size_t length = s.GetLength();
-	char *to = new char[s.GetLength() * 2 + 1];
+	String utf8s = Utility::ValidateUTF8(s);
 
-	PQescapeStringConn(m_Connection, to, s.CStr(), length, NULL);
+	size_t length = utf8s.GetLength();
+	char *to = new char[utf8s.GetLength() * 2 + 1];
+
+	PQescapeStringConn(m_Connection, to, utf8s.CStr(), length, NULL);
 
 	String result = String(to);
 
@@ -501,7 +532,7 @@ Dictionary::Ptr IdoPgsqlConnection::FetchRow(const IdoPgsqlResult& result, int r
 
 void IdoPgsqlConnection::ActivateObject(const DbObject::Ptr& dbobj)
 {
-	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalActivateObject, this, dbobj));
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalActivateObject, this, dbobj), PriorityLow);
 }
 
 void IdoPgsqlConnection::InternalActivateObject(const DbObject::Ptr& dbobj)
@@ -535,7 +566,7 @@ void IdoPgsqlConnection::InternalActivateObject(const DbObject::Ptr& dbobj)
 
 void IdoPgsqlConnection::DeactivateObject(const DbObject::Ptr& dbobj)
 {
-	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalDeactivateObject, this, dbobj));
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalDeactivateObject, this, dbobj), PriorityLow);
 }
 
 void IdoPgsqlConnection::InternalDeactivateObject(const DbObject::Ptr& dbobj)
@@ -563,13 +594,8 @@ bool IdoPgsqlConnection::FieldToEscapedString(const String& key, const Value& va
 	if (key == "instance_id") {
 		*result = static_cast<long>(m_InstanceID);
 		return true;
-	}
-	if (key == "session_token") {
-		*result = m_SessionToken;
-		return true;
-	}
-	if (key == "notification_id") {
-		*result = static_cast<long>(GetNotificationInsertID(value));
+	} else if (key == "session_token") {
+		*result = GetSessionToken();
 		return true;
 	}
 
@@ -583,12 +609,16 @@ bool IdoPgsqlConnection::FieldToEscapedString(const String& key, const Value& va
 			return true;
 		}
 
+		if (!IsIDCacheValid())
+			return false;
+
 		DbReference dbrefcol;
 
 		if (DbValue::IsObjectInsertID(value)) {
 			dbrefcol = GetInsertID(dbobjcol);
 
-			ASSERT(dbrefcol.IsValid());
+			if (!dbrefcol.IsValid())
+				return false;
 		} else {
 			dbrefcol = GetObjectID(dbobjcol);
 
@@ -610,6 +640,14 @@ bool IdoPgsqlConnection::FieldToEscapedString(const String& key, const Value& va
 		*result = Value(msgbuf.str());
 	} else if (DbValue::IsTimestampNow(value)) {
 		*result = "NOW()";
+	} else if (DbValue::IsObjectInsertID(value)) {
+		long id = static_cast<long>(rawvalue);
+
+		if (id <= 0)
+			return false;
+
+		*result = id;
+		return true;
 	} else {
 		Value fvalue;
 
@@ -628,22 +666,95 @@ void IdoPgsqlConnection::ExecuteQuery(const DbQuery& query)
 {
 	ASSERT(query.Category != DbCatInvalid);
 
-	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, (DbQueryType *)NULL), true);
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, -1), query.Priority, true);
 }
 
-void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, DbQueryType *typeOverride)
+void IdoPgsqlConnection::ExecuteMultipleQueries(const std::vector<DbQuery>& queries)
+{
+	if (queries.empty())
+		return;
+
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalExecuteMultipleQueries, this, queries), queries[0].Priority, true);
+}
+
+bool IdoPgsqlConnection::CanExecuteQuery(const DbQuery& query)
+{
+	if (query.Object && !IsIDCacheValid())
+		return false;
+
+	if (query.WhereCriteria) {
+		ObjectLock olock(query.WhereCriteria);
+		Value value;
+
+		for (const Dictionary::Pair& kv : query.WhereCriteria) {
+			if (!FieldToEscapedString(kv.first, kv.second, &value))
+				return false;
+		}
+	}
+
+	if (query.Fields) {
+		ObjectLock olock(query.Fields);
+
+		for (const Dictionary::Pair& kv : query.Fields) {
+			Value value;
+
+			if (kv.second.IsEmpty() && !kv.second.IsString())
+				continue;
+
+			if (!FieldToEscapedString(kv.first, kv.second, &value))
+				return false;
+		}
+	}
+
+	return true;
+}
+
+void IdoPgsqlConnection::InternalExecuteMultipleQueries(const std::vector<DbQuery>& queries)
 {
 	AssertOnWorkQueue();
 
-	if ((query.Category & GetCategories()) == 0)
-		return;
-
 	if (!GetConnected())
 		return;
 
+	for (const DbQuery& query : queries) {
+		ASSERT(query.Type == DbQueryNewTransaction || query.Category != DbCatInvalid);
+
+		if (!CanExecuteQuery(query)) {
+			m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalExecuteMultipleQueries, this, queries), query.Priority);
+			return;
+		}
+	}
+
+	for (const DbQuery& query : queries) {
+		InternalExecuteQuery(query);
+	}
+}
+
+void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, int typeOverride)
+{
+	AssertOnWorkQueue();
+
+	if (!GetConnected())
+		return;
+
+	if (query.Type == DbQueryNewTransaction) {
+		InternalNewTransaction();
+		return;
+	}
+
+	/* check whether we're allowed to execute the query first */
+	if (GetCategoryFilter() != DbCatEverything && (query.Category & GetCategoryFilter()) == 0)
+		return;
+
 	if (query.Object && query.Object->GetObject()->GetExtension("agent_check").ToBool())
 		return;
 
+	/* check if there are missing object/insert ids and re-enqueue the query */
+	if (!CanExecuteQuery(query)) {
+		m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, typeOverride), query.Priority);
+		return;
+	}
+
 	std::ostringstream qbuf, where;
 	int type;
 
@@ -654,9 +765,11 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, DbQueryType
 		Value value;
 		bool first = true;
 
-		BOOST_FOREACH(const Dictionary::Pair& kv, query.WhereCriteria) {
-			if (!FieldToEscapedString(kv.first, kv.second, &value))
+		for (const Dictionary::Pair& kv : query.WhereCriteria) {
+			if (!FieldToEscapedString(kv.first, kv.second, &value)) {
+				m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, -1), query.Priority);
 				return;
+			}
 
 			if (!first)
 				where << " AND ";
@@ -668,19 +781,19 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, DbQueryType
 		}
 	}
 
-	type = typeOverride ? *typeOverride : query.Type;
+	type = (typeOverride != -1) ? typeOverride : query.Type;
 
 	bool upsert = false;
 
 	if ((type & DbQueryInsert) && (type & DbQueryUpdate)) {
 		bool hasid = false;
 
-		ASSERT(query.Object);
-
-		if (query.ConfigUpdate)
-			hasid = GetConfigUpdate(query.Object);
-		else if (query.StatusUpdate)
-			hasid = GetStatusUpdate(query.Object);
+		if (query.Object) {
+			if (query.ConfigUpdate)
+				hasid = GetConfigUpdate(query.Object);
+			else if (query.StatusUpdate)
+				hasid = GetStatusUpdate(query.Object);
+		}
 
 		if (!hasid)
 			upsert = true;
@@ -688,6 +801,14 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, DbQueryType
 		type = DbQueryUpdate;
 	}
 
+	if ((type & DbQueryInsert) && (type & DbQueryDelete)) {
+		std::ostringstream qdel;
+		qdel << "DELETE FROM " << GetTablePrefix() << query.Table << where.str();
+		Query(qdel.str());
+
+		type = DbQueryInsert;
+	}
+
 	switch (type) {
 		case DbQueryInsert:
 			qbuf << "INSERT INTO " << GetTablePrefix() << query.Table;
@@ -712,12 +833,14 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, DbQueryType
 
 		Value value;
 		bool first = true;
-		BOOST_FOREACH(const Dictionary::Pair& kv, query.Fields) {
+		for (const Dictionary::Pair& kv : query.Fields) {
 			if (kv.second.IsEmpty() && !kv.second.IsString())
 				continue;
 
-			if (!FieldToEscapedString(kv.first, kv.second, &value))
+			if (!FieldToEscapedString(kv.first, kv.second, &value)) {
+				m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, -1), query.Priority);
 				return;
+			}
 
 			if (type == DbQueryInsert) {
 				if (!first) {
@@ -748,8 +871,7 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, DbQueryType
 	Query(qbuf.str());
 
 	if (upsert && GetAffectedRows() == 0) {
-		DbQueryType to = DbQueryInsert;
-		InternalExecuteQuery(query, &to);
+		InternalExecuteQuery(query, DbQueryDelete | DbQueryInsert);
 
 		return;
 	}
@@ -768,18 +890,15 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, DbQueryType
 			SetStatusUpdate(query.Object, true);
 	}
 
-	if (type == DbQueryInsert && query.Table == "notifications" && query.NotificationObject) { // FIXME remove hardcoded table name
-		String idField = "notification_id";
-		DbReference seqval = GetSequenceValue(GetTablePrefix() + query.Table, idField);
-		SetNotificationInsertID(query.NotificationObject, seqval);
-		Log(LogDebug, "IdoPgsqlConnection")
-		    << "saving contactnotification notification_id=" << Convert::ToString(seqval);
+	if (type == DbQueryInsert && query.Table == "notifications" && query.NotificationInsertID) {
+		DbReference seqval = GetSequenceValue(GetTablePrefix() + query.Table, "notification_id");
+		query.NotificationInsertID->SetValue(static_cast<long>(seqval));
 	}
 }
 
 void IdoPgsqlConnection::CleanUpExecuteQuery(const String& table, const String& time_column, double max_age)
 {
-	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalCleanUpExecuteQuery, this, table, time_column, max_age), true);
+	m_QueryQueue.Enqueue(boost::bind(&IdoPgsqlConnection::InternalCleanUpExecuteQuery, this, table, time_column, max_age), PriorityLow, true);
 }
 
 void IdoPgsqlConnection::InternalCleanUpExecuteQuery(const String& table, const String& time_column, double max_age)
@@ -796,7 +915,7 @@ void IdoPgsqlConnection::InternalCleanUpExecuteQuery(const String& table, const
 
 void IdoPgsqlConnection::FillIDCache(const DbType::Ptr& type)
 {
-	String query = "SELECT " + type->GetIDColumn() + " AS object_id, " + type->GetTable() + "_id FROM " + GetTablePrefix() + type->GetTable() + "s";
+	String query = "SELECT " + type->GetIDColumn() + " AS object_id, " + type->GetTable() + "_id, config_hash FROM " + GetTablePrefix() + type->GetTable() + "s";
 	IdoPgsqlResult result = Query(query);
 
 	Dictionary::Ptr row;
@@ -804,7 +923,9 @@ void IdoPgsqlConnection::FillIDCache(const DbType::Ptr& type)
 	int index = 0;
 	while ((row = FetchRow(result, index))) {
 		index++;
-		SetInsertID(type, DbReference(row->Get("object_id")), DbReference(row->Get(type->GetTable() + "_id")));
+		DbReference dbref(row->Get("object_id"));
+		SetInsertID(type, dbref, DbReference(row->Get(type->GetTable() + "_id")));
+		SetConfigHash(type, dbref, row->Get("config_hash"));
 	}
 }
 
