@@ -26,6 +26,7 @@
 #include "icinga/eventcommand.hpp"
 #include "icinga/notificationcommand.hpp"
 #include "remote/apiaction.hpp"
+#include "remote/apilistener.hpp"
 #include "remote/httputility.hpp"
 #include "base/utility.hpp"
 #include "base/convert.hpp"
@@ -45,6 +46,7 @@ REGISTER_APIACTION(schedule_downtime, "Service;Host", &ApiActions::ScheduleDownt
 REGISTER_APIACTION(remove_downtime, "Service;Host;Downtime", &ApiActions::RemoveDowntime);
 REGISTER_APIACTION(shutdown_process, "", &ApiActions::ShutdownProcess);
 REGISTER_APIACTION(restart_process, "", &ApiActions::RestartProcess);
+REGISTER_APIACTION(generate_ticket, "", &ApiActions::GenerateTicket);
 
 Dictionary::Ptr ApiActions::CreateResult(int code, const String& status,
     const Dictionary::Ptr& additional)
@@ -156,7 +158,7 @@ Dictionary::Ptr ApiActions::SendCustomNotification(const ConfigObject::Ptr& obje
 		checkable->SetForceNextNotification(true);
 
 	Checkable::OnNotificationsRequested(checkable, NotificationCustom, checkable->GetLastCheckResult(),
-	    HttpUtility::GetLastParameter(params, "author"), HttpUtility::GetLastParameter(params, "comment"));
+	    HttpUtility::GetLastParameter(params, "author"), HttpUtility::GetLastParameter(params, "comment"), MessageOrigin::Ptr());
 
 	return ApiActions::CreateResult(200, "Successfully sent custom notification for object '" + checkable->GetName() + "'.");
 }
@@ -172,7 +174,7 @@ Dictionary::Ptr ApiActions::DelayNotification(const ConfigObject::Ptr& object,
 	if (!params->Contains("timestamp"))
 		return ApiActions::CreateResult(403, "A timestamp is required to delay notifications");
 
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		notification->SetNextNotification(HttpUtility::GetLastParameter(params, "timestamp"));
 	}
 
@@ -273,7 +275,7 @@ Dictionary::Ptr ApiActions::RemoveComment(const ConfigObject::Ptr& object,
 	if (checkable) {
 		std::set<Comment::Ptr> comments = checkable->GetComments();
 
-		BOOST_FOREACH(const Comment::Ptr& comment, comments) {
+		for (const Comment::Ptr& comment : comments) {
 			Comment::RemoveComment(comment->GetName());
 		}
 
@@ -301,23 +303,33 @@ Dictionary::Ptr ApiActions::ScheduleDowntime(const ConfigObject::Ptr& object,
 		return ApiActions::CreateResult(404, "Can't schedule downtime for non-existent object.");
 
 	if (!params->Contains("start_time") || !params->Contains("end_time") ||
-	    !params->Contains("duration") || !params->Contains("author") ||
-	    !params->Contains("comment")) {
+	    !params->Contains("author") || !params->Contains("comment")) {
 
-		return ApiActions::CreateResult(404, "Options 'start_time', 'end_time', 'duration', 'author' and 'comment' are required");
+		return ApiActions::CreateResult(404, "Options 'start_time', 'end_time', 'author' and 'comment' are required");
 	}
 
 	bool fixed = true;
 	if (params->Contains("fixed"))
 		fixed = HttpUtility::GetLastParameter(params, "fixed");
 
-	String downtimeName = Downtime::AddDowntime(checkable,
-	    HttpUtility::GetLastParameter(params, "author"),
-	    HttpUtility::GetLastParameter(params, "comment"),
-	    HttpUtility::GetLastParameter(params, "start_time"),
-	    HttpUtility::GetLastParameter(params, "end_time"), fixed,
-	    HttpUtility::GetLastParameter(params, "trigger_name"),
-	    HttpUtility::GetLastParameter(params, "duration"));
+	if (!fixed && !params->Contains("duration"))
+		return ApiActions::CreateResult(404, "Option 'duration' is required for flexible downtime");
+
+	double duration = 0.0;
+	if (params->Contains("duration"))
+		duration = HttpUtility::GetLastParameter(params, "duration");
+
+	String triggerName;
+	if (params->Contains("trigger_name"))
+		triggerName = HttpUtility::GetLastParameter(params, "trigger_name");
+
+	String author = HttpUtility::GetLastParameter(params, "author");
+	String comment = HttpUtility::GetLastParameter(params, "comment");
+	double startTime = HttpUtility::GetLastParameter(params, "start_time");
+	double endTime = HttpUtility::GetLastParameter(params, "end_time");
+
+	String downtimeName = Downtime::AddDowntime(checkable, author, comment, startTime, endTime,
+	    fixed, triggerName, duration);
 
 	Downtime::Ptr downtime = Downtime::GetByName(downtimeName);
 
@@ -325,6 +337,44 @@ Dictionary::Ptr ApiActions::ScheduleDowntime(const ConfigObject::Ptr& object,
 	additional->Set("name", downtimeName);
 	additional->Set("legacy_id", downtime->GetLegacyId());
 
+	/* Schedule downtime for all child objects. */
+	int childOptions = 0;
+	if (params->Contains("child_options"))
+		childOptions = HttpUtility::GetLastParameter(params, "child_options");
+
+	if (childOptions > 0) {
+		/* '1' schedules child downtimes triggered by the parent downtime.
+		 * '2' schedules non-triggered downtimes for all children.
+		 */
+		if (childOptions == 1)
+			triggerName = downtimeName;
+
+		Array::Ptr childDowntimes = new Array();
+
+		Log(LogCritical, "ApiActions")
+		    << "Processing child options " << childOptions << " for downtime " << downtimeName;
+
+		for (const Checkable::Ptr& child : checkable->GetAllChildren()) {
+			Log(LogCritical, "ApiActions")
+			   << "Scheduling downtime for child object " << child->GetName();
+
+			String childDowntimeName = Downtime::AddDowntime(child, author, comment, startTime, endTime,
+			    fixed, triggerName, duration);
+
+			Log(LogCritical, "ApiActions")
+			    << "Add child downtime '" << childDowntimeName << "'.";
+
+			Downtime::Ptr childDowntime = Downtime::GetByName(childDowntimeName);
+
+			Dictionary::Ptr additionalChild = new Dictionary();
+			additionalChild->Set("name", childDowntimeName);
+			additionalChild->Set("legacy_id", childDowntime->GetLegacyId());
+			childDowntimes->Add(additionalChild);
+		}
+
+		additional->Set("child_downtimes", childDowntimes);
+	}
+
 	return ApiActions::CreateResult(200, "Successfully scheduled downtime '" +
 	     downtimeName + "' for object '" + checkable->GetName() + "'.", additional);
 }
@@ -337,7 +387,7 @@ Dictionary::Ptr ApiActions::RemoveDowntime(const ConfigObject::Ptr& object,
 	if (checkable) {
 		std::set<Downtime::Ptr> downtimes = checkable->GetDowntimes();
 
-		BOOST_FOREACH(const Downtime::Ptr& downtime, downtimes) {
+		for (const Downtime::Ptr& downtime : downtimes) {
 			Downtime::RemoveDowntime(downtime->GetName(), true);
 		}
 
@@ -372,3 +422,25 @@ Dictionary::Ptr ApiActions::RestartProcess(const ConfigObject::Ptr& object,
 	return ApiActions::CreateResult(200, "Restarting Icinga 2.");
 }
 
+Dictionary::Ptr ApiActions::GenerateTicket(const ConfigObject::Ptr&,
+    const Dictionary::Ptr& params)
+{
+	if (!params->Contains("cn"))
+		return ApiActions::CreateResult(404, "Option 'cn' is required");
+
+	String cn = HttpUtility::GetLastParameter(params, "cn");
+
+	ApiListener::Ptr listener = ApiListener::GetInstance();
+	String salt = listener->GetTicketSalt();
+
+	if (salt.IsEmpty())
+		return ApiActions::CreateResult(500, "Ticket salt is not configured in ApiListener object");
+
+	String ticket = PBKDF2_SHA1(cn, salt, 50000);
+
+	Dictionary::Ptr additional = new Dictionary();
+	additional->Set("ticket", ticket);
+
+	return ApiActions::CreateResult(200, "Generated PKI ticket '" + ticket + "' for common name '"
+	    + cn + "'.", additional);
+}
