@@ -44,6 +44,7 @@ void ApiEvents::StaticInitialize(void)
 
 	Downtime::OnDowntimeAdded.connect(&ApiEvents::DowntimeAddedHandler);
 	Downtime::OnDowntimeRemoved.connect(&ApiEvents::DowntimeRemovedHandler);
+	Downtime::OnDowntimeStarted.connect(&ApiEvents::DowntimeStartedHandler);
 	Downtime::OnDowntimeTriggered.connect(&ApiEvents::DowntimeTriggeredHandler);
 }
 
@@ -70,7 +71,7 @@ void ApiEvents::CheckResultHandler(const Checkable::Ptr& checkable, const CheckR
 
 	result->Set("check_result", Serialize(cr));
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
@@ -100,14 +101,14 @@ void ApiEvents::StateChangeHandler(const Checkable::Ptr& checkable, const CheckR
 	result->Set("state_type", checkable->GetStateType());
 	result->Set("check_result", Serialize(cr));
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
 
 void ApiEvents::NotificationSentToAllUsersHandler(const Notification::Ptr& notification,
     const Checkable::Ptr& checkable, const std::set<User::Ptr>& users, NotificationType type,
-    const CheckResult::Ptr& cr, const String& author, const String& text)
+    const CheckResult::Ptr& cr, const String& author, const String& text, const MessageOrigin::Ptr& origin)
 {
 	std::vector<EventQueue::Ptr> queues = EventQueue::GetQueuesForType("Notification");
 
@@ -130,7 +131,7 @@ void ApiEvents::NotificationSentToAllUsersHandler(const Notification::Ptr& notif
 
 	Array::Ptr userNames = new Array();
 
-	BOOST_FOREACH(const User::Ptr& user, users) {
+	for (const User::Ptr& user : users) {
 		userNames->Add(user->GetName());
 	}
 
@@ -140,7 +141,7 @@ void ApiEvents::NotificationSentToAllUsersHandler(const Notification::Ptr& notif
 	result->Set("text", text);
 	result->Set("check_result", Serialize(cr));
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
@@ -170,7 +171,7 @@ void ApiEvents::FlappingChangedHandler(const Checkable::Ptr& checkable, const Me
 	result->Set("state_type", checkable->GetStateType());
 	result->Set("is_flapping", checkable->IsFlapping());
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
@@ -207,7 +208,7 @@ void ApiEvents::AcknowledgementSetHandler(const Checkable::Ptr& checkable,
 	result->Set("notify", notify);
 	result->Set("expiry", expiry);
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
@@ -236,7 +237,7 @@ void ApiEvents::AcknowledgementClearedHandler(const Checkable::Ptr& checkable, c
 	result->Set("state", service ? static_cast<int>(service->GetState()) : static_cast<int>(host->GetState()));
 	result->Set("state_type", checkable->GetStateType());
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 
@@ -258,7 +259,7 @@ void ApiEvents::CommentAddedHandler(const Comment::Ptr& comment)
 
 	result->Set("comment", Serialize(comment, FAConfig | FAState));
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
@@ -278,7 +279,7 @@ void ApiEvents::CommentRemovedHandler(const Comment::Ptr& comment)
 
 	result->Set("comment", Serialize(comment, FAConfig | FAState));
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
@@ -298,7 +299,7 @@ void ApiEvents::DowntimeAddedHandler(const Downtime::Ptr& downtime)
 
 	result->Set("downtime", Serialize(downtime, FAConfig | FAState));
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
@@ -318,7 +319,27 @@ void ApiEvents::DowntimeRemovedHandler(const Downtime::Ptr& downtime)
 
 	result->Set("downtime", Serialize(downtime, FAConfig | FAState));
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
+		queue->ProcessEvent(result);
+	}
+}
+
+void ApiEvents::DowntimeStartedHandler(const Downtime::Ptr& downtime)
+{
+	std::vector<EventQueue::Ptr> queues = EventQueue::GetQueuesForType("DowntimeStarted");
+
+	if (queues.empty())
+		return;
+
+	Log(LogDebug, "ApiEvents", "Processing event type 'DowntimeStarted'.");
+
+	Dictionary::Ptr result = new Dictionary();
+	result->Set("type", "DowntimeStarted");
+	result->Set("timestamp", Utility::GetTime());
+
+	result->Set("downtime", Serialize(downtime, FAConfig | FAState));
+
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
@@ -336,9 +357,9 @@ void ApiEvents::DowntimeTriggeredHandler(const Downtime::Ptr& downtime)
 	result->Set("type", "DowntimeTriggered");
 	result->Set("timestamp", Utility::GetTime());
 
-	result->Set("downtime", Serialize(downtime));
+	result->Set("downtime", Serialize(downtime, FAConfig | FAState));
 
-	BOOST_FOREACH(const EventQueue::Ptr& queue, queues) {
+	for (const EventQueue::Ptr& queue : queues) {
 		queue->ProcessEvent(result);
 	}
 }
