@@ -24,20 +24,19 @@
 #include "base/exception.hpp"
 #include "base/context.hpp"
 #include "base/convert.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
 boost::signals2::signal<void (const Notification::Ptr&, const Checkable::Ptr&, const std::set<User::Ptr>&,
-    const NotificationType&, const CheckResult::Ptr&, const String&, const String&)> Checkable::OnNotificationSentToAllUsers;
-boost::signals2::signal<void (const Notification::Ptr&, const Checkable::Ptr&, const std::set<User::Ptr>&,
-    const NotificationType&, const CheckResult::Ptr&, const String&, const String&)> Checkable::OnNotificationSendStart;
+    const NotificationType&, const CheckResult::Ptr&, const String&, const String&,
+    const MessageOrigin::Ptr&)> Checkable::OnNotificationSentToAllUsers;
 boost::signals2::signal<void (const Notification::Ptr&, const Checkable::Ptr&, const User::Ptr&,
-    const NotificationType&, const CheckResult::Ptr&, const String&, const String&, const String&)> Checkable::OnNotificationSentToUser;
+    const NotificationType&, const CheckResult::Ptr&, const String&, const String&, const String&,
+    const MessageOrigin::Ptr&)> Checkable::OnNotificationSentToUser;
 
 void Checkable::ResetNotificationNumbers(void)
 {
-	BOOST_FOREACH(const Notification::Ptr& notification, GetNotifications()) {
+	for (const Notification::Ptr& notification : GetNotifications()) {
 		ObjectLock olock(notification);
 		notification->ResetNotificationNumber();
 	}
@@ -49,14 +48,14 @@ void Checkable::SendNotifications(NotificationType type, const CheckResult::Ptr&
 
 	bool force = GetForceNextNotification();
 
+	SetForceNextNotification(false);
+
 	if (!IcingaApplication::GetInstance()->GetEnableNotifications() || !GetEnableNotifications()) {
 		if (!force) {
 			Log(LogInformation, "Checkable")
 			    << "Notifications are disabled for service '" << GetName() << "'.";
 			return;
 		}
-
-		SetForceNextNotification(false);
 	}
 
 	Log(LogInformation, "Checkable")
@@ -71,9 +70,10 @@ void Checkable::SendNotifications(NotificationType type, const CheckResult::Ptr&
 	Log(LogDebug, "Checkable")
 	    << "Checkable '" << GetName() << "' has " << notifications.size() << " notification(s).";
 
-	BOOST_FOREACH(const Notification::Ptr& notification, notifications) {
+	for (const Notification::Ptr& notification : notifications) {
 		try {
-			notification->BeginExecuteNotification(type, cr, force, author, text);
+			if (!notification->IsPaused())
+				notification->BeginExecuteNotification(type, cr, force, false, author, text);
 		} catch (const std::exception& ex) {
 			Log(LogWarning, "Checkable")
 			    << "Exception occured during notification for service '"
