@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -24,22 +24,52 @@
 #include "base/objectlock.hpp"
 #include "base/utility.hpp"
 #include "base/exception.hpp"
-#include <boost/foreach.hpp>
-#include <boost/bind/apply.hpp>
 
 using namespace icinga;
 
-REGISTER_TYPE(Checkable);
+REGISTER_TYPE_WITH_PROTOTYPE(Checkable, Checkable::GetPrototype());
+INITIALIZE_ONCE(&Checkable::StaticInitialize);
 
 boost::signals2::signal<void (const Checkable::Ptr&, const String&, const String&, AcknowledgementType, bool, double, const MessageOrigin::Ptr&)> Checkable::OnAcknowledgementSet;
 boost::signals2::signal<void (const Checkable::Ptr&, const MessageOrigin::Ptr&)> Checkable::OnAcknowledgementCleared;
 
+void Checkable::StaticInitialize(void)
+{
+	/* fixed downtime start */
+	Downtime::OnDowntimeStarted.connect(boost::bind(&Checkable::NotifyFixedDowntimeStart, _1));
+	/* flexible downtime start */
+	Downtime::OnDowntimeTriggered.connect(boost::bind(&Checkable::NotifyFlexibleDowntimeStart, _1));
+	/* fixed/flexible downtime end */
+	Downtime::OnDowntimeRemoved.connect(boost::bind(&Checkable::NotifyDowntimeEnd, _1));
+}
+
 Checkable::Checkable(void)
 	: m_CheckRunning(false)
 {
 	SetSchedulingOffset(Utility::Random());
 }
 
+void Checkable::OnAllConfigLoaded(void)
+{
+	ObjectImpl<Checkable>::OnAllConfigLoaded();
+
+	Endpoint::Ptr endpoint = GetCommandEndpoint();
+
+	if (endpoint) {
+		Zone::Ptr checkableZone = static_pointer_cast<Zone>(GetZone());
+
+		if (!checkableZone)
+			checkableZone = Zone::GetLocalZone();
+
+		Zone::Ptr cmdZone = endpoint->GetZone();
+
+		if (checkableZone && cmdZone != checkableZone && cmdZone->GetParent() != checkableZone) {
+			BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("command_endpoint"),
+			    "Command endpoint must be in zone '" + checkableZone->GetName() + "' or in a direct child zone thereof."));
+		}
+	}
+}
+
 void Checkable::Start(bool runtimeCreated)
 {
 	double now = Utility::GetTime();
@@ -97,8 +127,8 @@ void Checkable::AcknowledgeProblem(const String& author, const String& comment,
 	SetAcknowledgementRaw(type);
 	SetAcknowledgementExpiry(expiry);
 
-	if (notify)
-		OnNotificationsRequested(this, NotificationAcknowledgement, GetLastCheckResult(), author, comment);
+	if (notify && !IsPaused())
+		OnNotificationsRequested(this, NotificationAcknowledgement, GetLastCheckResult(), author, comment, MessageOrigin::Ptr());
 
 	OnAcknowledgementSet(this, author, comment, type, notify, expiry, origin);
 }
@@ -116,6 +146,42 @@ Endpoint::Ptr Checkable::GetCommandEndpoint(void) const
 	return Endpoint::GetByName(GetCommandEndpointRaw());
 }
 
+void Checkable::NotifyFixedDowntimeStart(const Downtime::Ptr& downtime)
+{
+	if (!downtime->GetFixed())
+		return;
+
+	NotifyDowntimeInternal(downtime);
+}
+
+void Checkable::NotifyFlexibleDowntimeStart(const Downtime::Ptr& downtime)
+{
+	if (downtime->GetFixed())
+		return;
+
+	NotifyDowntimeInternal(downtime);
+}
+
+void Checkable::NotifyDowntimeInternal(const Downtime::Ptr& downtime)
+{
+	Checkable::Ptr checkable = downtime->GetCheckable();
+
+	if (!checkable->IsPaused())
+		OnNotificationsRequested(checkable, NotificationDowntimeStart, checkable->GetLastCheckResult(), downtime->GetAuthor(), downtime->GetComment(), MessageOrigin::Ptr());
+}
+
+void Checkable::NotifyDowntimeEnd(const Downtime::Ptr& downtime)
+{
+	/* don't send notifications for flexible downtimes which never triggered */
+	if (!downtime->GetFixed() && !downtime->IsTriggered())
+		return;
+
+	Checkable::Ptr checkable = downtime->GetCheckable();
+
+	if (!checkable->IsPaused())
+		OnNotificationsRequested(checkable, NotificationDowntimeEnd, checkable->GetLastCheckResult(), downtime->GetAuthor(), downtime->GetComment(), MessageOrigin::Ptr());
+}
+
 void Checkable::ValidateCheckInterval(double value, const ValidationUtils& utils)
 {
 	ObjectImpl<Checkable>::ValidateCheckInterval(value, utils);
