@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -36,3 +36,18 @@ void CheckResult::StaticInitialize(void)
 	ScriptGlobal::Set("HostUp", HostUp);
 	ScriptGlobal::Set("HostDown", HostDown);
 }
+
+double CheckResult::CalculateExecutionTime(void) const
+{
+	return GetExecutionEnd() - GetExecutionStart();
+}
+
+double CheckResult::CalculateLatency(void) const
+{
+	double latency = (GetScheduleEnd() - GetScheduleStart()) - CalculateExecutionTime();
+
+	if (latency < 0)
+		latency = 0;
+
+	return latency;
+}
