@@ -25,7 +25,6 @@
 #include "base/utility.hpp"
 #include "base/configtype.hpp"
 #include "base/statsfunction.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -80,8 +79,9 @@ CheckableCheckStatistics CIB::CalculateHostCheckStats(void)
 	int count_latency = 0;
 	double min_execution_time = -1, max_execution_time = 0, sum_execution_time = 0;
 	int count_execution_time = 0;
+	bool checkresult = false;
 
-	BOOST_FOREACH(const Host::Ptr& host, ConfigType::GetObjectsByType<Host>()) {
+	for (const Host::Ptr& host : ConfigType::GetObjectsByType<Host>()) {
 		ObjectLock olock(host);
 
 		CheckResult::Ptr cr = host->GetLastCheckResult();
@@ -89,6 +89,9 @@ CheckableCheckStatistics CIB::CalculateHostCheckStats(void)
 		if (!cr)
 			continue;
 
+		/* set to true, we have a checkresult */
+		checkresult = true;
+
 		/* latency */
 		double latency = cr->CalculateLatency();
 
@@ -114,6 +117,11 @@ CheckableCheckStatistics CIB::CalculateHostCheckStats(void)
 		count_execution_time++;
 	}
 
+	if (!checkresult) {
+		min_latency = 0;
+		min_execution_time = 0;
+	}
+
 	CheckableCheckStatistics ccs;
 
 	ccs.min_latency = min_latency;
@@ -132,8 +140,9 @@ CheckableCheckStatistics CIB::CalculateServiceCheckStats(void)
 	int count_latency = 0;
 	double min_execution_time = -1, max_execution_time = 0, sum_execution_time = 0;
 	int count_execution_time = 0;
+	bool checkresult = false;
 
-	BOOST_FOREACH(const Service::Ptr& service, ConfigType::GetObjectsByType<Service>()) {
+	for (const Service::Ptr& service : ConfigType::GetObjectsByType<Service>()) {
 		ObjectLock olock(service);
 
 		CheckResult::Ptr cr = service->GetLastCheckResult();
@@ -141,6 +150,9 @@ CheckableCheckStatistics CIB::CalculateServiceCheckStats(void)
 		if (!cr)
 			continue;
 
+		/* set to true, we have a checkresult */
+		checkresult = true;
+
 		/* latency */
 		double latency = cr->CalculateLatency();
 
@@ -166,6 +178,11 @@ CheckableCheckStatistics CIB::CalculateServiceCheckStats(void)
 		count_execution_time++;
 	}
 
+	if (!checkresult) {
+		min_latency = 0;
+		min_execution_time = 0;
+	}
+
 	CheckableCheckStatistics ccs;
 
 	ccs.min_latency = min_latency;
@@ -180,9 +197,9 @@ CheckableCheckStatistics CIB::CalculateServiceCheckStats(void)
 
 ServiceStatistics CIB::CalculateServiceStats(void)
 {
-	ServiceStatistics ss = {0};
+	ServiceStatistics ss = {};
 
-	BOOST_FOREACH(const Service::Ptr& service, ConfigType::GetObjectsByType<Service>()) {
+	for (const Service::Ptr& service : ConfigType::GetObjectsByType<Service>()) {
 		ObjectLock olock(service);
 
 		CheckResult::Ptr cr = service->GetLastCheckResult();
@@ -214,9 +231,9 @@ ServiceStatistics CIB::CalculateServiceStats(void)
 
 HostStatistics CIB::CalculateHostStats(void)
 {
-	HostStatistics hs = {0};
+	HostStatistics hs = {};
 
-	BOOST_FOREACH(const Host::Ptr& host, ConfigType::GetObjectsByType<Host>()) {
+	for (const Host::Ptr& host : ConfigType::GetObjectsByType<Host>()) {
 		ObjectLock olock(host);
 
 		if (host->IsReachable()) {
@@ -251,13 +268,8 @@ std::pair<Dictionary::Ptr, Array::Ptr> CIB::GetFeatureStats(void)
 	Array::Ptr perfdata = new Array();
 
 	String name;
-	BOOST_FOREACH(tie(name, boost::tuples::ignore), StatsFunctionRegistry::GetInstance()->GetItems()) {
-		StatsFunction::Ptr func = StatsFunctionRegistry::GetInstance()->GetItem(name);
-
-		if (!func)
-			BOOST_THROW_EXCEPTION(std::invalid_argument("Function '" + name + "' does not exist."));
-
-		func->Invoke(status, perfdata);
+	for (const auto& kv : StatsFunctionRegistry::GetInstance()->GetItems()) {
+		kv.second->Invoke(status, perfdata);
 	}
 
 	return std::make_pair(status, perfdata);
@@ -294,8 +306,8 @@ void CIB::StatsFunc(const Dictionary::Ptr& status, const Array::Ptr& perfdata) {
 	status->Set("min_latency", scs.min_latency);
 	status->Set("max_latency", scs.max_latency);
 	status->Set("avg_latency", scs.avg_latency);
-	status->Set("min_execution_time", scs.min_latency);
-	status->Set("max_execution_time", scs.max_latency);
+	status->Set("min_execution_time", scs.min_execution_time);
+	status->Set("max_execution_time", scs.max_execution_time);
 	status->Set("avg_execution_time", scs.avg_execution_time);
 
 	ServiceStatistics ss = CalculateServiceStats();
