@@ -20,13 +20,13 @@
 #include "icinga/compatutility.hpp"
 #include "icinga/checkcommand.hpp"
 #include "icinga/eventcommand.hpp"
+#include "icinga/notificationcommand.hpp"
 #include "icinga/pluginutility.hpp"
 #include "icinga/service.hpp"
 #include "base/utility.hpp"
 #include "base/configtype.hpp"
 #include "base/objectlock.hpp"
 #include "base/convert.hpp"
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/algorithm/string/join.hpp>
 
@@ -42,8 +42,7 @@ String CompatUtility::GetCommandLine(const Command::Ptr& command)
 		Array::Ptr args = commandLine;
 
 		ObjectLock olock(args);
-		String arg;
-		BOOST_FOREACH(arg, args) {
+		for (const String& arg : args) {
 			// This is obviously incorrect for non-trivial cases.
 			result += " \"" + EscapeString(arg) + "\"";
 		}
@@ -62,11 +61,11 @@ String CompatUtility::GetCommandNamePrefix(const Command::Ptr command)
 		return Empty;
 
 	String prefix;
-	if (command->GetType() == ConfigType::GetByName("CheckCommand"))
+	if (command->GetReflectionType() == CheckCommand::TypeInstance)
 		prefix = "check_";
-	else if (command->GetType() == ConfigType::GetByName("NotificationCommand"))
+	else if (command->GetReflectionType() == NotificationCommand::TypeInstance)
 		prefix = "notification_";
-	else if (command->GetType() == ConfigType::GetByName("EventCommand"))
+	else if (command->GetReflectionType() == EventCommand::TypeInstance)
 		prefix = "event_";
 
 	return prefix;
@@ -109,8 +108,8 @@ int CompatUtility::GetHostNotifyOnDown(const Host::Ptr& host)
 {
 	unsigned long notification_state_filter = GetCheckableNotificationStateFilter(host);
 
-	if (notification_state_filter & (1<<ServiceCritical) ||
-	    notification_state_filter & (1<<ServiceWarning))
+	if ((notification_state_filter & ServiceCritical) ||
+	    (notification_state_filter & ServiceWarning))
 		return 1;
 
 	return 0;
@@ -120,7 +119,7 @@ int CompatUtility::GetHostNotifyOnUnreachable(const Host::Ptr& host)
 {
 	unsigned long notification_state_filter = GetCheckableNotificationStateFilter(host);
 
-	if (notification_state_filter & (1<<ServiceUnknown))
+	if (notification_state_filter & ServiceUnknown)
 		return 1;
 
 	return 0;
@@ -143,7 +142,7 @@ String CompatUtility::GetCheckableCommandArgs(const Checkable::Ptr& checkable)
 
 		if (command_vars) {
 			ObjectLock olock(command_vars);
-			BOOST_FOREACH(const Dictionary::Pair& kv, command_vars) {
+			for (const Dictionary::Pair& kv : command_vars) {
 				String macro = "$" + kv.first + "$"; // this is too simple
 				if (command_line.Contains(macro))
 					args->Set(kv.first, kv.second);
@@ -155,7 +154,7 @@ String CompatUtility::GetCheckableCommandArgs(const Checkable::Ptr& checkable)
 
 		if (host_vars) {
 			ObjectLock olock(host_vars);
-			BOOST_FOREACH(const Dictionary::Pair& kv, host_vars) {
+			for (const Dictionary::Pair& kv : host_vars) {
 				String macro = "$" + kv.first + "$"; // this is too simple
 				if (command_line.Contains(macro))
 					args->Set(kv.first, kv.second);
@@ -170,7 +169,7 @@ String CompatUtility::GetCheckableCommandArgs(const Checkable::Ptr& checkable)
 
 			if (service_vars) {
 				ObjectLock olock(service_vars);
-				BOOST_FOREACH(const Dictionary::Pair& kv, service_vars) {
+				for (const Dictionary::Pair& kv : service_vars) {
 					String macro = "$" + kv.first + "$"; // this is too simple
 					if (command_line.Contains(macro))
 						args->Set(kv.first, kv.second);
@@ -183,7 +182,7 @@ String CompatUtility::GetCheckableCommandArgs(const Checkable::Ptr& checkable)
 
 		String arg_string;
 		ObjectLock olock(args);
-		BOOST_FOREACH(const Dictionary::Pair& kv, args) {
+		for (const Dictionary::Pair& kv : args) {
 			arg_string += Convert::ToString(kv.first) + "=" + Convert::ToString(kv.second) + "!";
 		}
 		return arg_string;
@@ -353,18 +352,14 @@ int CompatUtility::GetCheckableInCheckPeriod(const Checkable::Ptr& checkable)
 
 int CompatUtility::GetCheckableInNotificationPeriod(const Checkable::Ptr& checkable)
 {
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
-		ObjectLock olock(notification);
-
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		TimePeriod::Ptr timeperiod = notification->GetPeriod();
 
-		/* first notification wins */
-		if (timeperiod)
-			return (timeperiod->IsInside(Utility::GetTime()) ? 1 : 0);
+		if (!timeperiod || timeperiod->IsInside(Utility::GetTime()))
+			return 1;
 	}
 
-	/* none set means always notified */
-	return 1;
+	return 0;
 }
 
 /* vars attr */
@@ -397,7 +392,7 @@ int CompatUtility::GetCheckableNotificationsEnabled(const Checkable::Ptr& checka
 int CompatUtility::GetCheckableNotificationLastNotification(const Checkable::Ptr& checkable)
 {
 	double last_notification = 0.0;
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		if (notification->GetLastNotification() > last_notification)
 			last_notification = notification->GetLastNotification();
 	}
@@ -408,7 +403,7 @@ int CompatUtility::GetCheckableNotificationLastNotification(const Checkable::Ptr
 int CompatUtility::GetCheckableNotificationNextNotification(const Checkable::Ptr& checkable)
 {
 	double next_notification = 0.0;
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		if (next_notification == 0 || notification->GetNextNotification() < next_notification)
 			next_notification = notification->GetNextNotification();
 	}
@@ -419,7 +414,7 @@ int CompatUtility::GetCheckableNotificationNextNotification(const Checkable::Ptr
 int CompatUtility::GetCheckableNotificationNotificationNumber(const Checkable::Ptr& checkable)
 {
 	int notification_number = 0;
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		if (notification->GetNotificationNumber() > notification_number)
 			notification_number = notification->GetNotificationNumber();
 	}
@@ -431,7 +426,7 @@ double CompatUtility::GetCheckableNotificationNotificationInterval(const Checkab
 {
 	double notification_interval = -1;
 
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		if (notification_interval == -1 || notification->GetInterval() < notification_interval)
 			notification_interval = notification->GetInterval();
 	}
@@ -442,22 +437,6 @@ double CompatUtility::GetCheckableNotificationNotificationInterval(const Checkab
 	return notification_interval / 60.0;
 }
 
-String CompatUtility::GetCheckableNotificationNotificationPeriod(const Checkable::Ptr& checkable)
-{
-	TimePeriod::Ptr notification_period;
-
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
-
-		if (notification->GetPeriod())
-			notification_period = notification->GetPeriod();
-	}
-
-	if (!notification_period)
-		return Empty;
-
-	return notification_period->GetName();
-}
-
 String CompatUtility::GetCheckableNotificationNotificationOptions(const Checkable::Ptr& checkable)
 {
 
@@ -468,41 +447,41 @@ String CompatUtility::GetCheckableNotificationNotificationOptions(const Checkabl
 	unsigned long notification_type_filter = 0;
 	unsigned long notification_state_filter = 0;
 
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
-		notification_type_filter = notification->GetTypeFilter();
-		notification_state_filter = notification->GetStateFilter();
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
+		notification_type_filter |= notification->GetTypeFilter();
+		notification_state_filter |= notification->GetStateFilter();
 	}
 
 	std::vector<String> notification_options;
 
 	/* notification state filters */
 	if (service) {
-		if (notification_state_filter & (1<<ServiceWarning)) {
+		if (notification_state_filter & ServiceWarning) {
 			notification_options.push_back("w");
 		}
-		if (notification_state_filter & (1<<ServiceUnknown)) {
+		if (notification_state_filter & ServiceUnknown) {
 			notification_options.push_back("u");
 		}
-		if (notification_state_filter & (1<<ServiceCritical)) {
+		if (notification_state_filter & ServiceCritical) {
 			notification_options.push_back("c");
 		}
 	} else {
-		if (notification_state_filter & (1<<HostDown)) {
+		if (notification_state_filter & HostDown) {
 			notification_options.push_back("d");
 		}
 	}
 
 	/* notification type filters */
-	if (notification_type_filter & (1<<NotificationRecovery)) {
+	if (notification_type_filter & NotificationRecovery) {
 		notification_options.push_back("r");
 	}
-	if (notification_type_filter & (1<<NotificationFlappingStart) ||
-			notification_type_filter & (1<<NotificationFlappingEnd)) {
+	if ((notification_type_filter & NotificationFlappingStart) ||
+	    (notification_type_filter & NotificationFlappingEnd)) {
 		notification_options.push_back("f");
 	}
-	if (notification_type_filter & (1<<NotificationDowntimeStart) ||
-			notification_type_filter & (1<<NotificationDowntimeEnd) ||
-			notification_type_filter & (1<<NotificationDowntimeRemoved)) {
+	if ((notification_type_filter & NotificationDowntimeStart) ||
+	    (notification_type_filter & NotificationDowntimeEnd) ||
+	    (notification_type_filter & NotificationDowntimeRemoved)) {
 		notification_options.push_back("s");
 	}
 
@@ -513,10 +492,10 @@ int CompatUtility::GetCheckableNotificationTypeFilter(const Checkable::Ptr& chec
 {
 	unsigned long notification_type_filter = 0;
 
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		ObjectLock olock(notification);
 
-		notification_type_filter = notification->GetTypeFilter();
+		notification_type_filter |= notification->GetTypeFilter();
 	}
 
 	return notification_type_filter;
@@ -526,10 +505,10 @@ int CompatUtility::GetCheckableNotificationStateFilter(const Checkable::Ptr& che
 {
 	unsigned long notification_state_filter = 0;
 
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		ObjectLock olock(notification);
 
-		notification_state_filter = notification->GetStateFilter();
+		notification_state_filter |= notification->GetStateFilter();
 	}
 
 	return notification_state_filter;
@@ -537,7 +516,7 @@ int CompatUtility::GetCheckableNotificationStateFilter(const Checkable::Ptr& che
 
 int CompatUtility::GetCheckableNotifyOnWarning(const Checkable::Ptr& checkable)
 {
-	if (GetCheckableNotificationStateFilter(checkable) & (1<<ServiceWarning))
+	if (GetCheckableNotificationStateFilter(checkable) & ServiceWarning)
 		return 1;
 
 	return 0;
@@ -545,7 +524,7 @@ int CompatUtility::GetCheckableNotifyOnWarning(const Checkable::Ptr& checkable)
 
 int CompatUtility::GetCheckableNotifyOnCritical(const Checkable::Ptr& checkable)
 {
-	if (GetCheckableNotificationStateFilter(checkable) & (1<<ServiceCritical))
+	if (GetCheckableNotificationStateFilter(checkable) & ServiceCritical)
 		return 1;
 
 	return 0;
@@ -553,7 +532,7 @@ int CompatUtility::GetCheckableNotifyOnCritical(const Checkable::Ptr& checkable)
 
 int CompatUtility::GetCheckableNotifyOnUnknown(const Checkable::Ptr& checkable)
 {
-	if (GetCheckableNotificationStateFilter(checkable) & (1<<ServiceUnknown))
+	if (GetCheckableNotificationStateFilter(checkable) & ServiceUnknown)
 		return 1;
 
 	return 0;
@@ -561,7 +540,7 @@ int CompatUtility::GetCheckableNotifyOnUnknown(const Checkable::Ptr& checkable)
 
 int CompatUtility::GetCheckableNotifyOnRecovery(const Checkable::Ptr& checkable)
 {
-	if (GetCheckableNotificationTypeFilter(checkable) & (1<<NotificationRecovery))
+	if (GetCheckableNotificationTypeFilter(checkable) & NotificationRecovery)
 		return 1;
 
 	return 0;
@@ -571,8 +550,8 @@ int CompatUtility::GetCheckableNotifyOnFlapping(const Checkable::Ptr& checkable)
 {
 	unsigned long notification_type_filter = GetCheckableNotificationTypeFilter(checkable);
 
-	if (notification_type_filter & (1<<NotificationFlappingStart) ||
-	    notification_type_filter & (1<<NotificationFlappingEnd))
+	if ((notification_type_filter & NotificationFlappingStart) ||
+	    (notification_type_filter & NotificationFlappingEnd))
 		return 1;
 
 	return 0;
@@ -582,9 +561,9 @@ int CompatUtility::GetCheckableNotifyOnDowntime(const Checkable::Ptr& checkable)
 {
 	unsigned long notification_type_filter = GetCheckableNotificationTypeFilter(checkable);
 
-	if (notification_type_filter & (1<<NotificationDowntimeStart) ||
-	    notification_type_filter & (1<<NotificationDowntimeEnd) ||
-	    notification_type_filter & (1<<NotificationDowntimeRemoved))
+	if ((notification_type_filter & NotificationDowntimeStart) ||
+	    (notification_type_filter & NotificationDowntimeEnd) ||
+	    (notification_type_filter & NotificationDowntimeRemoved))
 		return 1;
 
 	return 0;
@@ -596,14 +575,14 @@ std::set<User::Ptr> CompatUtility::GetCheckableNotificationUsers(const Checkable
 	std::set<User::Ptr> allUsers;
 	std::set<User::Ptr> users;
 
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		ObjectLock olock(notification);
 
 		users = notification->GetUsers();
 
 		std::copy(users.begin(), users.end(), std::inserter(allUsers, allUsers.begin()));
 
-		BOOST_FOREACH(const UserGroup::Ptr& ug, notification->GetUserGroups()) {
+		for (const UserGroup::Ptr& ug : notification->GetUserGroups()) {
 			std::set<User::Ptr> members = ug->GetMembers();
 			std::copy(members.begin(), members.end(), std::inserter(allUsers, allUsers.begin()));
 		}
@@ -616,10 +595,10 @@ std::set<UserGroup::Ptr> CompatUtility::GetCheckableNotificationUserGroups(const
 {
 	std::set<UserGroup::Ptr> usergroups;
 	/* Service -> Notifications -> UserGroups */
-	BOOST_FOREACH(const Notification::Ptr& notification, checkable->GetNotifications()) {
+	for (const Notification::Ptr& notification : checkable->GetNotifications()) {
 		ObjectLock olock(notification);
 
-		BOOST_FOREACH(const UserGroup::Ptr& ug, notification->GetUserGroups()) {
+		for (const UserGroup::Ptr& ug : notification->GetUserGroups()) {
 			usergroups.insert(ug);
 		}
 	}
