@@ -22,7 +22,6 @@
 #include "icinga/service.hpp"
 #include "base/logger.hpp"
 #include "base/exception.hpp"
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/classification.hpp>
 
@@ -77,7 +76,7 @@ void Dependency::OnConfigLoaded(void)
 	else
 		defaultFilter = StateFilterOK | StateFilterWarning;
 
-	SetStateFilter(FilterArrayToInt(GetStates(), defaultFilter));
+	SetStateFilter(FilterArrayToInt(GetStates(), Notification::GetStateFilterMap(), defaultFilter));
 }
 
 void Dependency::OnAllConfigLoaded(void)
@@ -87,15 +86,10 @@ void Dependency::OnAllConfigLoaded(void)
 	Host::Ptr childHost = Host::GetByName(GetChildHostName());
 
 	if (childHost) {
-		if (GetChildServiceName().IsEmpty()) {
-			Log(LogDebug, "Dependency")
-			    << "Dependency '" << GetName() << "' child host '" << GetChildHostName() << ".";
+		if (GetChildServiceName().IsEmpty())
 			m_Child = childHost;
-		} else {
-			Log(LogDebug, "Dependency")
-			    << "Dependency '" << GetName() << "' child host '" << GetChildHostName() << "' service '" << GetChildServiceName() << "' .";
+		else
 			m_Child = childHost->GetServiceByShortName(GetChildServiceName());
-		}
 	}
 	
 	if (!m_Child)
@@ -106,15 +100,10 @@ void Dependency::OnAllConfigLoaded(void)
 	Host::Ptr parentHost = Host::GetByName(GetParentHostName());
 
 	if (parentHost) {
-		if (GetParentServiceName().IsEmpty()) {
-			Log(LogDebug, "Dependency")
-			    << "Dependency '" << GetName() << "' parent host '" << GetParentHostName() << ".";
+		if (GetParentServiceName().IsEmpty())
 			m_Parent = parentHost;
-		} else {
-			Log(LogDebug, "Dependency")
-			    << "Dependency '" << GetName() << "' parent host '" << GetParentHostName() << "' service '" << GetParentServiceName() << "' .";
+		else
 			m_Parent = parentHost->GetServiceByShortName(GetParentServiceName());
-		}
 	}
 	
 	if (!m_Parent)
@@ -224,7 +213,7 @@ void Dependency::ValidateStates(const Array::Ptr& value, const ValidationUtils&
 {
 	ObjectImpl<Dependency>::ValidateStates(value, utils);
 
-	int sfilter = FilterArrayToInt(value, 0);
+	int sfilter = FilterArrayToInt(value, Notification::GetStateFilterMap(), 0);
 
 	if (GetParentServiceName().IsEmpty() && (sfilter & ~(StateFilterUp | StateFilterDown)) != 0)
 		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("states"), "State filter is invalid for host dependency."));
