@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -20,11 +20,11 @@
 #include "icinga/downtime.hpp"
 #include "icinga/downtime.tcpp"
 #include "icinga/host.hpp"
+#include "icinga/scheduleddowntime.hpp"
 #include "remote/configobjectutility.hpp"
 #include "base/configtype.hpp"
 #include "base/utility.hpp"
 #include "base/timer.hpp"
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/classification.hpp>
 
@@ -34,9 +34,11 @@ static int l_NextDowntimeID = 1;
 static boost::mutex l_DowntimeMutex;
 static std::map<int, String> l_LegacyDowntimesCache;
 static Timer::Ptr l_DowntimesExpireTimer;
+static Timer::Ptr l_DowntimesStartTimer;
 
 boost::signals2::signal<void (const Downtime::Ptr&)> Downtime::OnDowntimeAdded;
 boost::signals2::signal<void (const Downtime::Ptr&)> Downtime::OnDowntimeRemoved;
+boost::signals2::signal<void (const Downtime::Ptr&)> Downtime::OnDowntimeStarted;
 boost::signals2::signal<void (const Downtime::Ptr&)> Downtime::OnDowntimeTriggered;
 
 INITIALIZE_ONCE(&Downtime::StaticInitialize);
@@ -45,6 +47,11 @@ REGISTER_TYPE(Downtime);
 
 void Downtime::StaticInitialize(void)
 {
+	l_DowntimesStartTimer = new Timer();
+	l_DowntimesStartTimer->SetInterval(5);
+	l_DowntimesStartTimer->OnTimerExpired.connect(boost::bind(&Downtime::DowntimesStartTimerHandler));
+	l_DowntimesStartTimer->Start();
+
 	l_DowntimesExpireTimer = new Timer();
 	l_DowntimesExpireTimer->SetInterval(60);
 	l_DowntimesExpireTimer->OnTimerExpired.connect(boost::bind(&Downtime::DowntimesExpireTimerHandler));
@@ -150,7 +157,7 @@ Checkable::Ptr Downtime::GetCheckable(void) const
 	return static_pointer_cast<Checkable>(m_Checkable);
 }
 
-bool Downtime::IsActive(void) const
+bool Downtime::IsInEffect(void) const
 {
 	double now = Utility::GetTime();
 
@@ -166,7 +173,7 @@ bool Downtime::IsActive(void) const
 	if (triggerTime == 0)
 		return false;
 
-	return (triggerTime + GetDuration() < now);
+	return (now < triggerTime + GetDuration());
 }
 
 bool Downtime::IsTriggered(void) const
@@ -180,7 +187,26 @@ bool Downtime::IsTriggered(void) const
 
 bool Downtime::IsExpired(void) const
 {
-	return (GetEndTime() < Utility::GetTime());
+	double now = Utility::GetTime();
+
+	if (GetFixed())
+		return (GetEndTime() < now);
+	else {
+		/* triggered flexible downtime not in effect anymore */
+		if (IsTriggered() && !IsInEffect())
+			return true;
+		/* flexible downtime never triggered */
+		else if (!IsTriggered() && (GetEndTime() < now))
+			return true;
+		else
+			return false;
+	}
+}
+
+bool Downtime::HasValidConfigOwner(void) const
+{
+	String configOwner = GetConfigOwner();
+	return configOwner.IsEmpty() || GetObject<ScheduledDowntime>(configOwner);
 }
 
 int Downtime::GetNextDowntimeID(void)
@@ -214,6 +240,7 @@ String Downtime::AddDowntime(const Checkable::Ptr& checkable, const String& auth
 	attrs->Set("triggered_by", triggeredBy);
 	attrs->Set("scheduled_by", scheduledBy);
 	attrs->Set("config_owner", scheduledDowntime);
+	attrs->Set("entry_time", Utility::GetTime());
 
 	Host::Ptr host;
 	Service::Ptr service;
@@ -223,13 +250,18 @@ String Downtime::AddDowntime(const Checkable::Ptr& checkable, const String& auth
 	if (service)
 		attrs->Set("service_name", service->GetShortName());
 
+	String zone = checkable->GetZoneName();
+
+	if (!zone.IsEmpty())
+		attrs->Set("zone", zone);
+
 	String config = ConfigObjectUtility::CreateObjectConfig(Downtime::TypeInstance, fullName, true, Array::Ptr(), attrs);
 
 	Array::Ptr errors = new Array();
 
 	if (!ConfigObjectUtility::CreateObject(Downtime::TypeInstance, fullName, config, errors)) {
 		ObjectLock olock(errors);
-		BOOST_FOREACH(const String& error, errors) {
+		for (const String& error : errors) {
 			Log(LogCritical, "Downtime", error);
 		}
 
@@ -237,21 +269,24 @@ String Downtime::AddDowntime(const Checkable::Ptr& checkable, const String& auth
 	}
 
 	if (!triggeredBy.IsEmpty()) {
-		Downtime::Ptr triggerDowntime = Downtime::GetByName(triggeredBy);
-		Array::Ptr triggers = triggerDowntime->GetTriggers();
+		Downtime::Ptr parentDowntime = Downtime::GetByName(triggeredBy);
+		Array::Ptr triggers = parentDowntime->GetTriggers();
 
-		if (!triggers->Contains(triggeredBy))
-			triggers->Add(triggeredBy);
+		ObjectLock olock(triggers);
+		if (!triggers->Contains(fullName))
+			triggers->Add(fullName);
 	}
 
 	Downtime::Ptr downtime = Downtime::GetByName(fullName);
 
+	if (!downtime)
+		BOOST_THROW_EXCEPTION(std::runtime_error("Could not create downtime object."));
+
 	Log(LogNotice, "Downtime")
 	    << "Added downtime '" << downtime->GetName()
 	    << "' between '" << Utility::FormatDateTime("%Y-%m-%d %H:%M:%S", startTime)
 	    << "' and '" << Utility::FormatDateTime("%Y-%m-%d %H:%M:%S", endTime) << "'.";
 
-
 	return fullName;
 }
 
@@ -275,11 +310,14 @@ void Downtime::RemoveDowntime(const String& id, bool cancelled, bool expired, co
 	Log(LogNotice, "Downtime")
 	    << "Removed downtime '" << downtime->GetName() << "' from object '" << downtime->GetCheckable()->GetName() << "'.";
 
+	if (downtime->GetPackage() != "_api")
+		return;
+
 	Array::Ptr errors = new Array();
 
 	if (!ConfigObjectUtility::DeleteObject(downtime, false, errors)) {
 		ObjectLock olock(errors);
-		BOOST_FOREACH(const String& error, errors) {
+		for (const String& error : errors) {
 			Log(LogCritical, "Downtime", error);
 		}
 
@@ -287,27 +325,26 @@ void Downtime::RemoveDowntime(const String& id, bool cancelled, bool expired, co
 	}
 }
 
-void Downtime::TriggerDowntime(void)
+bool Downtime::CanBeTriggered(void)
 {
-	if (IsActive() && IsTriggered()) {
-		Log(LogDebug, "Downtime")
-		    << "Not triggering downtime '" << GetName() << "': already triggered.";
-		return;
-	}
+	if (IsInEffect() && IsTriggered())
+		return false;
 
-	if (IsExpired()) {
-		Log(LogDebug, "Downtime")
-		    << "Not triggering downtime '" << GetName() << "': expired.";
-		return;
-	}
+	if (IsExpired())
+		return false;
 
 	double now = Utility::GetTime();
 
-	if (now < GetStartTime() || now > GetEndTime()) {
-		Log(LogDebug, "Downtime")
-		    << "Not triggering downtime '" << GetName() << "': current time is outside downtime window.";
+	if (now < GetStartTime() || now > GetEndTime())
+		return false;
+
+	return true;
+}
+
+void Downtime::TriggerDowntime(void)
+{
+	if (!CanBeTriggered())
 		return;
-	}
 
 	Log(LogNotice, "Downtime")
 		<< "Triggering downtime '" << GetName() << "'.";
@@ -319,7 +356,7 @@ void Downtime::TriggerDowntime(void)
 
 	{
 		ObjectLock olock(triggers);
-		BOOST_FOREACH(const String& triggerName, triggers) {
+		for (const String& triggerName : triggers) {
 			Downtime::Ptr downtime = Downtime::GetByName(triggerName);
 
 			if (!downtime)
@@ -336,7 +373,7 @@ String Downtime::GetDowntimeIDFromLegacyID(int id)
 {
 	boost::mutex::scoped_lock lock(l_DowntimeMutex);
 
-	std::map<int, String>::iterator it = l_LegacyDowntimesCache.find(id);
+	auto it = l_LegacyDowntimesCache.find(id);
 
 	if (it == l_LegacyDowntimesCache.end())
 		return Empty;
@@ -344,21 +381,38 @@ String Downtime::GetDowntimeIDFromLegacyID(int id)
 	return it->second;
 }
 
+void Downtime::DowntimesStartTimerHandler(void)
+{
+	/* Start fixed downtimes. Flexible downtimes will be triggered on-demand. */
+	for (const Downtime::Ptr& downtime : ConfigType::GetObjectsByType<Downtime>()) {
+		if (downtime->IsActive() &&
+		    downtime->CanBeTriggered() &&
+		    downtime->GetFixed()) {
+			/* Send notifications. */
+			OnDowntimeStarted(downtime);
+
+			/* Trigger fixed downtime immediately. */
+			downtime->TriggerDowntime();
+		}
+	}
+}
+
 void Downtime::DowntimesExpireTimerHandler(void)
 {
 	std::vector<Downtime::Ptr> downtimes;
 
-	BOOST_FOREACH(const Downtime::Ptr& downtime, ConfigType::GetObjectsByType<Downtime>()) {
+	for (const Downtime::Ptr& downtime : ConfigType::GetObjectsByType<Downtime>()) {
 		downtimes.push_back(downtime);
 	}
 
-	BOOST_FOREACH(const Downtime::Ptr& downtime, downtimes) {
-		if (downtime->IsExpired())
+	for (const Downtime::Ptr& downtime : downtimes) {
+		/* Only remove downtimes which are activated after daemon start. */
+		if (downtime->IsActive() && (downtime->IsExpired() || !downtime->HasValidConfigOwner()))
 			RemoveDowntime(downtime->GetName(), false, true);
 	}
 }
 
-void Downtime::ValidateStartTime(double value, const ValidationUtils& utils)
+void Downtime::ValidateStartTime(const Timestamp& value, const ValidationUtils& utils)
 {
 	ObjectImpl<Downtime>::ValidateStartTime(value, utils);
 
@@ -366,7 +420,7 @@ void Downtime::ValidateStartTime(double value, const ValidationUtils& utils)
 		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("start_time"), "Start time must be greater than 0."));
 }
 
-void Downtime::ValidateEndTime(double value, const ValidationUtils& utils)
+void Downtime::ValidateEndTime(const Timestamp& value, const ValidationUtils& utils)
 {
 	ObjectImpl<Downtime>::ValidateEndTime(value, utils);
 
