@@ -41,6 +41,7 @@ public:
 
 	static boost::signals2::signal<void (const Downtime::Ptr&)> OnDowntimeAdded;
 	static boost::signals2::signal<void (const Downtime::Ptr&)> OnDowntimeRemoved;
+	static boost::signals2::signal<void (const Downtime::Ptr&)> OnDowntimeStarted;
 	static boost::signals2::signal<void (const Downtime::Ptr&)> OnDowntimeTriggered;
 
 	intrusive_ptr<Checkable> GetCheckable(void) const;
@@ -48,6 +49,7 @@ public:
 	bool IsInEffect(void) const;
 	bool IsTriggered(void) const;
 	bool IsExpired(void) const;
+	bool HasValidConfigOwner(void) const;
 
 	static int GetNextDowntimeID(void);
 
@@ -70,12 +72,15 @@ protected:
 	virtual void Start(bool runtimeCreated) override;
 	virtual void Stop(bool runtimeRemoved) override;
 
-	virtual void ValidateStartTime(double value, const ValidationUtils& utils) override;
-	virtual void ValidateEndTime(double value, const ValidationUtils& utils) override;
+	virtual void ValidateStartTime(const Timestamp& value, const ValidationUtils& utils) override;
+	virtual void ValidateEndTime(const Timestamp& value, const ValidationUtils& utils) override;
 
 private:
 	ObjectImpl<Checkable>::Ptr m_Checkable;
 
+	bool CanBeTriggered(void);
+
+	static void DowntimesStartTimerHandler(void);
 	static void DowntimesExpireTimerHandler(void);
 };
 
