@@ -38,7 +38,6 @@
 #include "base/exception.hpp"
 #include <fstream>
 #include <boost/algorithm/string/classification.hpp>
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/split.hpp>
 
 using namespace icinga;
@@ -115,8 +114,7 @@ void ExternalCommandProcessor::Execute(double time, const String& command, const
 	{
 		boost::mutex::scoped_lock lock(GetMutex());
 
-		std::map<String, ExternalCommandInfo>::iterator it;
-		it = GetCommands().find(command);
+		auto it = GetCommands().find(command);
 
 		if (it == GetCommands().end())
 			BOOST_THROW_EXCEPTION(std::invalid_argument("The external command '" + command + "' does not exist."));
@@ -195,6 +193,8 @@ void ExternalCommandProcessor::StaticInitialize(void)
 	RegisterCommand("SCHEDULE_SVC_DOWNTIME", &ExternalCommandProcessor::ScheduleSvcDowntime, 9);
 	RegisterCommand("DEL_SVC_DOWNTIME", &ExternalCommandProcessor::DelSvcDowntime, 1);
 	RegisterCommand("SCHEDULE_HOST_DOWNTIME", &ExternalCommandProcessor::ScheduleHostDowntime, 8);
+	RegisterCommand("SCHEDULE_AND_PROPAGATE_HOST_DOWNTIME", &ExternalCommandProcessor::ScheduleAndPropagateHostDowntime, 8);
+	RegisterCommand("SCHEDULE_AND_PROPAGATE_TRIGGERED_HOST_DOWNTIME", &ExternalCommandProcessor::ScheduleAndPropagateTriggeredHostDowntime, 8);
 	RegisterCommand("DEL_HOST_DOWNTIME", &ExternalCommandProcessor::DelHostDowntime, 1);
 	RegisterCommand("DEL_DOWNTIME_BY_HOST_NAME", &ExternalCommandProcessor::DelDowntimeByHostName, 1, 4);
 	RegisterCommand("SCHEDULE_HOST_SVC_DOWNTIME", &ExternalCommandProcessor::ScheduleHostSvcDowntime, 8);
@@ -545,7 +545,7 @@ void ExternalCommandProcessor::ScheduleForcedHostSvcChecks(double, const std::ve
 	if (!host)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot reschedule forced host service checks for non-existent host '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Rescheduling next check for service '" << service->GetName() << "'";
 
@@ -569,7 +569,7 @@ void ExternalCommandProcessor::ScheduleHostSvcChecks(double, const std::vector<S
 	if (planned_check < Utility::GetTime())
 		planned_check = Utility::GetTime();
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (planned_check > service->GetNextCheck()) {
 			Log(LogNotice, "ExternalCommandProcessor")
 			    << "Ignoring reschedule request for service '"
@@ -594,7 +594,7 @@ void ExternalCommandProcessor::EnableHostSvcChecks(double, const std::vector<Str
 	if (!host)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable host service checks for non-existent host '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Enabling active checks for service '" << service->GetName() << "'";
 
@@ -609,7 +609,7 @@ void ExternalCommandProcessor::DisableHostSvcChecks(double, const std::vector<St
 	if (!host)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable host service checks for non-existent host '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Disabling active checks for service '" << service->GetName() << "'";
 
@@ -741,8 +741,8 @@ void ExternalCommandProcessor::EnableHostgroupSvcChecks(double, const std::vecto
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable hostgroup service checks for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			Log(LogNotice, "ExternalCommandProcessor")
 			    << "Enabling active checks for service '" << service->GetName() << "'";
 
@@ -758,8 +758,8 @@ void ExternalCommandProcessor::DisableHostgroupSvcChecks(double, const std::vect
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable hostgroup service checks for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			Log(LogNotice, "ExternalCommandProcessor")
 			    << "Disabling active checks for service '" << service->GetName() << "'";
 
@@ -775,7 +775,7 @@ void ExternalCommandProcessor::EnableServicegroupSvcChecks(double, const std::ve
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable servicegroup service checks for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Enabling active checks for service '" << service->GetName() << "'";
 
@@ -790,7 +790,7 @@ void ExternalCommandProcessor::DisableServicegroupSvcChecks(double, const std::v
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable servicegroup service checks for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Disabling active checks for service '" << service->GetName() << "'";
 
@@ -857,7 +857,7 @@ void ExternalCommandProcessor::EnableServicegroupPassiveSvcChecks(double, const
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable servicegroup passive service checks for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Enabling passive checks for service '" << service->GetName() << "'";
 
@@ -872,7 +872,7 @@ void ExternalCommandProcessor::DisableServicegroupPassiveSvcChecks(double, const
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable servicegroup passive service checks for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Disabling passive checks for service '" << service->GetName() << "'";
 
@@ -887,8 +887,8 @@ void ExternalCommandProcessor::EnableHostgroupPassiveSvcChecks(double, const std
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable hostgroup passive service checks for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			Log(LogNotice, "ExternalCommandProcessor")
 			    << "Enabling passive checks for service '" << service->GetName() << "'";
 
@@ -904,8 +904,8 @@ void ExternalCommandProcessor::DisableHostgroupPassiveSvcChecks(double, const st
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable hostgroup passive service checks for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			Log(LogNotice, "ExternalCommandProcessor")
 			    << "Disabling passive checks for service '" << service->GetName() << "'";
 
@@ -1003,6 +1003,78 @@ void ExternalCommandProcessor::ScheduleHostDowntime(double, const std::vector<St
 	    Convert::ToBool(is_fixed), triggeredBy, Convert::ToDouble(arguments[5]));
 }
 
+void ExternalCommandProcessor::ScheduleAndPropagateHostDowntime(double, const std::vector<String>& arguments)
+{
+	Host::Ptr host = Host::GetByName(arguments[0]);
+
+	if (!host)
+		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot schedule and propagate host downtime for non-existent host '" + arguments[0] + "'"));
+
+	String triggeredBy;
+	int triggeredByLegacy = Convert::ToLong(arguments[4]);
+	int is_fixed = Convert::ToLong(arguments[3]);
+	if (triggeredByLegacy != 0)
+		triggeredBy = Downtime::GetDowntimeIDFromLegacyID(triggeredByLegacy);
+
+	Log(LogNotice, "ExternalCommandProcessor")
+	    << "Creating downtime for host " << host->GetName();
+
+	(void) Downtime::AddDowntime(host, arguments[6], arguments[7],
+	    Convert::ToDouble(arguments[1]), Convert::ToDouble(arguments[2]),
+	    Convert::ToBool(is_fixed), triggeredBy, Convert::ToDouble(arguments[5]));
+
+	/* Schedule downtime for all child hosts */
+	for (const Checkable::Ptr& child : host->GetAllChildren()) {
+		Host::Ptr host;
+		Service::Ptr service;
+		tie(host, service) = GetHostService(child);
+
+		/* ignore all service children */
+		if (service)
+			continue;
+
+		(void) Downtime::AddDowntime(child, arguments[6], arguments[7],
+		    Convert::ToDouble(arguments[1]), Convert::ToDouble(arguments[2]),
+		    Convert::ToBool(is_fixed), triggeredBy, Convert::ToDouble(arguments[5]));
+	}
+}
+
+void ExternalCommandProcessor::ScheduleAndPropagateTriggeredHostDowntime(double, const std::vector<String>& arguments)
+{
+	Host::Ptr host = Host::GetByName(arguments[0]);
+
+	if (!host)
+		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot schedule and propagate triggered host downtime for non-existent host '" + arguments[0] + "'"));
+
+	String triggeredBy;
+	int triggeredByLegacy = Convert::ToLong(arguments[4]);
+	int is_fixed = Convert::ToLong(arguments[3]);
+	if (triggeredByLegacy != 0)
+		triggeredBy = Downtime::GetDowntimeIDFromLegacyID(triggeredByLegacy);
+
+	Log(LogNotice, "ExternalCommandProcessor")
+	    << "Creating downtime for host " << host->GetName();
+
+	String parentDowntime = Downtime::AddDowntime(host, arguments[6], arguments[7],
+	    Convert::ToDouble(arguments[1]), Convert::ToDouble(arguments[2]),
+	    Convert::ToBool(is_fixed), triggeredBy, Convert::ToDouble(arguments[5]));
+
+	/* Schedule downtime for all child hosts and explicitely trigger them through the parent host's downtime */
+	for (const Checkable::Ptr& child : host->GetAllChildren()) {
+		Host::Ptr host;
+		Service::Ptr service;
+		tie(host, service) = GetHostService(child);
+
+		/* ignore all service children */
+		if (service)
+			continue;
+
+		(void) Downtime::AddDowntime(child, arguments[6], arguments[7],
+		    Convert::ToDouble(arguments[1]), Convert::ToDouble(arguments[2]),
+		    Convert::ToBool(is_fixed), parentDowntime, Convert::ToDouble(arguments[5]));
+	}
+}
+
 void ExternalCommandProcessor::DelHostDowntime(double, const std::vector<String>& arguments)
 {
 	int id = Convert::ToLong(arguments[0]);
@@ -1035,18 +1107,18 @@ void ExternalCommandProcessor::DelDowntimeByHostName(double, const std::vector<S
 		Log(LogWarning, "ExternalCommandProcessor")
 		    << ("Ignoring additional parameters for host '" + arguments[0] + "' downtime deletion.");
 
-	BOOST_FOREACH(const Downtime::Ptr& downtime, host->GetDowntimes()) {
+	for (const Downtime::Ptr& downtime : host->GetDowntimes()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Removing downtime '" << downtime->GetName() << "'.";
 
 		Downtime::RemoveDowntime(downtime->GetName(), true);
 	}
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (!serviceName.IsEmpty() && serviceName != service->GetName())
 			continue;
 
-		BOOST_FOREACH(const Downtime::Ptr& downtime, service->GetDowntimes()) {
+		for (const Downtime::Ptr& downtime : service->GetDowntimes()) {
 			if (!startTime.IsEmpty() && downtime->GetStartTime() != Convert::ToDouble(startTime))
 				continue;
 
@@ -1081,7 +1153,7 @@ void ExternalCommandProcessor::ScheduleHostSvcDowntime(double, const std::vector
 	    Convert::ToDouble(arguments[1]), Convert::ToDouble(arguments[2]),
 	    Convert::ToBool(is_fixed), triggeredBy, Convert::ToDouble(arguments[5]));
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Creating downtime for service " << service->GetName();
 		(void) Downtime::AddDowntime(service, arguments[6], arguments[7],
@@ -1103,7 +1175,7 @@ void ExternalCommandProcessor::ScheduleHostgroupHostDowntime(double, const std::
 	if (triggeredByLegacy != 0)
 		triggeredBy = Downtime::GetDowntimeIDFromLegacyID(triggeredByLegacy);
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    <<  "Creating downtime for host " << host->GetName();
 
@@ -1132,13 +1204,13 @@ void ExternalCommandProcessor::ScheduleHostgroupSvcDowntime(double, const std::v
 
 	std::set<Service::Ptr> services;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			services.insert(service);
 		}
 	}
 
-	BOOST_FOREACH(const Service::Ptr& service, services) {
+	for (const Service::Ptr& service : services) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Creating downtime for service " << service->GetName();
 		(void) Downtime::AddDowntime(service, arguments[6], arguments[7],
@@ -1166,12 +1238,12 @@ void ExternalCommandProcessor::ScheduleServicegroupHostDowntime(double, const st
 
 	std::set<Host::Ptr> hosts;
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Host::Ptr host = service->GetHost();
 		hosts.insert(host);
 	}
 
-	BOOST_FOREACH(const Host::Ptr& host, hosts) {
+	for (const Host::Ptr& host : hosts) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Creating downtime for host " << host->GetName();
 		(void) Downtime::AddDowntime(host, arguments[6], arguments[7],
@@ -1193,7 +1265,7 @@ void ExternalCommandProcessor::ScheduleServicegroupSvcDowntime(double, const std
 	if (triggeredByLegacy != 0)
 		triggeredBy = Downtime::GetDowntimeIDFromLegacyID(triggeredByLegacy);
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Creating downtime for service " << service->GetName();
 		(void) Downtime::AddDowntime(service, arguments[6], arguments[7],
@@ -1290,7 +1362,8 @@ void ExternalCommandProcessor::SendCustomHostNotification(double, const std::vec
 		host->SetForceNextNotification(true);
 	}
 
-	Checkable::OnNotificationsRequested(host, NotificationCustom, host->GetLastCheckResult(), arguments[2], arguments[3]);
+	Checkable::OnNotificationsRequested(host, NotificationCustom,
+	    host->GetLastCheckResult(), arguments[2], arguments[3], MessageOrigin::Ptr());
 }
 
 void ExternalCommandProcessor::SendCustomSvcNotification(double, const std::vector<String>& arguments)
@@ -1309,7 +1382,8 @@ void ExternalCommandProcessor::SendCustomSvcNotification(double, const std::vect
 		service->SetForceNextNotification(true);
 	}
 
-	Service::OnNotificationsRequested(service, NotificationCustom, service->GetLastCheckResult(), arguments[3], arguments[4]);
+	Service::OnNotificationsRequested(service, NotificationCustom,
+	    service->GetLastCheckResult(), arguments[3], arguments[4], MessageOrigin::Ptr());
 }
 
 void ExternalCommandProcessor::DelayHostNotification(double, const std::vector<String>& arguments)
@@ -1322,7 +1396,7 @@ void ExternalCommandProcessor::DelayHostNotification(double, const std::vector<S
 	Log(LogNotice, "ExternalCommandProcessor")
 	    << "Delaying notifications for host '" << host->GetName() << "'";
 
-	BOOST_FOREACH(const Notification::Ptr& notification, host->GetNotifications()) {
+	for (const Notification::Ptr& notification : host->GetNotifications()) {
 		notification->SetNextNotification(Convert::ToDouble(arguments[1]));
 	}
 }
@@ -1337,7 +1411,7 @@ void ExternalCommandProcessor::DelaySvcNotification(double, const std::vector<St
 	Log(LogNotice, "ExternalCommandProcessor")
 	    << "Delaying notifications for service " << service->GetName();
 
-	BOOST_FOREACH(const Notification::Ptr& notification, service->GetNotifications()) {
+	for (const Notification::Ptr& notification : service->GetNotifications()) {
 		notification->SetNextNotification(Convert::ToDouble(arguments[2]));
 	}
 }
@@ -1404,7 +1478,7 @@ void ExternalCommandProcessor::EnableHostSvcNotifications(double, const std::vec
 	Log(LogNotice, "ExternalCommandProcessor")
 	    << "Enabling notifications for all services on host '" << arguments[0] << "'";
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Enabling notifications for service '" << service->GetName() << "'";
 
@@ -1422,7 +1496,7 @@ void ExternalCommandProcessor::DisableHostSvcNotifications(double, const std::ve
 	Log(LogNotice, "ExternalCommandProcessor")
 	    << "Disabling notifications for all services on host '" << arguments[0] << "'";
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Disabling notifications for service '" << service->GetName() << "'";
 
@@ -1437,7 +1511,7 @@ void ExternalCommandProcessor::DisableHostgroupHostChecks(double, const std::vec
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable hostgroup host checks for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Disabling active checks for host '" << host->GetName() << "'";
 
@@ -1452,7 +1526,7 @@ void ExternalCommandProcessor::DisableHostgroupPassiveHostChecks(double, const s
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable hostgroup passive host checks for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Disabling passive checks for host '" << host->GetName() << "'";
 
@@ -1467,7 +1541,7 @@ void ExternalCommandProcessor::DisableServicegroupHostChecks(double, const std::
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable servicegroup host checks for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Host::Ptr host = service->GetHost();
 
 		Log(LogNotice, "ExternalCommandProcessor")
@@ -1484,7 +1558,7 @@ void ExternalCommandProcessor::DisableServicegroupPassiveHostChecks(double, cons
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable servicegroup passive host checks for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Host::Ptr host = service->GetHost();
 
 		Log(LogNotice, "ExternalCommandProcessor")
@@ -1501,7 +1575,7 @@ void ExternalCommandProcessor::EnableHostgroupHostChecks(double, const std::vect
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable hostgroup host checks for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Enabling active checks for host '" << host->GetName() << "'";
 
@@ -1516,7 +1590,7 @@ void ExternalCommandProcessor::EnableHostgroupPassiveHostChecks(double, const st
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable hostgroup passive host checks for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Enabling passive checks for host '" << host->GetName() << "'";
 
@@ -1531,7 +1605,7 @@ void ExternalCommandProcessor::EnableServicegroupHostChecks(double, const std::v
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable servicegroup host checks for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Host::Ptr host = service->GetHost();
 
 		Log(LogNotice, "ExternalCommandProcessor")
@@ -1548,7 +1622,7 @@ void ExternalCommandProcessor::EnableServicegroupPassiveHostChecks(double, const
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable servicegroup passive host checks for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Host::Ptr host = service->GetHost();
 
 		Log(LogNotice, "ExternalCommandProcessor")
@@ -2043,7 +2117,7 @@ void ExternalCommandProcessor::EnableHostgroupHostNotifications(double, const st
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable host notifications for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Enabling notifications for host '" << host->GetName() << "'";
 
@@ -2058,8 +2132,8 @@ void ExternalCommandProcessor::EnableHostgroupSvcNotifications(double, const std
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable service notifications for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			Log(LogNotice, "ExternalCommandProcessor")
 			    << "Enabling notifications for service '" << service->GetName() << "'";
 
@@ -2075,7 +2149,7 @@ void ExternalCommandProcessor::DisableHostgroupHostNotifications(double, const s
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable host notifications for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Disabling notifications for host '" << host->GetName() << "'";
 
@@ -2090,8 +2164,8 @@ void ExternalCommandProcessor::DisableHostgroupSvcNotifications(double, const st
 	if (!hg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable service notifications for non-existent hostgroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			Log(LogNotice, "ExternalCommandProcessor")
 			    << "Disabling notifications for service '" << service->GetName() << "'";
 
@@ -2107,7 +2181,7 @@ void ExternalCommandProcessor::EnableServicegroupHostNotifications(double, const
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable host notifications for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Host::Ptr host = service->GetHost();
 
 		Log(LogNotice, "ExternalCommandProcessor")
@@ -2124,7 +2198,7 @@ void ExternalCommandProcessor::EnableServicegroupSvcNotifications(double, const
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot enable service notifications for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Enabling notifications for service '" << service->GetName() << "'";
 
@@ -2139,7 +2213,7 @@ void ExternalCommandProcessor::DisableServicegroupHostNotifications(double, cons
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable host notifications for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Host::Ptr host = service->GetHost();
 
 		Log(LogNotice, "ExternalCommandProcessor")
@@ -2156,7 +2230,7 @@ void ExternalCommandProcessor::DisableServicegroupSvcNotifications(double, const
 	if (!sg)
 		BOOST_THROW_EXCEPTION(std::invalid_argument("Cannot disable service notifications for non-existent servicegroup '" + arguments[0] + "'"));
 
-	BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+	for (const Service::Ptr& service : sg->GetMembers()) {
 		Log(LogNotice, "ExternalCommandProcessor")
 		    << "Disabling notifications for service '" << service->GetName() << "'";
 
