@@ -82,6 +82,8 @@ private:
 	static void ScheduleSvcDowntime(double time, const std::vector<String>& arguments);
 	static void DelSvcDowntime(double time, const std::vector<String>& arguments);
 	static void ScheduleHostDowntime(double time, const std::vector<String>& arguments);
+	static void ScheduleAndPropagateHostDowntime(double, const std::vector<String>& arguments);
+	static void ScheduleAndPropagateTriggeredHostDowntime(double, const std::vector<String>& arguments);
 	static void DelHostDowntime(double time, const std::vector<String>& arguments);
 	static void DelDowntimeByHostName(double, const std::vector<String>& arguments);
 	static void ScheduleHostSvcDowntime(double time, const std::vector<String>& arguments);
