@@ -28,7 +28,6 @@
 #include "base/utility.hpp"
 #include "base/debug.hpp"
 #include "base/json.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -56,7 +55,7 @@ void Host::OnAllConfigLoaded(void)
 
 		ObjectLock olock(groups);
 
-		BOOST_FOREACH(const String& name, groups) {
+		for (const String& name : groups) {
 			HostGroup::Ptr hg = HostGroup::GetByName(name);
 
 			if (hg)
@@ -67,16 +66,16 @@ void Host::OnAllConfigLoaded(void)
 
 void Host::CreateChildObjects(const Type::Ptr& childType)
 {
-	if (childType->GetName() == "ScheduledDowntime")
+	if (childType == ScheduledDowntime::TypeInstance)
 		ScheduledDowntime::EvaluateApplyRules(this);
 
-	if (childType->GetName() == "Notification")
+	if (childType == Notification::TypeInstance)
 		Notification::EvaluateApplyRules(this);
 
-	if (childType->GetName() == "Dependency")
+	if (childType == Dependency::TypeInstance)
 		Dependency::EvaluateApplyRules(this);
 
-	if (childType->GetName() == "Service")
+	if (childType == Service::TypeInstance)
 		Service::EvaluateApplyRules(this);
 }
 
@@ -89,7 +88,7 @@ void Host::Stop(bool runtimeRemoved)
 	if (groups) {
 		ObjectLock olock(groups);
 
-		BOOST_FOREACH(const String& name, groups) {
+		for (const String& name : groups) {
 			HostGroup::Ptr hg = HostGroup::GetByName(name);
 
 			if (hg)
@@ -107,7 +106,7 @@ std::vector<Service::Ptr> Host::GetServices(void) const
 	std::vector<Service::Ptr> services;
 	services.reserve(m_Services.size());
 	typedef std::pair<String, Service::Ptr> ServicePair;
-	BOOST_FOREACH(const ServicePair& kv, m_Services) {
+	for (const ServicePair& kv : m_Services) {
 		services.push_back(kv.second);
 	}
 
@@ -139,7 +138,7 @@ Service::Ptr Host::GetServiceByShortName(const Value& name)
 		{
 			boost::mutex::scoped_lock lock(m_ServicesMutex);
 
-			std::map<String, Service::Ptr>::const_iterator it = m_Services.find(name);
+			auto it = m_Services.find(name);
 
 			if (it != m_Services.end())
 				return it->second;
@@ -237,7 +236,7 @@ bool Host::ResolveMacro(const String& macro, const CheckResult::Ptr&, Value *res
 		*result = StateToString(GetState());
 		return true;
 	} else if (macro == "state_id") {
-		*result = Convert::ToString(GetState());
+		*result = GetState();
 		return true;
 	} else if (macro == "state_type") {
 		*result = StateTypeToString(GetStateType());
@@ -246,19 +245,19 @@ bool Host::ResolveMacro(const String& macro, const CheckResult::Ptr&, Value *res
 		*result = StateToString(GetLastState());
 		return true;
 	} else if (macro == "last_state_id") {
-		*result = Convert::ToString(GetLastState());
+		*result = GetLastState();
 		return true;
 	} else if (macro == "last_state_type") {
 		*result = StateTypeToString(GetLastStateType());
 		return true;
 	} else if (macro == "last_state_change") {
-		*result = Convert::ToString((long)GetLastStateChange());
+		*result = static_cast<long>(GetLastStateChange());
 		return true;
 	} else if (macro == "downtime_depth") {
-		*result = Convert::ToString((long)GetDowntimeDepth());
+		*result = GetDowntimeDepth();
 		return true;
 	} else if (macro == "duration_sec") {
-		*result = Convert::ToString((long)(Utility::GetTime() - GetLastStateChange()));
+		*result = Utility::GetTime() - GetLastStateChange();
 		return true;
 	} else if (macro == "num_services" || macro == "num_services_ok" || macro == "num_services_warning"
 		    || macro == "num_services_unknown" || macro == "num_services_critical") {
@@ -274,14 +273,14 @@ bool Host::ResolveMacro(const String& macro, const CheckResult::Ptr&, Value *res
 			else if (macro == "num_services_critical")
 				filter = ServiceCritical;
 
-			BOOST_FOREACH(const Service::Ptr& service, GetServices()) {
+			for (const Service::Ptr& service : GetServices()) {
 				if (filter != -1 && service->GetState() != filter)
 					continue;
 
 				count++;
 			}
 
-			*result = Convert::ToString(count);
+			*result = count;
 			return true;
 	}
 
@@ -289,10 +288,10 @@ bool Host::ResolveMacro(const String& macro, const CheckResult::Ptr&, Value *res
 
 	if (cr) {
 		if (macro == "latency") {
-			*result = Convert::ToString(cr->CalculateLatency());
+			*result = cr->CalculateLatency();
 			return true;
 		} else if (macro == "execution_time") {
-			*result = Convert::ToString(cr->CalculateExecutionTime());
+			*result = cr->CalculateExecutionTime();
 			return true;
 		} else if (macro == "output") {
 			*result = cr->GetOutput();
@@ -300,9 +299,6 @@ bool Host::ResolveMacro(const String& macro, const CheckResult::Ptr&, Value *res
 		} else if (macro == "perfdata") {
 			*result = PluginUtility::FormatPerfdata(cr->GetPerformanceData());
 			return true;
-		} else if (macro == "last_check") {
-			*result = Convert::ToString((long)cr->GetScheduleStart());
-			return true;
 		} else if (macro == "check_source") {
 			*result = cr->GetCheckSource();
 			return true;
