@@ -27,19 +27,15 @@
 #include "base/context.hpp"
 #include "base/workqueue.hpp"
 #include "base/exception.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
-INITIALIZE_ONCE(&Notification::RegisterApplyRuleHandler);
-
-void Notification::RegisterApplyRuleHandler(void)
-{
+INITIALIZE_ONCE([]() {
 	std::vector<String> targets;
 	targets.push_back("Host");
 	targets.push_back("Service");
 	ApplyRule::RegisterType("Notification", targets);
-}
+});
 
 bool Notification::EvaluateApplyRuleInstance(const Checkable::Ptr& checkable, const String& name, ScriptFrame& frame, const ApplyRule& rule)
 {
@@ -48,8 +44,10 @@ bool Notification::EvaluateApplyRuleInstance(const Checkable::Ptr& checkable, co
 
 	DebugInfo di = rule.GetDebugInfo();
 
+#ifdef _DEBUG
 	Log(LogDebug, "Notification")
 	    << "Applying notification '" << name << "' to object '" << checkable->GetName() << "' for rule " << di;
+#endif /* _DEBUG */
 
 	ConfigItemBuilder::Ptr builder = new ConfigItemBuilder(di);
 	builder->SetType("Notification");
@@ -75,6 +73,8 @@ bool Notification::EvaluateApplyRuleInstance(const Checkable::Ptr& checkable, co
 
 	builder->AddExpression(new OwnedExpression(rule.GetExpression()));
 
+	builder->AddExpression(new ImportDefaultTemplatesExpression());
+
 	ConfigItem::Ptr notificationItem = builder->Compile();
 	notificationItem->Register();
 
@@ -124,7 +124,7 @@ bool Notification::EvaluateApplyRule(const Checkable::Ptr& checkable, const Appl
 		Array::Ptr arr = vinstances;
 
 		ObjectLock olock(arr);
-		BOOST_FOREACH(const Value& instance, arr) {
+		for (const Value& instance : arr) {
 			String name = rule.GetName();
 
 			if (!rule.GetFKVar().IsEmpty()) {
@@ -141,7 +141,7 @@ bool Notification::EvaluateApplyRule(const Checkable::Ptr& checkable, const Appl
 	
 		Dictionary::Ptr dict = vinstances;
 
-		BOOST_FOREACH(const String& key, dict->GetKeys()) {
+		for (const String& key : dict->GetKeys()) {
 			frame.Locals->Set(rule.GetFKVar(), key);
 			frame.Locals->Set(rule.GetFVVar(), dict->Get(key));
 
@@ -157,7 +157,7 @@ void Notification::EvaluateApplyRules(const Host::Ptr& host)
 {
 	CONTEXT("Evaluating 'apply' rules for host '" + host->GetName() + "'");
 
-	BOOST_FOREACH(ApplyRule& rule, ApplyRule::GetRules("Notification"))
+	for (ApplyRule& rule : ApplyRule::GetRules("Notification"))
 	{
 		if (rule.GetTargetType() != "Host")
 			continue;
@@ -171,7 +171,7 @@ void Notification::EvaluateApplyRules(const Service::Ptr& service)
 {
 	CONTEXT("Evaluating 'apply' rules for service '" + service->GetName() + "'");
 
-	BOOST_FOREACH(ApplyRule& rule, ApplyRule::GetRules("Notification")) {
+	for (ApplyRule& rule : ApplyRule::GetRules("Notification")) {
 		if (rule.GetTargetType() != "Service")
 			continue;
 
