@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -21,6 +21,7 @@
 #include "icinga/notification.tcpp"
 #include "icinga/notificationcommand.hpp"
 #include "icinga/service.hpp"
+#include "remote/apilistener.hpp"
 #include "base/objectlock.hpp"
 #include "base/logger.hpp"
 #include "base/utility.hpp"
@@ -28,7 +29,6 @@
 #include "base/exception.hpp"
 #include "base/initialize.hpp"
 #include "base/scriptglobal.hpp"
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/classification.hpp>
 
@@ -37,6 +37,9 @@ using namespace icinga;
 REGISTER_TYPE(Notification);
 INITIALIZE_ONCE(&Notification::StaticInitialize);
 
+std::map<String, int> Notification::m_StateFilterMap;
+std::map<String, int> Notification::m_TypeFilterMap;
+
 boost::signals2::signal<void (const Notification::Ptr&, const MessageOrigin::Ptr&)> Notification::OnNextNotificationChanged;
 
 String NotificationNameComposer::MakeName(const String& shortName, const Object::Ptr& context) const
@@ -79,30 +82,47 @@ Dictionary::Ptr NotificationNameComposer::ParseName(const String& name) const
 
 void Notification::StaticInitialize(void)
 {
-	ScriptGlobal::Set("OK", StateFilterOK);
-	ScriptGlobal::Set("Warning", StateFilterWarning);
-	ScriptGlobal::Set("Critical", StateFilterCritical);
-	ScriptGlobal::Set("Unknown", StateFilterUnknown);
-	ScriptGlobal::Set("Up", StateFilterUp);
-	ScriptGlobal::Set("Down", StateFilterDown);
+	ScriptGlobal::Set("OK", "OK");
+	ScriptGlobal::Set("Warning", "Warning");
+	ScriptGlobal::Set("Critical", "Critical");
+	ScriptGlobal::Set("Unknown", "Unknown");
+	ScriptGlobal::Set("Up", "Up");
+	ScriptGlobal::Set("Down", "Down");
 
-	ScriptGlobal::Set("DowntimeStart", 1 << NotificationDowntimeStart);
-	ScriptGlobal::Set("DowntimeEnd", 1 << NotificationDowntimeEnd);
-	ScriptGlobal::Set("DowntimeRemoved", 1 << NotificationDowntimeRemoved);
-	ScriptGlobal::Set("Custom", 1 << NotificationCustom);
-	ScriptGlobal::Set("Acknowledgement", 1 << NotificationAcknowledgement);
-	ScriptGlobal::Set("Problem", 1 << NotificationProblem);
-	ScriptGlobal::Set("Recovery", 1 << NotificationRecovery);
-	ScriptGlobal::Set("FlappingStart", 1 << NotificationFlappingStart);
-	ScriptGlobal::Set("FlappingEnd", 1 << NotificationFlappingEnd);
+	ScriptGlobal::Set("DowntimeStart", "DowntimeStart");
+	ScriptGlobal::Set("DowntimeEnd", "DowntimeEnd");
+	ScriptGlobal::Set("DowntimeRemoved", "DowntimeRemoved");
+	ScriptGlobal::Set("Custom", "Custom");
+	ScriptGlobal::Set("Acknowledgement", "Acknowledgement");
+	ScriptGlobal::Set("Problem", "Problem");
+	ScriptGlobal::Set("Recovery", "Recovery");
+	ScriptGlobal::Set("FlappingStart", "FlappingStart");
+	ScriptGlobal::Set("FlappingEnd", "FlappingEnd");
+
+	m_StateFilterMap["OK"] = StateFilterOK;
+	m_StateFilterMap["Warning"] = StateFilterWarning;
+	m_StateFilterMap["Critical"] = StateFilterCritical;
+	m_StateFilterMap["Unknown"] = StateFilterUnknown;
+	m_StateFilterMap["Up"] = StateFilterUp;
+	m_StateFilterMap["Down"] = StateFilterDown;
+
+	m_TypeFilterMap["DowntimeStart"] = NotificationDowntimeStart;
+	m_TypeFilterMap["DowntimeEnd"] = NotificationDowntimeEnd;
+	m_TypeFilterMap["DowntimeRemoved"] = NotificationDowntimeRemoved;
+	m_TypeFilterMap["Custom"] = NotificationCustom;
+	m_TypeFilterMap["Acknowledgement"] = NotificationAcknowledgement;
+	m_TypeFilterMap["Problem"] = NotificationProblem;
+	m_TypeFilterMap["Recovery"] = NotificationRecovery;
+	m_TypeFilterMap["FlappingStart"] = NotificationFlappingStart;
+	m_TypeFilterMap["FlappingEnd"] = NotificationFlappingEnd;
 }
 
 void Notification::OnConfigLoaded(void)
 {
 	ObjectImpl<Notification>::OnConfigLoaded();
 
-	SetTypeFilter(FilterArrayToInt(GetTypes(), ~0));
-	SetStateFilter(FilterArrayToInt(GetStates(), ~0));
+	SetTypeFilter(FilterArrayToInt(GetTypes(), GetTypeFilterMap(), ~0));
+	SetStateFilter(FilterArrayToInt(GetStates(), GetStateFilterMap(), ~0));
 }
 
 void Notification::OnAllConfigLoaded(void)
@@ -124,12 +144,15 @@ void Notification::OnAllConfigLoaded(void)
 
 void Notification::Start(bool runtimeCreated)
 {
-	ObjectImpl<Notification>::Start(runtimeCreated);
-
 	Checkable::Ptr obj = GetCheckable();
 
 	if (obj)
 		obj->RegisterNotification(this);
+
+	if (ApiListener::IsHACluster() && GetNextNotification() < Utility::GetTime() + 60)
+		SetNextNotification(Utility::GetTime() + 60, true);
+
+	ObjectImpl<Notification>::Start(runtimeCreated);
 }
 
 void Notification::Stop(bool runtimeRemoved)
@@ -161,7 +184,7 @@ std::set<User::Ptr> Notification::GetUsers(void) const
 	if (users) {
 		ObjectLock olock(users);
 
-		BOOST_FOREACH(const String& name, users) {
+		for (const String& name : users) {
 			User::Ptr user = User::GetByName(name);
 
 			if (!user)
@@ -183,7 +206,7 @@ std::set<UserGroup::Ptr> Notification::GetUserGroups(void) const
 	if (groups) {
 		ObjectLock olock(groups);
 
-		BOOST_FOREACH(const String& name, groups) {
+		for (const String& name : groups) {
 			UserGroup::Ptr ug = UserGroup::GetByName(name);
 
 			if (!ug)
@@ -238,10 +261,10 @@ String Notification::NotificationTypeToString(NotificationType type)
 	}
 }
 
-void Notification::BeginExecuteNotification(NotificationType type, const CheckResult::Ptr& cr, bool force, const String& author, const String& text)
+void Notification::BeginExecuteNotification(NotificationType type, const CheckResult::Ptr& cr, bool force, bool reminder, const String& author, const String& text)
 {
 	Log(LogNotice, "Notification")
-	    << "Attempting to send notifications for notification object '" << GetName() << "'.";
+	    << "Attempting to send " << (reminder ? "reminder " : " ") << "notifications for notification object '" << GetName() << "'.";
 
 	Checkable::Ptr checkable = GetCheckable();
 
@@ -250,47 +273,53 @@ void Notification::BeginExecuteNotification(NotificationType type, const CheckRe
 
 		if (tp && !tp->IsInside(Utility::GetTime())) {
 			Log(LogNotice, "Notification")
-			    << "Not sending notifications for notification object '" << GetName() << "': not in timeperiod";
+			    << "Not sending " << (reminder ? "reminder " : " ") << "notifications for notification object '" << GetName()
+			    << "': not in timeperiod '" << tp->GetName() << "'";
 			return;
 		}
 
 		double now = Utility::GetTime();
 		Dictionary::Ptr times = GetTimes();
 
-		if (type == NotificationProblem) {
-			if (times && times->Contains("begin") && now < checkable->GetLastHardStateChange() + times->Get("begin")) {
+		if (times && type == NotificationProblem) {
+			Value timesBegin = times->Get("begin");
+			Value timesEnd = times->Get("end");
+
+			if (timesBegin != Empty && timesBegin >= 0 && now < checkable->GetLastHardStateChange() + timesBegin) {
 				Log(LogNotice, "Notification")
-				    << "Not sending notifications for notification object '" << GetName() << "': before escalation range";
+				    << "Not sending " << (reminder ? "reminder " : " ") << "notifications for notification object '" << GetName()
+				    << "': before specified begin time (" << Utility::FormatDuration(timesBegin) << ")";
 
 				/* we need to adjust the next notification time
 				 * to now + begin delaying the first notification
 				 */
-				double nextProposedNotification = now + times->Get("begin") + 1.0;
+				double nextProposedNotification = now + timesBegin + 1.0;
 				if (GetNextNotification() > nextProposedNotification)
 					SetNextNotification(nextProposedNotification);
 
 				return;
 			}
 
-			if (times && times->Contains("end") && now > checkable->GetLastHardStateChange() + times->Get("end")) {
+			if (timesEnd != Empty && timesEnd >= 0 && now > checkable->GetLastHardStateChange() + timesEnd) {
 				Log(LogNotice, "Notification")
-				    << "Not sending notifications for notification object '" << GetName() << "': after escalation range";
+				    << "Not sending " << (reminder ? "reminder " : " ") << "notifications for notification object '" << GetName()
+				    << "': after specified end time (" << Utility::FormatDuration(timesEnd) << ")";
 				return;
 			}
 		}
 
-		unsigned long ftype = 1 << type;
+		unsigned long ftype = type;
 
 		Log(LogDebug, "Notification")
 		    << "Type '" << NotificationTypeToStringInternal(type)
-		    << "', TypeFilter " << NotificationFilterToString(GetTypeFilter())
+		    << "', TypeFilter: " << NotificationFilterToString(GetTypeFilter(), GetTypeFilterMap())
 		    << " (FType=" << ftype << ", TypeFilter=" << GetTypeFilter() << ")";
 
 		if (!(ftype & GetTypeFilter())) {
 			Log(LogNotice, "Notification")
-			    << "Not sending notifications for notification object '" << GetName() << "': type '"
-			    << NotificationTypeToStringInternal(type) << "' does not match type filter '"
-			    << NotificationFilterToString(GetTypeFilter()) << ".";
+			    << "Not sending " << (reminder ? "reminder " : " ") << "notifications for notification object '" << GetName() << "': type '"
+			    << NotificationTypeToStringInternal(type) << "' does not match type filter: "
+			    << NotificationFilterToString(GetTypeFilter(), GetTypeFilterMap()) << ".";
 			return;
 		}
 
@@ -312,24 +341,36 @@ void Notification::BeginExecuteNotification(NotificationType type, const CheckRe
 			}
 
 			Log(LogDebug, "Notification")
-			    << "State '" << stateStr << "', StateFilter " << NotificationFilterToString(GetStateFilter())
+			    << "State '" << stateStr << "', StateFilter: " << NotificationFilterToString(GetStateFilter(), GetStateFilterMap())
 			    << " (FState=" << fstate << ", StateFilter=" << GetStateFilter() << ")";
 
 			if (!(fstate & GetStateFilter())) {
 				Log(LogNotice, "Notification")
-				    << "Not sending notifications for notification object '" << GetName() << "': state '" << stateStr
-				    << "' does not match state filter " << NotificationFilterToString(GetStateFilter()) << ".";
+				    << "Not sending " << (reminder ? "reminder " : " ") << "notifications for notification object '" << GetName() << "': state '" << stateStr
+				    << "' does not match state filter: " << NotificationFilterToString(GetStateFilter(), GetStateFilterMap()) << ".";
 				return;
 			}
 		}
+	} else {
+		Log(LogNotice, "Notification")
+		    << "Not checking " << (reminder ? "reminder " : " ") << "notification filters for notification object '" << GetName() << "': Notification was forced.";
 	}
 
 	{
 		ObjectLock olock(this);
 
+		UpdateNotificationNumber();
 		double now = Utility::GetTime();
 		SetLastNotification(now);
 
+		if (type == NotificationProblem && GetInterval() <= 0)
+			SetNoMoreNotifications(true);
+		else
+			SetNoMoreNotifications(false);
+
+		if (type == NotificationProblem && GetInterval() > 0)
+			SetNextNotification(now + GetInterval());
+
 		if (type == NotificationProblem)
 			SetLastProblemNotification(now);
 	}
@@ -339,17 +380,15 @@ void Notification::BeginExecuteNotification(NotificationType type, const CheckRe
 	std::set<User::Ptr> users = GetUsers();
 	std::copy(users.begin(), users.end(), std::inserter(allUsers, allUsers.begin()));
 
-	BOOST_FOREACH(const UserGroup::Ptr& ug, GetUserGroups()) {
+	for (const UserGroup::Ptr& ug : GetUserGroups()) {
 		std::set<User::Ptr> members = ug->GetMembers();
 		std::copy(members.begin(), members.end(), std::inserter(allUsers, allUsers.begin()));
 	}
 
-	Service::OnNotificationSendStart(this, checkable, allUsers, type, cr, author, text);
-
 	std::set<User::Ptr> allNotifiedUsers;
-	Array::Ptr notifiedUsers = GetNotifiedUsers();
+	Array::Ptr notifiedProblemUsers = GetNotifiedProblemUsers();
 
-	BOOST_FOREACH(const User::Ptr& user, allUsers) {
+	for (const User::Ptr& user : allUsers) {
 		String userName = user->GetName();
 
 		if (!user->GetEnableNotifications()) {
@@ -358,7 +397,7 @@ void Notification::BeginExecuteNotification(NotificationType type, const CheckRe
 			continue;
 		}
 
-		if (!CheckNotificationUserFilters(type, user, force)) {
+		if (!CheckNotificationUserFilters(type, user, force, reminder)) {
 			Log(LogNotice, "Notification")
 			    << "Notification filters for user '" << userName << "' not matched. Not sending notification.";
 			continue;
@@ -366,15 +405,16 @@ void Notification::BeginExecuteNotification(NotificationType type, const CheckRe
 
 		/* on recovery, check if user was notified before */
 		if (type == NotificationRecovery) {
-			if (!notifiedUsers->Contains(userName)) {
+			if (!notifiedProblemUsers->Contains(userName)) {
 				Log(LogNotice, "Notification")
-				    << "We did not notify user '" << userName << "' before. Not sending recovery notification.";
+				    << "We did not notify user '" << userName << "' for a problem before. Not sending recovery notification.";
 				continue;
 			}
 		}
 
 		Log(LogInformation, "Notification")
-		    << "Sending notification '" << GetName() << "' for user '" << userName << "'";
+		    << "Sending " << (reminder ? "reminder " : "") << "'" << NotificationTypeToStringInternal(type) << "' notification '"
+		    << GetName() << " for user '" << userName << "'";
 
 		Utility::QueueAsyncCallback(boost::bind(&Notification::ExecuteNotificationHelper, this, type, user, cr, force, author, text));
 
@@ -382,44 +422,45 @@ void Notification::BeginExecuteNotification(NotificationType type, const CheckRe
 		allNotifiedUsers.insert(user);
 
 		/* store all notified users for later recovery checks */
-		if (!notifiedUsers->Contains(userName))
-			notifiedUsers->Add(userName);
+		if (type == NotificationProblem && !notifiedProblemUsers->Contains(userName))
+			notifiedProblemUsers->Add(userName);
 	}
 
 	/* if this was a recovery notification, reset all notified users */
 	if (type == NotificationRecovery)
-		notifiedUsers->Clear();
+		notifiedProblemUsers->Clear();
 
 	/* used in db_ido for notification history */
-	Service::OnNotificationSentToAllUsers(this, checkable, allNotifiedUsers, type, cr, author, text);
+	Service::OnNotificationSentToAllUsers(this, checkable, allNotifiedUsers, type, cr, author, text, MessageOrigin::Ptr());
 }
 
-bool Notification::CheckNotificationUserFilters(NotificationType type, const User::Ptr& user, bool force)
+bool Notification::CheckNotificationUserFilters(NotificationType type, const User::Ptr& user, bool force, bool reminder)
 {
 	if (!force) {
 		TimePeriod::Ptr tp = user->GetPeriod();
 
 		if (tp && !tp->IsInside(Utility::GetTime())) {
 			Log(LogNotice, "Notification")
-			    << "Not sending notifications for notification object '"
-			    << GetName() << " and user '" << user->GetName() << "': user not in timeperiod";
+			    << "Not sending " << (reminder ? "reminder " : " ") << "notifications for notification object '"
+			    << GetName() << " and user '" << user->GetName()
+			    << "': user period not in timeperiod '" << tp->GetName() << "'";
 			return false;
 		}
 
-		unsigned long ftype = 1 << type;
+		unsigned long ftype = type;
 
 		Log(LogDebug, "Notification")
 		    << "User notification, Type '" << NotificationTypeToStringInternal(type)
-		    << "', TypeFilter " << NotificationFilterToString(user->GetTypeFilter())
+		    << "', TypeFilter: " << NotificationFilterToString(user->GetTypeFilter(), GetTypeFilterMap())
 		    << " (FType=" << ftype << ", TypeFilter=" << GetTypeFilter() << ")";
 
 
 		if (!(ftype & user->GetTypeFilter())) {
 			Log(LogNotice, "Notification")
-			    << "Not sending notifications for notification object '"
+			    << "Not sending " << (reminder ? "reminder " : " ") << "notifications for notification object '"
 			    << GetName() << " and user '" << user->GetName() << "': type '"
-			    << NotificationTypeToStringInternal(type) << "' does not match type filter "
-			    << NotificationFilterToString(user->GetTypeFilter()) << ".";
+			    << NotificationTypeToStringInternal(type) << "' does not match type filter: "
+			    << NotificationFilterToString(user->GetTypeFilter(), GetTypeFilterMap()) << ".";
 			return false;
 		}
 
@@ -442,18 +483,22 @@ bool Notification::CheckNotificationUserFilters(NotificationType type, const Use
 			}
 
 			Log(LogDebug, "Notification")
-			    << "User notification, State '" << stateStr << "', StateFilter "
-			    << NotificationFilterToString(user->GetStateFilter())
+			    << "User notification, State '" << stateStr << "', StateFilter: "
+			    << NotificationFilterToString(user->GetStateFilter(), GetStateFilterMap())
 			    << " (FState=" << fstate << ", StateFilter=" << user->GetStateFilter() << ")";
 
 			if (!(fstate & user->GetStateFilter())) {
 				Log(LogNotice, "Notification")
-				    << "Not sending notifications for notification object '"
+				    << "Not " << (reminder ? "reminder " : " ") << "sending notifications for notification object '"
 				    << GetName() << " and user '" << user->GetName() << "': state '" << stateStr
-				    << "' does not match state filter " << NotificationFilterToString(user->GetStateFilter()) << ".";
+				    << "' does not match state filter: " << NotificationFilterToString(user->GetStateFilter(), GetStateFilterMap()) << ".";
 				return false;
 			}
 		}
+	} else {
+		Log(LogNotice, "Notification")
+		    << "Not checking " << (reminder ? "reminder " : " ") << "notification filters for notification object '"
+		    << GetName() << "' and user '" << user->GetName() << "': Notification was forced.";
 	}
 
 	return true;
@@ -466,23 +511,20 @@ void Notification::ExecuteNotificationHelper(NotificationType type, const User::
 
 		if (!command) {
 			Log(LogDebug, "Notification")
-			    << "No notification_command found for notification '" << GetName() << "'. Skipping execution.";
+			    << "No command found for notification '" << GetName() << "'. Skipping execution.";
 			return;
 		}
 
 		command->Execute(this, user, cr, type, author, text);
 
-		{
-			ObjectLock olock(this);
-			UpdateNotificationNumber();
-			SetLastNotification(Utility::GetTime());
-		}
-
 		/* required by compatlogger */
-		Service::OnNotificationSentToUser(this, GetCheckable(), user, type, cr, author, text, command->GetName());
+		Service::OnNotificationSentToUser(this, GetCheckable(), user, type, cr, author, text, command->GetName(), MessageOrigin::Ptr());
 
 		Log(LogInformation, "Notification")
-		    << "Completed sending notification '" << GetName() << "' for checkable '" << GetCheckable()->GetName() << "'";
+		    << "Completed sending '" << NotificationTypeToStringInternal(type)
+		    << "' notification '" << GetName()
+		    << "' for checkable '" << GetCheckable()->GetName()
+		    << "' and user '" << user->GetName() << "'.";
 	} catch (const std::exception& ex) {
 		Log(LogWarning, "Notification")
 		    << "Exception occured during notification for checkable '"
@@ -518,62 +560,17 @@ int icinga::HostStateToFilter(HostState state)
 	}
 }
 
-int icinga::FilterArrayToInt(const Array::Ptr& typeFilters, int defaultValue)
+String Notification::NotificationFilterToString(int filter, const std::map<String, int>& filterMap)
 {
-	Value resultTypeFilter;
+	std::vector<String> sFilters;
 
-	if (!typeFilters)
-		return defaultValue;
-
-	resultTypeFilter = 0;
-
-	ObjectLock olock(typeFilters);
-	BOOST_FOREACH(const Value& typeFilter, typeFilters) {
-		resultTypeFilter = resultTypeFilter | typeFilter;
+	typedef std::pair<String, int> kv_pair;
+	for (const kv_pair& kv : filterMap) {
+		if (filter & kv.second)
+			sFilters.push_back(kv.first);
 	}
 
-	return resultTypeFilter;
-}
-
-std::vector<String> icinga::FilterIntToArray(int iFilter)
-{
-	std::vector<String> filter;
-
-	if (iFilter & StateFilterOK)
-		filter.push_back("OK");
-	if (iFilter & StateFilterWarning)
-		filter.push_back("Warning");
-	if (iFilter & StateFilterUnknown)
-		filter.push_back("Unknown");
-	if (iFilter & StateFilterUp)
-		filter.push_back("Up");
-	if (iFilter & StateFilterDown)
-		filter.push_back("Down");
-	if (iFilter & NotificationDowntimeStart)
-		filter.push_back("DowntimeStart");
-	if (iFilter & NotificationDowntimeEnd)
-		filter.push_back("DowntimeEnd");
-	if (iFilter & NotificationDowntimeRemoved)
-		filter.push_back("DowntimeRemoved");
-	if (iFilter & NotificationCustom)
-		filter.push_back("Custom");
-	if (iFilter & NotificationAcknowledgement)
-		filter.push_back("Acknowledgement");
-	if (iFilter & NotificationProblem)
-		filter.push_back("Problem");
-	if (iFilter & NotificationRecovery)
-		filter.push_back("Recovery");
-	if (iFilter & NotificationFlappingStart)
-		filter.push_back("FlappingStart");
-	if (iFilter & NotificationFlappingEnd)
-		filter.push_back("FlappingEnd");
-
-	return filter;
-}
-
-String Notification::NotificationFilterToString(int filter)
-{
-	return Utility::NaturalJoin(FilterIntToArray(filter));
+	return Utility::NaturalJoin(sFilters);
 }
 
 /* internal for logging */
@@ -649,24 +646,24 @@ void Notification::ValidateStates(const Array::Ptr& value, const ValidationUtils
 {
 	ObjectImpl<Notification>::ValidateStates(value, utils);
 
-	int sfilter = FilterArrayToInt(value, 0);
+	int filter = FilterArrayToInt(value, GetStateFilterMap(), 0);
 
-	if (GetServiceName().IsEmpty() && (sfilter & ~(StateFilterUp | StateFilterDown)) != 0)
+	if (GetServiceName().IsEmpty() && (filter == -1 || (filter & ~(StateFilterUp | StateFilterDown)) != 0))
 		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("states"), "State filter is invalid."));
 
-	if (!GetServiceName().IsEmpty() && (sfilter & ~(StateFilterOK | StateFilterWarning | StateFilterCritical | StateFilterUnknown)) != 0)
-		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("types"), "State filter is invalid."));
+	if (!GetServiceName().IsEmpty() && (filter == -1 || (filter & ~(StateFilterOK | StateFilterWarning | StateFilterCritical | StateFilterUnknown)) != 0))
+		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("states"), "State filter is invalid."));
 }
 
 void Notification::ValidateTypes(const Array::Ptr& value, const ValidationUtils& utils)
 {
 	ObjectImpl<Notification>::ValidateTypes(value, utils);
 
-	int tfilter = FilterArrayToInt(value, 0);
+	int filter = FilterArrayToInt(value, GetTypeFilterMap(), 0);
 
-	if ((tfilter & ~(1 << NotificationDowntimeStart | 1 << NotificationDowntimeEnd | 1 << NotificationDowntimeRemoved |
-	    1 << NotificationCustom | 1 << NotificationAcknowledgement | 1 << NotificationProblem | 1 << NotificationRecovery |
-	    1 << NotificationFlappingStart | 1 << NotificationFlappingEnd)) != 0)
+	if (filter == -1 || (filter & ~(NotificationDowntimeStart | NotificationDowntimeEnd | NotificationDowntimeRemoved |
+	    NotificationCustom | NotificationAcknowledgement | NotificationProblem | NotificationRecovery |
+	    NotificationFlappingStart | NotificationFlappingEnd)) != 0)
 		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("types"), "Type filter is invalid."));
 }
 
@@ -675,3 +672,12 @@ Endpoint::Ptr Notification::GetCommandEndpoint(void) const
 	return Endpoint::GetByName(GetCommandEndpointRaw());
 }
 
+const std::map<String, int>& Notification::GetStateFilterMap(void)
+{
+	return m_StateFilterMap;
+}
+
+const std::map<String, int>& Notification::GetTypeFilterMap(void)
+{
+	return m_TypeFilterMap;
+}
