@@ -25,8 +25,6 @@
 #include "base/objectlock.hpp"
 #include "base/convert.hpp"
 #include "base/utility.hpp"
-#include <boost/foreach.hpp>
-#include <boost/bind/apply.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/classification.hpp>
 
@@ -86,7 +84,7 @@ void Service::OnAllConfigLoaded(void)
 
 		ObjectLock olock(groups);
 
-		BOOST_FOREACH(const String& name, groups) {
+		for (const String& name : groups) {
 			ServiceGroup::Ptr sg = ServiceGroup::GetByName(name);
 
 			if (sg)
@@ -97,13 +95,13 @@ void Service::OnAllConfigLoaded(void)
 
 void Service::CreateChildObjects(const Type::Ptr& childType)
 {
-	if (childType->GetName() == "ScheduledDowntime")
+	if (childType == ScheduledDowntime::TypeInstance)
 		ScheduledDowntime::EvaluateApplyRules(this);
 
-	if (childType->GetName() == "Notification")
+	if (childType == Notification::TypeInstance)
 		Notification::EvaluateApplyRules(this);
 
-	if (childType->GetName() == "Dependency")
+	if (childType == Dependency::TypeInstance)
 		Dependency::EvaluateApplyRules(this);
 }
 
@@ -192,7 +190,7 @@ bool Service::ResolveMacro(const String& macro, const CheckResult::Ptr& cr, Valu
 		*result = StateToString(GetState());
 		return true;
 	} else if (macro == "state_id") {
-		*result = Convert::ToString(GetState());
+		*result = GetState();
 		return true;
 	} else if (macro == "state_type") {
 		*result = StateTypeToString(GetStateType());
@@ -201,28 +199,28 @@ bool Service::ResolveMacro(const String& macro, const CheckResult::Ptr& cr, Valu
 		*result = StateToString(GetLastState());
 		return true;
 	} else if (macro == "last_state_id") {
-		*result = Convert::ToString(GetLastState());
+		*result = GetLastState();
 		return true;
 	} else if (macro == "last_state_type") {
 		*result = StateTypeToString(GetLastStateType());
 		return true;
 	} else if (macro == "last_state_change") {
-		*result = Convert::ToString((long)GetLastStateChange());
+		*result = static_cast<long>(GetLastStateChange());
 		return true;
 	} else if (macro == "downtime_depth") {
-		*result = Convert::ToString((long)GetDowntimeDepth());
+		*result = GetDowntimeDepth();
 		return true;
 	} else if (macro == "duration_sec") {
-		*result = Convert::ToString((long)(Utility::GetTime() - GetLastStateChange()));
+		*result = Utility::GetTime() - GetLastStateChange();
 		return true;
 	}
 
 	if (cr) {
 		if (macro == "latency") {
-			*result = Convert::ToString(cr->CalculateLatency());
+			*result = cr->CalculateLatency();
 			return true;
 		} else if (macro == "execution_time") {
-			*result = Convert::ToString(cr->CalculateExecutionTime());
+			*result = cr->CalculateExecutionTime();
 			return true;
 		} else if (macro == "output") {
 			*result = cr->GetOutput();
@@ -230,9 +228,6 @@ bool Service::ResolveMacro(const String& macro, const CheckResult::Ptr& cr, Valu
 		} else if (macro == "perfdata") {
 			*result = PluginUtility::FormatPerfdata(cr->GetPerformanceData());
 			return true;
-		} else if (macro == "last_check") {
-			*result = Convert::ToString((long)cr->GetExecutionEnd());
-			return true;
 		} else if (macro == "check_source") {
 			*result = cr->GetCheckSource();
 			return true;
