@@ -26,7 +26,6 @@
 #include "base/logger.hpp"
 #include "base/timer.hpp"
 #include "base/utility.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -51,7 +50,9 @@ void TimePeriod::Start(bool runtimeCreated)
 	/* Pre-fill the time period for the next 24 hours. */
 	double now = Utility::GetTime();
 	UpdateRegion(now, now + 24 * 3600, true);
+#ifdef _DEBUG
 	Dump();
+#endif /* _DEBUG */
 }
 
 void TimePeriod::AddSegment(double begin, double end)
@@ -73,19 +74,26 @@ void TimePeriod::AddSegment(double begin, double end)
 	if (segments) {
 		/* Try to merge the new segment into an existing segment. */
 		ObjectLock dlock(segments);
-		BOOST_FOREACH(const Dictionary::Ptr& segment, segments) {
+		for (const Dictionary::Ptr& segment : segments) {
 			if (segment->Get("begin") <= begin && segment->Get("end") >= end)
 				return; /* New segment is fully contained in this segment. */
 
-			if (segment->Get("begin") <= begin && segment->Get("end") >= begin) {
-				segment->Set("end", end); /* Extend an existing segment. */
+			if (segment->Get("begin") >= begin && segment->Get("end") <= end) {
+				segment->Set("begin", begin);
+				segment->Set("end", end); /* Extend an existing segment to both sides */
+				return;
+			}
+
+			if (segment->Get("end") >= begin && segment->Get("end") <= end) {
+				segment->Set("end", end); /* Extend an existing segment to right. */
 				return;
 			}
 
 			if (segment->Get("begin") >= begin && segment->Get("begin") <= end) {
-				segment->Set("begin", begin); /* Extend an existing segment. */
+				segment->Set("begin", begin); /* Extend an existing segment to left. */
 				return;
 			}
+
 		}
 	}
 
@@ -130,7 +138,7 @@ void TimePeriod::RemoveSegment(double begin, double end)
 
 	/* Try to split or adjust an existing segment. */
 	ObjectLock dlock(segments);
-	BOOST_FOREACH(const Dictionary::Ptr& segment, segments) {
+	for (const Dictionary::Ptr& segment : segments) {
 		/* Fully contained in the specified range? */
 		if (segment->Get("begin") >= begin && segment->Get("end") <= end)
 			continue;
@@ -142,6 +150,21 @@ void TimePeriod::RemoveSegment(double begin, double end)
 			continue;
 		}
 
+		/* Cut between */
+		if (segment->Get("begin") < begin && segment->Get("end") > end) {
+			Dictionary::Ptr firstsegment = new Dictionary();
+			firstsegment->Set("begin", segment->Get("begin"));
+			firstsegment->Set("end", begin);
+
+			Dictionary::Ptr secondsegment = new Dictionary();
+			secondsegment->Set("begin", end);
+			secondsegment->Set("end", segment->Get("end"));
+
+			newSegments->Add(firstsegment);
+			newSegments->Add(secondsegment);
+			continue;
+		}
+
 		/* Adjust the begin/end timestamps so as to not overlap with the specified range. */
 		if (segment->Get("begin") > begin && segment->Get("begin") < end)
 			segment->Set("begin", end);
@@ -154,7 +177,14 @@ void TimePeriod::RemoveSegment(double begin, double end)
 
 	SetSegments(newSegments);
 
+#ifdef _DEBUG
 	Dump();
+#endif /* _DEBUG */
+}
+
+void TimePeriod::RemoveSegment(const Dictionary::Ptr& segment)
+{
+	RemoveSegment(segment->Get("begin"), segment->Get("end"));
 }
 
 void TimePeriod::PurgeSegments(double end)
@@ -179,7 +209,7 @@ void TimePeriod::PurgeSegments(double end)
 
 	/* Remove old segments. */
 	ObjectLock dlock(segments);
-	BOOST_FOREACH(const Dictionary::Ptr& segment, segments) {
+	for (const Dictionary::Ptr& segment : segments) {
 		if (segment->Get("end") >= end)
 			newSegments->Add(segment);
 	}
@@ -187,6 +217,23 @@ void TimePeriod::PurgeSegments(double end)
 	SetSegments(newSegments);
 }
 
+void TimePeriod::Merge(const TimePeriod::Ptr& timeperiod, bool include)
+{
+	Log(LogDebug, "TimePeriod")
+	    << "Merge TimePeriod '" << GetName() << "' with '" << timeperiod->GetName() << "' "
+	    << "Method: " << (include ? "include" : "exclude");
+
+	Array::Ptr segments = timeperiod->GetSegments();
+
+	if (segments) {
+		ObjectLock dlock(segments);
+		ObjectLock ilock(this);
+		for (const Dictionary::Ptr& segment : segments) {
+			include ? AddSegment(segment) : RemoveSegment(segment);
+		}
+	}
+}
+
 void TimePeriod::UpdateRegion(double begin, double end, bool clearExisting)
 {
 	if (!clearExisting) {
@@ -210,11 +257,39 @@ void TimePeriod::UpdateRegion(double begin, double end, bool clearExisting)
 
 		if (segments) {
 			ObjectLock dlock(segments);
-			BOOST_FOREACH(const Dictionary::Ptr& segment, segments) {
+			for (const Dictionary::Ptr& segment : segments) {
 				AddSegment(segment);
 			}
 		}
 	}
+
+	bool preferInclude = GetPreferIncludes();
+
+	/* First handle the non preferred timeranges */
+	Array::Ptr timeranges = preferInclude ? GetExcludes() : GetIncludes();
+
+	if (timeranges) {
+		ObjectLock olock(timeranges);
+		for (const String& name : timeranges) {
+			const TimePeriod::Ptr timeperiod = TimePeriod::GetByName(name);
+
+			if (timeperiod)
+				Merge(timeperiod, !preferInclude);
+		}
+	}
+
+	/* Preferred timeranges must be handled at the end */
+	timeranges = preferInclude ? GetIncludes() : GetExcludes();
+
+	if (timeranges) {
+		ObjectLock olock(timeranges);
+		for (const String& name : timeranges) {
+			const TimePeriod::Ptr timeperiod = TimePeriod::GetByName(name);
+
+			if (timeperiod)
+				Merge(timeperiod, preferInclude);
+		}
+	}
 }
 
 bool TimePeriod::GetIsInside(void) const
@@ -233,7 +308,7 @@ bool TimePeriod::IsInside(double ts) const
 
 	if (segments) {
 		ObjectLock dlock(segments);
-		BOOST_FOREACH(const Dictionary::Ptr& segment, segments) {
+		for (const Dictionary::Ptr& segment : segments) {
 			if (ts > segment->Get("begin") && ts < segment->Get("end"))
 				return true;
 		}
@@ -252,7 +327,7 @@ double TimePeriod::FindNextTransition(double begin)
 
 	if (segments) {
 		ObjectLock dlock(segments);
-		BOOST_FOREACH(const Dictionary::Ptr& segment, segments) {
+		for (const Dictionary::Ptr& segment : segments) {
 			if (segment->Get("begin") > begin && (segment->Get("begin") < closestTransition || closestTransition == -1))
 				closestTransition = segment->Get("begin");
 
@@ -268,7 +343,10 @@ void TimePeriod::UpdateTimerHandler(void)
 {
 	double now = Utility::GetTime();
 
-	BOOST_FOREACH(const TimePeriod::Ptr& tp, ConfigType::GetObjectsByType<TimePeriod>()) {
+	for (const TimePeriod::Ptr& tp : ConfigType::GetObjectsByType<TimePeriod>()) {
+		if (!tp->IsActive())
+			continue;
+
 		double valid_end;
 
 		{
@@ -279,7 +357,9 @@ void TimePeriod::UpdateTimerHandler(void)
 		}
 
 		tp->UpdateRegion(valid_end, now + 24 * 3600, false);
+#ifdef _DEBUG
 		tp->Dump();
+#endif /* _DEBUG */
 	}
 }
 
@@ -296,7 +376,7 @@ void TimePeriod::Dump(void)
 
 	if (segments) {
 		ObjectLock dlock(segments);
-		BOOST_FOREACH(const Dictionary::Ptr& segment, segments) {
+		for (const Dictionary::Ptr& segment : segments) {
 			Log(LogDebug, "TimePeriod")
 			    << "Segment: " << Utility::FormatDateTime("%c", segment->Get("begin")) << " <-> "
 			    << Utility::FormatDateTime("%c", segment->Get("end"));
@@ -317,7 +397,7 @@ void TimePeriod::ValidateRanges(const Dictionary::Ptr& value, const ValidationUt
 	Array::Ptr segments = new Array();
 
 	ObjectLock olock(value);
-	BOOST_FOREACH(const Dictionary::Pair& kv, value) {
+	for (const Dictionary::Pair& kv : value) {
 		try {
 			tm begin_tm, end_tm;
 			int stride;
