@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -24,7 +24,6 @@
 #include "icinga/usergroup.hpp"
 #include "base/objectlock.hpp"
 #include "base/exception.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -34,8 +33,8 @@ void User::OnConfigLoaded(void)
 {
 	ObjectImpl<User>::OnConfigLoaded();
 
-	SetTypeFilter(FilterArrayToInt(GetTypes(), ~0));
-	SetStateFilter(FilterArrayToInt(GetStates(), ~0));
+	SetTypeFilter(FilterArrayToInt(GetTypes(), Notification::GetTypeFilterMap(), ~0));
+	SetStateFilter(FilterArrayToInt(GetStates(), Notification::GetStateFilterMap(), ~0));
 }
 
 void User::OnAllConfigLoaded(void)
@@ -51,7 +50,7 @@ void User::OnAllConfigLoaded(void)
 
 		ObjectLock olock(groups);
 
-		BOOST_FOREACH(const String& name, groups) {
+		for (const String& name : groups) {
 			UserGroup::Ptr ug = UserGroup::GetByName(name);
 
 			if (ug)
@@ -69,7 +68,7 @@ void User::Stop(bool runtimeRemoved)
 	if (groups) {
 		ObjectLock olock(groups);
 
-		BOOST_FOREACH(const String& name, groups) {
+		for (const String& name : groups) {
 			UserGroup::Ptr ug = UserGroup::GetByName(name);
 
 			if (ug)
@@ -102,20 +101,20 @@ void User::ValidateStates(const Array::Ptr& value, const ValidationUtils& utils)
 {
 	ObjectImpl<User>::ValidateStates(value, utils);
 
-	int sfilter = FilterArrayToInt(value, 0);
+	int filter = FilterArrayToInt(value, Notification::GetStateFilterMap(), 0);
 
-	if ((sfilter & ~(StateFilterUp | StateFilterDown | StateFilterOK | StateFilterWarning | StateFilterCritical | StateFilterUnknown)) != 0) {
+	if (filter == -1 || (filter & ~(StateFilterUp | StateFilterDown | StateFilterOK | StateFilterWarning | StateFilterCritical | StateFilterUnknown)) != 0)
 		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("states"), "State filter is invalid."));
-	}
 }
 
 void User::ValidateTypes(const Array::Ptr& value, const ValidationUtils& utils)
 {
-	int tfilter = FilterArrayToInt(value, 0);
+	ObjectImpl<User>::ValidateTypes(value, utils);
 
-	if ((tfilter & ~(1 << NotificationDowntimeStart | 1 << NotificationDowntimeEnd | 1 << NotificationDowntimeRemoved |
-	    1 << NotificationCustom | 1 << NotificationAcknowledgement | 1 << NotificationProblem | 1 << NotificationRecovery |
-	    1 << NotificationFlappingStart | 1 << NotificationFlappingEnd)) != 0) {
+	int filter = FilterArrayToInt(value, Notification::GetTypeFilterMap(), 0);
+
+	if (filter == -1 || (filter & ~(NotificationDowntimeStart | NotificationDowntimeEnd | NotificationDowntimeRemoved |
+	    NotificationCustom | NotificationAcknowledgement | NotificationProblem | NotificationRecovery |
+	    NotificationFlappingStart | NotificationFlappingEnd)) != 0)
 		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("types"), "Type filter is invalid."));
-	}
 }
