@@ -22,7 +22,6 @@
 #include "icinga/host.hpp"
 #include "icinga/service.hpp"
 #include "base/configtype.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -73,7 +72,7 @@ String HostGroupsTable::GetPrefix(void) const
 
 void HostGroupsTable::FetchRows(const AddRowFunction& addRowFn)
 {
-	BOOST_FOREACH(const HostGroup::Ptr& hg, ConfigType::GetObjectsByType<HostGroup>()) {
+	for (const HostGroup::Ptr& hg : ConfigType::GetObjectsByType<HostGroup>()) {
 		if (!addRowFn(hg, LivestatusGroupByNone, Empty))
 			return;
 	}
@@ -138,7 +137,7 @@ Value HostGroupsTable::MembersAccessor(const Value& row)
 
 	Array::Ptr members = new Array();
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		members->Add(host->GetName());
 	}
 
@@ -154,7 +153,7 @@ Value HostGroupsTable::MembersWithStateAccessor(const Value& row)
 
 	Array::Ptr members = new Array();
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		Array::Ptr member_state = new Array();
 		member_state->Add(host->GetName());
 		member_state->Add(host->GetState());
@@ -173,7 +172,7 @@ Value HostGroupsTable::WorstHostStateAccessor(const Value& row)
 
 	int worst_host = HostUp;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		if (host->GetState() > worst_host)
 			worst_host = host->GetState();
 	}
@@ -200,7 +199,7 @@ Value HostGroupsTable::NumHostsPendingAccessor(const Value& row)
 
 	int num_hosts = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		/* no checkresult */
 		if (!host->GetLastCheckResult())
 			num_hosts++;
@@ -218,7 +217,7 @@ Value HostGroupsTable::NumHostsUpAccessor(const Value& row)
 
 	int num_hosts = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		if (host->GetState() == HostUp)
 			num_hosts++;
 	}
@@ -235,7 +234,7 @@ Value HostGroupsTable::NumHostsDownAccessor(const Value& row)
 
 	int num_hosts = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		if (host->GetState() == HostDown)
 			num_hosts++;
 	}
@@ -252,7 +251,7 @@ Value HostGroupsTable::NumHostsUnreachAccessor(const Value& row)
 
 	int num_hosts = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		if (!host->IsReachable())
 			num_hosts++;
 	}
@@ -272,7 +271,7 @@ Value HostGroupsTable::NumServicesAccessor(const Value& row)
 	if (hg->GetMembers().size() == 0)
 		return 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
 		num_services += host->GetServices().size();
 	}
 
@@ -288,8 +287,8 @@ Value HostGroupsTable::WorstServicesStateAccessor(const Value& row)
 
 	Value worst_service = ServiceOK;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetState() > worst_service)
 				worst_service = service->GetState();
 		}
@@ -307,8 +306,8 @@ Value HostGroupsTable::NumServicesPendingAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (!service->GetLastCheckResult())
 				num_services++;
 		}
@@ -326,8 +325,8 @@ Value HostGroupsTable::NumServicesOkAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetState() == ServiceOK)
 				num_services++;
 		}
@@ -345,8 +344,8 @@ Value HostGroupsTable::NumServicesWarnAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetState() == ServiceWarning)
 				num_services++;
 		}
@@ -364,8 +363,8 @@ Value HostGroupsTable::NumServicesCritAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetState() == ServiceCritical)
 				num_services++;
 		}
@@ -383,8 +382,8 @@ Value HostGroupsTable::NumServicesUnknownAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetState() == ServiceUnknown)
 				num_services++;
 		}
@@ -402,8 +401,8 @@ Value HostGroupsTable::WorstServiceHardStateAccessor(const Value& row)
 
 	Value worst_service = ServiceOK;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetStateType() == StateTypeHard) {
 				if (service->GetState() > worst_service)
 					worst_service = service->GetState();
@@ -423,8 +422,8 @@ Value HostGroupsTable::NumServicesHardOkAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetStateType() == StateTypeHard && service->GetState() == ServiceOK)
 				num_services++;
 		}
@@ -442,8 +441,8 @@ Value HostGroupsTable::NumServicesHardWarnAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetStateType() == StateTypeHard && service->GetState() == ServiceWarning)
 				num_services++;
 		}
@@ -461,8 +460,8 @@ Value HostGroupsTable::NumServicesHardCritAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetStateType() == StateTypeHard && service->GetState() == ServiceCritical)
 				num_services++;
 		}
@@ -480,8 +479,8 @@ Value HostGroupsTable::NumServicesHardUnknownAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
-		BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Host::Ptr& host : hg->GetMembers()) {
+		for (const Service::Ptr& service : host->GetServices()) {
 			if (service->GetStateType() == StateTypeHard && service->GetState() == ServiceUnknown)
 				num_services++;
 		}
