@@ -34,7 +34,6 @@
 #include "base/json.hpp"
 #include "base/convert.hpp"
 #include "base/utility.hpp"
-#include <boost/foreach.hpp>
 #include <boost/tuple/tuple.hpp>
 #include <boost/algorithm/string/replace.hpp>
 
@@ -58,7 +57,7 @@ void HostsTable::AddColumns(Table *table, const String& prefix,
 	table->AddColumn(prefix + "check_command", Column(&HostsTable::CheckCommandAccessor, objectAccessor));
 	table->AddColumn(prefix + "check_command_expanded", Column(&HostsTable::CheckCommandExpandedAccessor, objectAccessor));
 	table->AddColumn(prefix + "event_handler", Column(&HostsTable::EventHandlerAccessor, objectAccessor));
-	table->AddColumn(prefix + "notification_period", Column(&HostsTable::NotificationPeriodAccessor, objectAccessor));
+	table->AddColumn(prefix + "notification_period", Column(&Table::EmptyStringAccessor, objectAccessor));
 	table->AddColumn(prefix + "check_period", Column(&HostsTable::CheckPeriodAccessor, objectAccessor));
 	table->AddColumn(prefix + "notes", Column(&HostsTable::NotesAccessor, objectAccessor));
 	table->AddColumn(prefix + "notes_expanded", Column(&HostsTable::NotesExpandedAccessor, objectAccessor));
@@ -188,15 +187,15 @@ String HostsTable::GetPrefix(void) const
 void HostsTable::FetchRows(const AddRowFunction& addRowFn)
 {
 	if (GetGroupByType() == LivestatusGroupByHostGroup) {
-		BOOST_FOREACH(const HostGroup::Ptr& hg, ConfigType::GetObjectsByType<HostGroup>()) {
-			BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+		for (const HostGroup::Ptr& hg : ConfigType::GetObjectsByType<HostGroup>()) {
+			for (const Host::Ptr& host : hg->GetMembers()) {
 				/* the caller must know which groupby type and value are set for this row */
 				if (!addRowFn(host, LivestatusGroupByHostGroup, hg))
 					return;
 			}
 		}
 	} else {
-		BOOST_FOREACH(const Host::Ptr& host, ConfigType::GetObjectsByType<Host>()) {
+		for (const Host::Ptr& host : ConfigType::GetObjectsByType<Host>()) {
 			if (!addRowFn(host, LivestatusGroupByNone, Empty))
 				return;
 		}
@@ -297,16 +296,6 @@ Value HostsTable::EventHandlerAccessor(const Value& row)
 	return Empty;
 }
 
-Value HostsTable::NotificationPeriodAccessor(const Value& row)
-{
-	Host::Ptr host = static_cast<Host::Ptr>(row);
-
-	if (!host)
-		return Empty;
-
-	return CompatUtility::GetCheckableNotificationNotificationPeriod(host);
-}
-
 Value HostsTable::CheckPeriodAccessor(const Value& row)
 {
 	Host::Ptr host = static_cast<Host::Ptr>(row);
@@ -898,7 +887,7 @@ Value HostsTable::ContactsAccessor(const Value& row)
 
 	Array::Ptr contact_names = new Array();
 
-	BOOST_FOREACH(const User::Ptr& user, CompatUtility::GetCheckableNotificationUsers(host)) {
+	for (const User::Ptr& user : CompatUtility::GetCheckableNotificationUsers(host)) {
 		contact_names->Add(user->GetName());
 	}
 
@@ -914,7 +903,7 @@ Value HostsTable::DowntimesAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Downtime::Ptr& downtime, host->GetDowntimes()) {
+	for (const Downtime::Ptr& downtime : host->GetDowntimes()) {
 		if (downtime->IsExpired())
 			continue;
 
@@ -933,7 +922,7 @@ Value HostsTable::DowntimesWithInfoAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Downtime::Ptr& downtime, host->GetDowntimes()) {
+	for (const Downtime::Ptr& downtime : host->GetDowntimes()) {
 		if (downtime->IsExpired())
 			continue;
 
@@ -955,7 +944,7 @@ Value HostsTable::CommentsAccessor(const Value& row)
 		return Empty;
 
 	Array::Ptr results = new Array();
-	BOOST_FOREACH(const Comment::Ptr& comment, host->GetComments()) {
+	for (const Comment::Ptr& comment : host->GetComments()) {
 		if (comment->IsExpired())
 			continue;
 
@@ -974,7 +963,7 @@ Value HostsTable::CommentsWithInfoAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Comment::Ptr& comment, host->GetComments()) {
+	for (const Comment::Ptr& comment : host->GetComments()) {
 		if (comment->IsExpired())
 			continue;
 
@@ -997,7 +986,7 @@ Value HostsTable::CommentsWithExtraInfoAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Comment::Ptr& comment, host->GetComments()) {
+	for (const Comment::Ptr& comment : host->GetComments()) {
 		if (comment->IsExpired())
 			continue;
 
@@ -1033,7 +1022,7 @@ Value HostsTable::CustomVariableNamesAccessor(const Value& row)
 	Array::Ptr cv = new Array();
 
 	ObjectLock olock(vars);
-	BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
+	for (const Dictionary::Pair& kv : vars) {
 		cv->Add(kv.first);
 	}
 
@@ -1060,7 +1049,7 @@ Value HostsTable::CustomVariableValuesAccessor(const Value& row)
 	Array::Ptr cv = new Array();
 
 	ObjectLock olock(vars);
-	BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
+	for (const Dictionary::Pair& kv : vars) {
 		if (kv.second.IsObjectType<Array>() || kv.second.IsObjectType<Dictionary>())
 			cv->Add(JsonEncode(kv.second));
 		else
@@ -1090,7 +1079,7 @@ Value HostsTable::CustomVariablesAccessor(const Value& row)
 	Array::Ptr cv = new Array();
 
 	ObjectLock olock(vars);
-	BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
+	for (const Dictionary::Pair& kv : vars) {
 		Array::Ptr key_val = new Array();
 		key_val->Add(kv.first);
 
@@ -1125,7 +1114,7 @@ Value HostsTable::CVIsJsonAccessor(const Value& row)
 	bool cv_is_json = false;
 
 	ObjectLock olock(vars);
-	BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
+	for (const Dictionary::Pair& kv : vars) {
 		if (kv.second.IsObjectType<Array>() || kv.second.IsObjectType<Dictionary>())
 			cv_is_json = true;
 	}
@@ -1142,7 +1131,7 @@ Value HostsTable::ParentsAccessor(const Value& row)
 
 	Array::Ptr parents = new Array();
 
-	BOOST_FOREACH(const Checkable::Ptr& parent, host->GetParents()) {
+	for (const Checkable::Ptr& parent : host->GetParents()) {
 		Host::Ptr parent_host = dynamic_pointer_cast<Host>(parent);
 
 		if (!parent_host)
@@ -1163,7 +1152,7 @@ Value HostsTable::ChildsAccessor(const Value& row)
 
 	Array::Ptr childs = new Array();
 
-	BOOST_FOREACH(const Checkable::Ptr& child, host->GetChildren()) {
+	for (const Checkable::Ptr& child : host->GetChildren()) {
 		Host::Ptr child_host = dynamic_pointer_cast<Host>(child);
 
 		if (!child_host)
@@ -1195,7 +1184,7 @@ Value HostsTable::WorstServiceStateAccessor(const Value& row)
 
 	Value worst_service = ServiceOK;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetState() > worst_service)
 			worst_service = service->GetState();
 	}
@@ -1212,7 +1201,7 @@ Value HostsTable::NumServicesOkAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetState() == ServiceOK)
 			num_services++;
 	}
@@ -1229,7 +1218,7 @@ Value HostsTable::NumServicesWarnAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetState() == ServiceWarning)
 			num_services++;
 	}
@@ -1246,7 +1235,7 @@ Value HostsTable::NumServicesCritAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetState() == ServiceCritical)
 			num_services++;
 	}
@@ -1263,7 +1252,7 @@ Value HostsTable::NumServicesUnknownAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetState() == ServiceUnknown)
 			num_services++;
 	}
@@ -1280,7 +1269,7 @@ Value HostsTable::NumServicesPendingAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (!service->GetLastCheckResult())
 			num_services++;
 	}
@@ -1297,7 +1286,7 @@ Value HostsTable::WorstServiceHardStateAccessor(const Value& row)
 
 	Value worst_service = ServiceOK;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetStateType() == StateTypeHard) {
 			if (service->GetState() > worst_service)
 				worst_service = service->GetState();
@@ -1316,7 +1305,7 @@ Value HostsTable::NumServicesHardOkAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetStateType() == StateTypeHard && service->GetState() == ServiceOK)
 			num_services++;
 	}
@@ -1333,7 +1322,7 @@ Value HostsTable::NumServicesHardWarnAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetStateType() == StateTypeHard && service->GetState() == ServiceWarning)
 			num_services++;
 	}
@@ -1350,7 +1339,7 @@ Value HostsTable::NumServicesHardCritAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetStateType() == StateTypeHard && service->GetState() == ServiceCritical)
 			num_services++;
 	}
@@ -1367,7 +1356,7 @@ Value HostsTable::NumServicesHardUnknownAccessor(const Value& row)
 
 	int num_services = 0;
 
-	BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+	for (const Service::Ptr& service : host->GetServices()) {
 		if (service->GetStateType() == StateTypeHard && service->GetState() == ServiceUnknown)
 			num_services++;
 	}
@@ -1424,7 +1413,7 @@ Value HostsTable::ContactGroupsAccessor(const Value& row)
 
 	Array::Ptr contactgroup_names = new Array();
 
-	BOOST_FOREACH(const UserGroup::Ptr& usergroup, CompatUtility::GetCheckableNotificationUserGroups(host)) {
+	for (const UserGroup::Ptr& usergroup : CompatUtility::GetCheckableNotificationUserGroups(host)) {
 		contactgroup_names->Add(usergroup->GetName());
 	}
 
@@ -1443,7 +1432,7 @@ Value HostsTable::ServicesAccessor(const Value& row)
 	Array::Ptr services = new Array();
 	services->Reserve(rservices.size());
 
-	BOOST_FOREACH(const Service::Ptr& service, rservices) {
+	for (const Service::Ptr& service : rservices) {
 		services->Add(service->GetShortName());
 	}
 
@@ -1462,7 +1451,7 @@ Value HostsTable::ServicesWithStateAccessor(const Value& row)
 	Array::Ptr services = new Array();
 	services->Reserve(rservices.size());
 
-	BOOST_FOREACH(const Service::Ptr& service, rservices) {
+	for (const Service::Ptr& service : rservices) {
 		Array::Ptr svc_add = new Array();
 
 		svc_add->Add(service->GetShortName());
@@ -1486,7 +1475,7 @@ Value HostsTable::ServicesWithInfoAccessor(const Value& row)
 	Array::Ptr services = new Array();
 	services->Reserve(rservices.size());
 
-	BOOST_FOREACH(const Service::Ptr& service, rservices) {
+	for (const Service::Ptr& service : rservices) {
 		Array::Ptr svc_add = new Array();
 
 		svc_add->Add(service->GetShortName());
