@@ -36,7 +36,6 @@
 #include "base/json.hpp"
 #include "base/convert.hpp"
 #include "base/utility.hpp"
-#include <boost/foreach.hpp>
 #include <boost/tuple/tuple.hpp>
 #include <boost/algorithm/string/replace.hpp>
 
@@ -61,7 +60,7 @@ void ServicesTable::AddColumns(Table *table, const String& prefix,
 	table->AddColumn(prefix + "plugin_output", Column(&ServicesTable::PluginOutputAccessor, objectAccessor));
 	table->AddColumn(prefix + "long_plugin_output", Column(&ServicesTable::LongPluginOutputAccessor, objectAccessor));
 	table->AddColumn(prefix + "perf_data", Column(&ServicesTable::PerfDataAccessor, objectAccessor));
-	table->AddColumn(prefix + "notification_period", Column(&ServicesTable::NotificationPeriodAccessor, objectAccessor));
+	table->AddColumn(prefix + "notification_period", Column(&Table::EmptyStringAccessor, objectAccessor));
 	table->AddColumn(prefix + "check_period", Column(&ServicesTable::CheckPeriodAccessor, objectAccessor));
 	table->AddColumn(prefix + "notes", Column(&ServicesTable::NotesAccessor, objectAccessor));
 	table->AddColumn(prefix + "notes_expanded", Column(&ServicesTable::NotesExpandedAccessor, objectAccessor));
@@ -168,19 +167,19 @@ String ServicesTable::GetPrefix(void) const
 void ServicesTable::FetchRows(const AddRowFunction& addRowFn)
 {
 	if (GetGroupByType() == LivestatusGroupByServiceGroup) {
-		BOOST_FOREACH(const ServiceGroup::Ptr& sg, ConfigType::GetObjectsByType<ServiceGroup>()) {
-			BOOST_FOREACH(const Service::Ptr& service, sg->GetMembers()) {
+		for (const ServiceGroup::Ptr& sg : ConfigType::GetObjectsByType<ServiceGroup>()) {
+			for (const Service::Ptr& service : sg->GetMembers()) {
 				/* the caller must know which groupby type and value are set for this row */
 				if (!addRowFn(service, LivestatusGroupByServiceGroup, sg))
 					return;
 			}
 		}
 	} else if (GetGroupByType() == LivestatusGroupByHostGroup) {
-		BOOST_FOREACH(const HostGroup::Ptr& hg, ConfigType::GetObjectsByType<HostGroup>()) {
+		for (const HostGroup::Ptr& hg : ConfigType::GetObjectsByType<HostGroup>()) {
 			ObjectLock ylock(hg);
-			BOOST_FOREACH(const Host::Ptr& host, hg->GetMembers()) {
+			for (const Host::Ptr& host : hg->GetMembers()) {
 				ObjectLock ylock(host);
-				BOOST_FOREACH(const Service::Ptr& service, host->GetServices()) {
+				for (const Service::Ptr& service : host->GetServices()) {
 					/* the caller must know which groupby type and value are set for this row */
 					if (!addRowFn(service, LivestatusGroupByHostGroup, hg))
 						return;
@@ -188,7 +187,7 @@ void ServicesTable::FetchRows(const AddRowFunction& addRowFn)
 			}
 		}
 	} else {
-		BOOST_FOREACH(const Service::Ptr& service, ConfigType::GetObjectsByType<Service>()) {
+		for (const Service::Ptr& service : ConfigType::GetObjectsByType<Service>()) {
 			if (!addRowFn(service, LivestatusGroupByNone, Empty))
 				return;
 		}
@@ -349,16 +348,6 @@ Value ServicesTable::PerfDataAccessor(const Value& row)
 	return perfdata;
 }
 
-Value ServicesTable::NotificationPeriodAccessor(const Value& row)
-{
-	Service::Ptr service = static_cast<Service::Ptr>(row);
-
-	if (!service)
-		return Empty;
-
-	return CompatUtility::GetCheckableNotificationNotificationPeriod(service);
-}
-
 Value ServicesTable::CheckPeriodAccessor(const Value& row)
 {
 	Service::Ptr service = static_cast<Service::Ptr>(row);
@@ -936,7 +925,7 @@ Value ServicesTable::ContactsAccessor(const Value& row)
 
 	Array::Ptr contact_names = new Array();
 
-	BOOST_FOREACH(const User::Ptr& user, CompatUtility::GetCheckableNotificationUsers(service)) {
+	for (const User::Ptr& user : CompatUtility::GetCheckableNotificationUsers(service)) {
 		contact_names->Add(user->GetName());
 	}
 
@@ -952,7 +941,7 @@ Value ServicesTable::DowntimesAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Downtime::Ptr& downtime, service->GetDowntimes()) {
+	for (const Downtime::Ptr& downtime : service->GetDowntimes()) {
 		if (downtime->IsExpired())
 			continue;
 
@@ -971,7 +960,7 @@ Value ServicesTable::DowntimesWithInfoAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Downtime::Ptr& downtime, service->GetDowntimes()) {
+	for (const Downtime::Ptr& downtime : service->GetDowntimes()) {
 		if (downtime->IsExpired())
 			continue;
 
@@ -994,7 +983,7 @@ Value ServicesTable::CommentsAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Comment::Ptr& comment, service->GetComments()) {
+	for (const Comment::Ptr& comment : service->GetComments()) {
 		if (comment->IsExpired())
 			continue;
 
@@ -1013,7 +1002,7 @@ Value ServicesTable::CommentsWithInfoAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Comment::Ptr& comment, service->GetComments()) {
+	for (const Comment::Ptr& comment : service->GetComments()) {
 		if (comment->IsExpired())
 			continue;
 
@@ -1036,7 +1025,7 @@ Value ServicesTable::CommentsWithExtraInfoAccessor(const Value& row)
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Comment::Ptr& comment, service->GetComments()) {
+	for (const Comment::Ptr& comment : service->GetComments()) {
 		if (comment->IsExpired())
 			continue;
 
@@ -1072,7 +1061,7 @@ Value ServicesTable::CustomVariableNamesAccessor(const Value& row)
 	Array::Ptr cv = new Array();
 
 	ObjectLock olock(vars);
-	BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
+	for (const Dictionary::Pair& kv : vars) {
 		cv->Add(kv.first);
 	}
 
@@ -1099,7 +1088,7 @@ Value ServicesTable::CustomVariableValuesAccessor(const Value& row)
 	Array::Ptr cv = new Array();
 
 	ObjectLock olock(vars);
-	BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
+	for (const Dictionary::Pair& kv : vars) {
 		if (kv.second.IsObjectType<Array>() || kv.second.IsObjectType<Dictionary>())
 			cv->Add(JsonEncode(kv.second));
 		else
@@ -1129,7 +1118,7 @@ Value ServicesTable::CustomVariablesAccessor(const Value& row)
 	Array::Ptr cv = new Array();
 
 	ObjectLock olock(vars);
-	BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
+	for (const Dictionary::Pair& kv : vars) {
 		Array::Ptr key_val = new Array();
 		key_val->Add(kv.first);
 
@@ -1164,7 +1153,7 @@ Value ServicesTable::CVIsJsonAccessor(const Value& row)
 	bool cv_is_json = false;
 
 	ObjectLock olock(vars);
-	BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
+	for (const Dictionary::Pair& kv : vars) {
 		if (kv.second.IsObjectType<Array>() || kv.second.IsObjectType<Dictionary>())
 			cv_is_json = true;
 	}
@@ -1196,7 +1185,7 @@ Value ServicesTable::ContactGroupsAccessor(const Value& row)
 
 	Array::Ptr contactgroup_names = new Array();
 
-	BOOST_FOREACH(const UserGroup::Ptr& usergroup, CompatUtility::GetCheckableNotificationUserGroups(service)) {
+	for (const UserGroup::Ptr& usergroup : CompatUtility::GetCheckableNotificationUserGroups(service)) {
 		contactgroup_names->Add(usergroup->GetName());
 	}
 
