@@ -30,11 +30,10 @@
 #include "base/function.hpp"
 #include "base/configtype.hpp"
 #include <boost/algorithm/string/join.hpp>
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
-REGISTER_SCRIPTFUNCTION(ClusterCheck, &ClusterCheckTask::ScriptFunc);
+REGISTER_SCRIPTFUNCTION_NS(Internal, ClusterCheck, &ClusterCheckTask::ScriptFunc);
 
 void ClusterCheckTask::ScriptFunc(const Checkable::Ptr& checkable, const CheckResult::Ptr& cr,
     const Dictionary::Ptr& resolvedMacros, bool useResolvedMacros)
@@ -82,7 +81,7 @@ String ClusterCheckTask::FormatArray(const Array::Ptr& arr)
 
 	if (arr) {
 		ObjectLock olock(arr);
-		BOOST_FOREACH(const Value& value, arr) {
+		for (const Value& value : arr) {
 			if (first)
 				first = false;
 			else
