@@ -27,7 +27,6 @@
 #include "base/utility.hpp"
 #include "base/exception.hpp"
 #include "base/statsfunction.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -39,7 +38,7 @@ void NotificationComponent::StatsFunc(const Dictionary::Ptr& status, const Array
 {
 	Dictionary::Ptr nodes = new Dictionary();
 
-	BOOST_FOREACH(const NotificationComponent::Ptr& notification_component, ConfigType::GetObjectsByType<NotificationComponent>()) {
+	for (const NotificationComponent::Ptr& notification_component : ConfigType::GetObjectsByType<NotificationComponent>()) {
 		nodes->Set(notification_component->GetName(), 1); //add more stats
 	}
 
@@ -71,19 +70,19 @@ void NotificationComponent::NotificationTimerHandler(void)
 {
 	double now = Utility::GetTime();
 
-	BOOST_FOREACH(const Notification::Ptr& notification, ConfigType::GetObjectsByType<Notification>()) {
+	for (const Notification::Ptr& notification : ConfigType::GetObjectsByType<Notification>()) {
 		if (!notification->IsActive())
 			continue;
 
+		if (notification->IsPaused() && GetEnableHA())
+			continue;
+
 		Checkable::Ptr checkable = notification->GetCheckable();
 
-		if (checkable->IsPaused() && GetEnableHA())
-			continue;
-
 		if (!IcingaApplication::GetInstance()->GetEnableNotifications() || !checkable->GetEnableNotifications())
 			continue;
 
-		if (notification->GetInterval() <= 0 && notification->GetLastProblemNotification() > checkable->GetLastHardStateChange())
+		if (notification->GetInterval() <= 0 && notification->GetNoMoreNotifications())
 			continue;
 
 		if (notification->GetNextNotification() > now)
@@ -115,8 +114,8 @@ void NotificationComponent::NotificationTimerHandler(void)
 
 		try {
 			Log(LogNotice, "NotificationComponent")
-			    << "Attempting to send reminder notification for object '" << checkable->GetName() << "'";
-			notification->BeginExecuteNotification(NotificationProblem, checkable->GetLastCheckResult(), false);
+			    << "Attempting to send reminder notification '" << notification->GetName() << "'";
+			notification->BeginExecuteNotification(NotificationProblem, checkable->GetLastCheckResult(), false, true);
 		} catch (const std::exception& ex) {
 			Log(LogWarning, "NotificationComponent")
 			    << "Exception occured during notification for object '"
@@ -131,8 +130,5 @@ void NotificationComponent::NotificationTimerHandler(void)
 void NotificationComponent::SendNotificationsHandler(const Checkable::Ptr& checkable, NotificationType type,
     const CheckResult::Ptr& cr, const String& author, const String& text)
 {
-	if (checkable->IsPaused() && GetEnableHA())
-		return;
-
 	checkable->SendNotifications(type, cr, author, text);
 }
