@@ -73,20 +73,16 @@ public:
 };
 
 #define REGISTER_APIACTION(name, types, callback) \
-	namespace { namespace UNIQUE_NAME(apia) { namespace apia ## name { \
-		void RegisterAction(void) \
-		{ \
-			String registerName = #name; \
-			boost::algorithm::replace_all(registerName, "_", "-"); \
-			std::vector<String> registerTypes; \
-			String typeNames = types; \
-			if (!typeNames.IsEmpty()) \
-				boost::algorithm::split(registerTypes, typeNames, boost::is_any_of(";")); \
-			ApiAction::Ptr action = new ApiAction(registerTypes, callback); \
-			ApiActionRegistry::GetInstance()->Register(registerName, action); \
-		} \
-		INITIALIZE_ONCE(RegisterAction); \
-	} } }
+	INITIALIZE_ONCE([]() { \
+		String registerName = #name; \
+		boost::algorithm::replace_all(registerName, "_", "-"); \
+		std::vector<String> registerTypes; \
+		String typeNames = types; \
+		if (!typeNames.IsEmpty()) \
+			boost::algorithm::split(registerTypes, typeNames, boost::is_any_of(";")); \
+		ApiAction::Ptr action = new ApiAction(registerTypes, callback); \
+		ApiActionRegistry::GetInstance()->Register(registerName, action); \
+	})
 
 }
 
