@@ -67,14 +67,10 @@ public:
 };
 
 #define REGISTER_APIFUNCTION(name, ns, callback) \
-	namespace { namespace UNIQUE_NAME(apif) { namespace apif ## name { \
-		void RegisterFunction(void) \
-		{ \
-			ApiFunction::Ptr func = new ApiFunction(callback); \
-			ApiFunctionRegistry::GetInstance()->Register(#ns "::" #name, func); \
-		} \
-		INITIALIZE_ONCE(RegisterFunction); \
-	} } }
+	INITIALIZE_ONCE([]() { \
+		ApiFunction::Ptr func = new ApiFunction(callback); \
+		ApiFunctionRegistry::GetInstance()->Register(#ns "::" #name, func); \
+	})
 
 }
 
