@@ -49,7 +49,10 @@ REGISTER_APIFUNCTION(Hello, icinga, &ApiListener::HelloAPIHandler);
 
 ApiListener::ApiListener(void)
 	: m_SyncQueue(0, 4), m_LogMessageCount(0)
-{ }
+{
+	m_RelayQueue.SetName("ApiListener, RelayQueue");
+	m_SyncQueue.SetName("ApiListener, SyncQueue");
+}
 
 void ApiListener::OnConfigLoaded(void)
 {
@@ -92,6 +95,23 @@ void ApiListener::OnConfigLoaded(void)
 			    + GetCrlPath() + "'.", GetDebugInfo()));
 		}
 	}
+
+	if (!GetCipherList().IsEmpty()) {
+		try {
+			SetCipherListToSSLContext(m_SSLContext, GetCipherList());
+		} catch (const std::exception&) {
+			BOOST_THROW_EXCEPTION(ScriptError("Cannot set cipher list to SSL context for cipher list: '"
+			    + GetCipherList() + "'.", GetDebugInfo()));
+		}
+	}
+
+	if (!GetTlsProtocolmin().IsEmpty()){
+		try {
+			SetTlsProtocolminToSSLContext(m_SSLContext, GetTlsProtocolmin());
+		} catch (const std::exception&) {
+			BOOST_THROW_EXCEPTION(ScriptError("Cannot set minimum TLS protocol version to SSL context with tls_protocolmin: '" + GetTlsProtocolmin() + "'.", GetDebugInfo()));
+		}
+	}
 }
 
 void ApiListener::OnAllConfigLoaded(void)
@@ -130,9 +150,28 @@ void ApiListener::Start(bool runtimeCreated)
 	m_Timer->Start();
 	m_Timer->Reschedule(0);
 
+	m_ReconnectTimer = new Timer();
+	m_ReconnectTimer->OnTimerExpired.connect(boost::bind(&ApiListener::ApiReconnectTimerHandler, this));
+	m_ReconnectTimer->SetInterval(60);
+	m_ReconnectTimer->Start();
+	m_ReconnectTimer->Reschedule(0);
+
+	m_AuthorityTimer = new Timer();
+	m_AuthorityTimer->OnTimerExpired.connect(boost::bind(&ApiListener::UpdateObjectAuthority));
+	m_AuthorityTimer->SetInterval(30);
+	m_AuthorityTimer->Start();
+
 	OnMasterChanged(true);
 }
 
+void ApiListener::Stop(bool runtimeDeleted)
+{
+	ObjectImpl<ApiListener>::Stop(runtimeDeleted);
+
+	boost::mutex::scoped_lock lock(m_LogLock);
+	CloseLogFile();
+}
+
 ApiListener::Ptr ApiListener::GetInstance(void)
 {
 	return m_Instance;
@@ -152,7 +191,7 @@ Endpoint::Ptr ApiListener::GetMaster(void) const
 
 	std::vector<String> names;
 
-	BOOST_FOREACH(const Endpoint::Ptr& endpoint, zone->GetEndpoints())
+	for (const Endpoint::Ptr& endpoint : zone->GetEndpoints())
 		if (endpoint->GetConnected() || endpoint->GetName() == GetIdentity())
 			names.push_back(endpoint->GetName());
 
@@ -288,6 +327,15 @@ void ApiListener::NewClientHandlerInternal(const Socket::Ptr& client, const Stri
 {
 	CONTEXT("Handling new API client connection");
 
+	String conninfo;
+
+	if (role == RoleClient)
+		conninfo = "to";
+	else
+		conninfo = "from";
+
+	conninfo += " " + client->GetPeerAddress();
+
 	TlsStream::Ptr tlsStream;
 
 	{
@@ -295,7 +343,8 @@ void ApiListener::NewClientHandlerInternal(const Socket::Ptr& client, const Stri
 		try {
 			tlsStream = new TlsStream(client, hostname, role, m_SSLContext);
 		} catch (const std::exception&) {
-			Log(LogCritical, "ApiListener", "Cannot create TLS stream from client connection.");
+			Log(LogCritical, "ApiListener")
+			    << "Cannot create TLS stream from client connection (" << conninfo << ")";
 			return;
 		}
 	}
@@ -303,7 +352,8 @@ void ApiListener::NewClientHandlerInternal(const Socket::Ptr& client, const Stri
 	try {
 		tlsStream->Handshake();
 	} catch (const std::exception& ex) {
-		Log(LogCritical, "ApiListener", "Client TLS handshake failed");
+		Log(LogCritical, "ApiListener")
+		    << "Client TLS handshake failed (" << conninfo << ")";
 		return;
 	}
 
@@ -330,21 +380,28 @@ void ApiListener::NewClientHandlerInternal(const Socket::Ptr& client, const Stri
 				return;
 			} else if (!verify_ok) {
 				Log(LogWarning, "ApiListener")
-					<< "Peer certificate for endpoint '" << hostname
-					<< "' is not signed by the certificate authority.";
+					<< "Certificate validation failed for endpoint '" << hostname
+					<< "': " << tlsStream->GetVerifyError();
 				return;
 			}
 		}
 
-		Log(LogInformation, "ApiListener")
-		    << "New client connection for identity '" << identity << "'"
-		    << (verify_ok ? "" : " (client certificate not signed by CA)");
-
 		if (verify_ok)
 			endpoint = Endpoint::GetByName(identity);
+
+		{
+			Log log(LogInformation, "ApiListener");
+
+			log << "New client connection for identity '" << identity << "' " << conninfo;
+
+			if (!verify_ok)
+				log << " (certificate validation failed: " << tlsStream->GetVerifyError() << ")";
+			else if (!endpoint)
+				log << " (no Endpoint object found for identity)";
+		}
 	} else {
 		Log(LogInformation, "ApiListener")
-		    << "New client connection (no client certificate)";
+		    << "New client connection " << conninfo << " (no client certificate)";
 	}
 
 	ClientType ctype;
@@ -360,7 +417,8 @@ void ApiListener::NewClientHandlerInternal(const Socket::Ptr& client, const Stri
 		tlsStream->WaitForData(5);
 
 		if (!tlsStream->IsDataAvailable()) {
-			Log(LogWarning, "ApiListener", "No data received on new API connection.");
+			Log(LogWarning, "ApiListener")
+			    << "No data received on new API connection for identity '" << identity << "'. Ensure that the remote endpoints are properly configured in a cluster setup.";
 			return;
 		}
 
@@ -453,10 +511,10 @@ void ApiListener::ApiTimerHandler(void)
 	Utility::Glob(GetApiDir() + "log/*", boost::bind(&ApiListener::LogGlobHandler, boost::ref(files), _1), GlobFile);
 	std::sort(files.begin(), files.end());
 
-	BOOST_FOREACH(int ts, files) {
+	for (int ts : files) {
 		bool need = false;
 
-		BOOST_FOREACH(const Endpoint::Ptr& endpoint, ConfigType::GetObjectsByType<Endpoint>()) {
+		for (const Endpoint::Ptr& endpoint : ConfigType::GetObjectsByType<Endpoint>()) {
 			if (endpoint == GetLocalEndpoint())
 				continue;
 
@@ -477,9 +535,49 @@ void ApiListener::ApiTimerHandler(void)
 		}
 	}
 
+	for (const Endpoint::Ptr& endpoint : ConfigType::GetObjectsByType<Endpoint>()) {
+		if (!endpoint->GetConnected())
+			continue;
+
+		double ts = endpoint->GetRemoteLogPosition();
+
+		if (ts == 0)
+			continue;
+
+		Dictionary::Ptr lparams = new Dictionary();
+		lparams->Set("log_position", ts);
+
+		Dictionary::Ptr lmessage = new Dictionary();
+		lmessage->Set("jsonrpc", "2.0");
+		lmessage->Set("method", "log::SetLogPosition");
+		lmessage->Set("params", lparams);
+
+		double maxTs = 0;
+
+		for (const JsonRpcConnection::Ptr& client : endpoint->GetClients()) {
+			if (client->GetTimestamp() > maxTs)
+				maxTs = client->GetTimestamp();
+		}
+
+		for (const JsonRpcConnection::Ptr& client : endpoint->GetClients()) {
+			if (client->GetTimestamp() != maxTs)
+				client->Disconnect();
+			else
+				client->SendMessage(lmessage);
+		}
+
+		Log(LogNotice, "ApiListener")
+		    << "Setting log position for identity '" << endpoint->GetName() << "': "
+		    << Utility::FormatDateTime("%Y/%m/%d %H:%M:%S", ts);
+	}
+
+}
+
+void ApiListener::ApiReconnectTimerHandler(void)
+{
 	Zone::Ptr my_zone = Zone::GetLocalZone();
 
-	BOOST_FOREACH(const Zone::Ptr& zone, ConfigType::GetObjectsByType<Zone>()) {
+	for (const Zone::Ptr& zone : ConfigType::GetObjectsByType<Zone>()) {
 		/* don't connect to global zones */
 		if (zone->GetGlobal())
 			continue;
@@ -492,7 +590,7 @@ void ApiListener::ApiTimerHandler(void)
 			continue;
 		}
 
-		BOOST_FOREACH(const Endpoint::Ptr& endpoint, zone->GetEndpoints()) {
+		for (const Endpoint::Ptr& endpoint : zone->GetEndpoints()) {
 			/* don't connect to ourselves */
 			if (endpoint == GetLocalEndpoint()) {
 				Log(LogDebug, "ApiListener")
@@ -529,42 +627,6 @@ void ApiListener::ApiTimerHandler(void)
 		}
 	}
 
-	BOOST_FOREACH(const Endpoint::Ptr& endpoint, ConfigType::GetObjectsByType<Endpoint>()) {
-		if (!endpoint->GetConnected())
-			continue;
-
-		double ts = endpoint->GetRemoteLogPosition();
-
-		if (ts == 0)
-			continue;
-
-		Dictionary::Ptr lparams = new Dictionary();
-		lparams->Set("log_position", ts);
-
-		Dictionary::Ptr lmessage = new Dictionary();
-		lmessage->Set("jsonrpc", "2.0");
-		lmessage->Set("method", "log::SetLogPosition");
-		lmessage->Set("params", lparams);
-
-		double maxTs = 0;
-
-		BOOST_FOREACH(const JsonRpcConnection::Ptr& client, endpoint->GetClients()) {
-			if (client->GetTimestamp() > maxTs)
-				maxTs = client->GetTimestamp();
-		}
-
-		BOOST_FOREACH(const JsonRpcConnection::Ptr& client, endpoint->GetClients()) {
-			if (client->GetTimestamp() != maxTs)
-				client->Disconnect();
-			else
-				client->SendMessage(lmessage);
-		}
-
-		Log(LogNotice, "ApiListener")
-		    << "Setting log position for identity '" << endpoint->GetName() << "': "
-		    << Utility::FormatDateTime("%Y/%m/%d %H:%M:%S", ts);
-	}
-
 	Endpoint::Ptr master = GetMaster();
 
 	if (master)
@@ -572,7 +634,7 @@ void ApiListener::ApiTimerHandler(void)
 		    << "Current zone master: " << master->GetName();
 
 	std::vector<String> names;
-	BOOST_FOREACH(const Endpoint::Ptr& endpoint, ConfigType::GetObjectsByType<Endpoint>())
+	for (const Endpoint::Ptr& endpoint : ConfigType::GetObjectsByType<Endpoint>())
 		if (endpoint->GetConnected())
 			names.push_back(endpoint->GetName() + " (" + Convert::ToString(endpoint->GetClients().size()) + ")");
 
@@ -599,10 +661,13 @@ void ApiListener::PersistMessage(const Dictionary::Ptr& message, const ConfigObj
 	pmessage->Set("timestamp", ts);
 
 	pmessage->Set("message", JsonEncode(message));
-	Dictionary::Ptr secname = new Dictionary();
-	secname->Set("type", secobj->GetType()->GetName());
-	secname->Set("name", secobj->GetName());
-	pmessage->Set("secobj", secname);
+
+	if (secobj) {
+		Dictionary::Ptr secname = new Dictionary();
+		secname->Set("type", secobj->GetReflectionType()->GetName());
+		secname->Set("name", secobj->GetName());
+		pmessage->Set("secobj", secname);
+	}
 
 	boost::mutex::scoped_lock lock(m_LogLock);
 	if (m_LogFile) {
@@ -624,16 +689,16 @@ void ApiListener::SyncSendMessage(const Endpoint::Ptr& endpoint, const Dictionar
 
 	if (!endpoint->GetSyncing()) {
 		Log(LogNotice, "ApiListener")
-		    << "Sending message to '" << endpoint->GetName() << "'";
+		    << "Sending message '" << message->Get("method") << "' to '" << endpoint->GetName() << "'";
 
 		double maxTs = 0;
 
-		BOOST_FOREACH(const JsonRpcConnection::Ptr& client, endpoint->GetClients()) {
+		for (const JsonRpcConnection::Ptr& client : endpoint->GetClients()) {
 			if (client->GetTimestamp() > maxTs)
 				maxTs = client->GetTimestamp();
 		}
 
-		BOOST_FOREACH(const JsonRpcConnection::Ptr& client, endpoint->GetClients()) {
+		for (const JsonRpcConnection::Ptr& client : endpoint->GetClients()) {
 			if (client->GetTimestamp() != maxTs)
 				continue;
 
@@ -648,9 +713,13 @@ bool ApiListener::RelayMessageOne(const Zone::Ptr& targetZone, const MessageOrig
 
 	Zone::Ptr myZone = Zone::GetLocalZone();
 
-	/* only relay the message to a) the same zone, b) the parent zone and c) direct child zones */
-	if (targetZone != myZone && targetZone != myZone->GetParent() && targetZone->GetParent() != myZone)
+	/* only relay the message to a) the same zone, b) the parent zone and c) direct child zones. Exception is a global zone. */
+	if (!targetZone->GetGlobal() &&
+	    targetZone != myZone &&
+	    targetZone != myZone->GetParent() &&
+	    targetZone->GetParent() != myZone) {
 		return true;
+	}
 
 	Endpoint::Ptr myEndpoint = GetLocalEndpoint();
 
@@ -658,7 +727,23 @@ bool ApiListener::RelayMessageOne(const Zone::Ptr& targetZone, const MessageOrig
 
 	bool relayed = false, log_needed = false, log_done = false;
 
-	BOOST_FOREACH(const Endpoint::Ptr& endpoint, targetZone->GetEndpoints()) {
+	std::set<Endpoint::Ptr> targetEndpoints;
+
+	if (targetZone->GetGlobal()) {
+		targetEndpoints = myZone->GetEndpoints();
+
+		for (const Zone::Ptr& zone : ConfigType::GetObjectsByType<Zone>()) {
+			/* Fetch immediate child zone members */
+			if (zone->GetParent() == myZone) {
+				std::set<Endpoint::Ptr> endpoints = zone->GetEndpoints();
+				targetEndpoints.insert(endpoints.begin(), endpoints.end());
+			}
+		}
+	} else {
+		targetEndpoints = targetZone->GetEndpoints();
+	}
+
+	for (const Endpoint::Ptr& endpoint : targetEndpoints) {
 		/* don't relay messages to ourselves */
 		if (endpoint == GetLocalEndpoint())
 			continue;
@@ -707,7 +792,7 @@ bool ApiListener::RelayMessageOne(const Zone::Ptr& targetZone, const MessageOrig
 	if (!skippedEndpoints.empty()) {
 		double ts = message->Get("ts");
 
-		BOOST_FOREACH(const Endpoint::Ptr& endpoint, skippedEndpoints)
+		for (const Endpoint::Ptr& endpoint : skippedEndpoints)
 			endpoint->SetLocalLogPosition(ts);
 	}
 
@@ -742,7 +827,7 @@ void ApiListener::SyncRelayMessage(const MessageOrigin::Ptr& origin,
 
 	bool need_log = !RelayMessageOne(target_zone, origin, message, master);
 
-	BOOST_FOREACH(const Zone::Ptr& zone, target_zone->GetAllParents()) {
+	for (const Zone::Ptr& zone : target_zone->GetAllParents()) {
 		if (!RelayMessageOne(zone, origin, message, master))
 			need_log = true;
 	}
@@ -862,7 +947,7 @@ void ApiListener::ReplayLog(const JsonRpcConnection::Ptr& client)
 		Utility::Glob(GetApiDir() + "log/*", boost::bind(&ApiListener::LogGlobHandler, boost::ref(files), _1), GlobFile);
 		std::sort(files.begin(), files.end());
 
-		BOOST_FOREACH(int ts, files) {
+		for (int ts : files) {
 			String path = GetApiDir() + "log/" + Convert::ToString(ts);
 
 			if (ts < peer_ts)
@@ -903,12 +988,7 @@ void ApiListener::ReplayLog(const JsonRpcConnection::Ptr& client)
 				Dictionary::Ptr secname = pmessage->Get("secobj");
 
 				if (secname) {
-					ConfigType::Ptr dtype = ConfigType::GetByName(secname->Get("type"));
-
-					if (!dtype)
-						continue;
-
-					ConfigObject::Ptr secobj = dtype->GetObject(secname->Get("name"));
+					ConfigObject::Ptr secobj = ConfigObject::GetObject(secname->Get("type"), secname->Get("name"));
 
 					if (!secobj)
 						continue;
@@ -979,7 +1059,7 @@ void ApiListener::StatsFunc(const Dictionary::Ptr& status, const Array::Ptr& per
 	stats = listener->GetStatus();
 
 	ObjectLock olock(stats.second);
-	BOOST_FOREACH(const Dictionary::Pair& kv, stats.second)
+	for (const Dictionary::Pair& kv : stats.second)
 		perfdata->Add("'api_" + kv.first + "'=" + Convert::ToString(kv.second));
 
 	status->Set("api", stats.first);
@@ -1001,7 +1081,7 @@ std::pair<Dictionary::Ptr, Dictionary::Ptr> ApiListener::GetStatus(void)
 
 	Dictionary::Ptr connectedZones = new Dictionary();
 
-	BOOST_FOREACH(const Zone::Ptr& zone, ConfigType::GetObjectsByType<Zone>()) {
+	for (const Zone::Ptr& zone : ConfigType::GetObjectsByType<Zone>()) {
 		/* only check endpoints in a) the same zone b) our parent zone c) immediate child zones */
 		if (my_zone != zone && my_zone != zone->GetParent() && zone != my_zone->GetParent()) {
 			Log(LogDebug, "ApiListener")
@@ -1015,7 +1095,7 @@ std::pair<Dictionary::Ptr, Dictionary::Ptr> ApiListener::GetStatus(void)
 
 		Array::Ptr zoneEndpoints = new Array();
 
-		BOOST_FOREACH(const Endpoint::Ptr& endpoint, zone->GetEndpoints()) {
+		for (const Endpoint::Ptr& endpoint : zone->GetEndpoints()) {
 			zoneEndpoints->Add(endpoint->GetName());
 
 			if (endpoint->GetName() == GetIdentity())
@@ -1127,3 +1207,33 @@ Endpoint::Ptr ApiListener::GetLocalEndpoint(void) const
 {
 	return m_LocalEndpoint;
 }
+
+void ApiListener::ValidateTlsProtocolmin(const String& value, const ValidationUtils& utils)
+{
+	ObjectImpl<ApiListener>::ValidateTlsProtocolmin(value, utils);
+
+	if (value != SSL_TXT_TLSV1
+#ifdef SSL_TXT_TLSV1_1
+	    && value != SSL_TXT_TLSV1_1 &&
+	    value != SSL_TXT_TLSV1_2
+#endif /* SSL_TXT_TLSV1_1 */
+	    ) {
+		String message = "Invalid TLS version. Must be one of '" SSL_TXT_TLSV1 "'";
+#ifdef SSL_TXT_TLSV1_1
+		message += ", '" SSL_TXT_TLSV1_1 "' or '" SSL_TXT_TLSV1_2 "'";
+#endif /* SSL_TXT_TLSV1_1 */
+
+		BOOST_THROW_EXCEPTION(ValidationError(this, boost::assign::list_of("tls_protocolmin"), message));
+	}
+}
+
+bool ApiListener::IsHACluster(void)
+{
+	Zone::Ptr zone = Zone::GetLocalZone();
+
+	if (!zone)
+		return false;
+
+	return zone->IsSingleInstance();
+}
+
