@@ -48,24 +48,21 @@ static void ScriptFrameCleanupHandler(void)
 
 	typedef std::pair<String, ApiScriptFrame> KVPair;
 
-	BOOST_FOREACH(const KVPair& kv, l_ApiScriptFrames) {
+	for (const KVPair& kv : l_ApiScriptFrames) {
 		if (kv.second.Seen < Utility::GetTime() - 1800)
 			cleanup_keys.push_back(kv.first);
 	}
 
-	BOOST_FOREACH(const String& key, cleanup_keys)
+	for (const String& key : cleanup_keys)
 		l_ApiScriptFrames.erase(key);
 }
 
-static void InitScriptFrameCleanup(void)
-{
+INITIALIZE_ONCE([]() {
 	l_FrameCleanupTimer = new Timer();
 	l_FrameCleanupTimer->OnTimerExpired.connect(boost::bind(ScriptFrameCleanupHandler));
 	l_FrameCleanupTimer->SetInterval(30);
 	l_FrameCleanupTimer->Start();
-}
-
-INITIALIZE_ONCE(InitScriptFrameCleanup);
+});
 
 bool ConsoleHandler::HandleRequest(const ApiUser::Ptr& user, HttpRequest& request, HttpResponse& response, const Dictionary::Ptr& params)
 {
@@ -102,7 +99,7 @@ bool ConsoleHandler::HandleRequest(const ApiUser::Ptr& user, HttpRequest& reques
 bool ConsoleHandler::ExecuteScriptHelper(HttpRequest& request, HttpResponse& response,
     const String& command, const String& session, bool sandboxed)
 {
-	Log(LogInformation, "Console")
+	Log(LogNotice, "Console")
 	    << "Executing expression: " << command;
 
 	ApiScriptFrame& lsf = l_ApiScriptFrames[session];
@@ -215,28 +212,77 @@ static void AddSuggestion(std::vector<String>& matches, const String& word, cons
 	matches.push_back(suggestion);
 }
 
+static void AddSuggestions(std::vector<String>& matches, const String& word, const String& pword, bool withFields, const Value& value)
+{
+	String prefix;
+
+	if (!pword.IsEmpty())
+		prefix = pword + ".";
+
+	if (value.IsObjectType<Dictionary>()) {
+		Dictionary::Ptr dict = value;
+
+		ObjectLock olock(dict);
+		for (const Dictionary::Pair& kv : dict) {
+			AddSuggestion(matches, word, prefix + kv.first);
+		}
+	}
+
+	if (withFields) {
+		Type::Ptr type = value.GetReflectionType();
+
+		for (int i = 0; i < type->GetFieldCount(); i++) {
+			Field field = type->GetFieldInfo(i);
+
+			AddSuggestion(matches, word, prefix + field.Name);
+		}
+
+		while (type) {
+			Object::Ptr prototype = type->GetPrototype();
+			Dictionary::Ptr dict = dynamic_pointer_cast<Dictionary>(prototype);
+
+			if (dict) {
+				ObjectLock olock(dict);
+				for (const Dictionary::Pair& kv : dict) {
+					AddSuggestion(matches, word, prefix + kv.first);
+				}
+			}
+
+			type = type->GetBaseType();
+		}
+	}
+}
+
 std::vector<String> ConsoleHandler::GetAutocompletionSuggestions(const String& word, ScriptFrame& frame)
 {	
 	std::vector<String> matches;
 
-	BOOST_FOREACH(const String& keyword, ConfigWriter::GetKeywords()) {
+	for (const String& keyword : ConfigWriter::GetKeywords()) {
 		AddSuggestion(matches, word, keyword);
 	}
 
 	{
 		ObjectLock olock(frame.Locals);
-		BOOST_FOREACH(const Dictionary::Pair& kv, frame.Locals) {
+		for (const Dictionary::Pair& kv : frame.Locals) {
 			AddSuggestion(matches, word, kv.first);
 		}
 	}
 
 	{
 		ObjectLock olock(ScriptGlobal::GetGlobals());
-		BOOST_FOREACH(const Dictionary::Pair& kv, ScriptGlobal::GetGlobals()) {
+		for (const Dictionary::Pair& kv : ScriptGlobal::GetGlobals()) {
 			AddSuggestion(matches, word, kv.first);
 		}
 	}
 
+	{
+		Array::Ptr imports = ScriptFrame::GetImports();
+		ObjectLock olock(imports);
+		for (const Value& import : imports) {
+			AddSuggestions(matches, word, "", false, import);
+		}
+	}
+
 	String::SizeType cperiod = word.RFind(".");
 
 	if (cperiod != String::NPos) {
@@ -250,36 +296,8 @@ std::vector<String> ConsoleHandler::GetAutocompletionSuggestions(const String& w
 			if (expr)
 				value = expr->Evaluate(frame);
 
-			if (value.IsObjectType<Dictionary>()) {
-				Dictionary::Ptr dict = value;
+			AddSuggestions(matches, word, pword, true, value);
 
-				ObjectLock olock(dict);
-				BOOST_FOREACH(const Dictionary::Pair& kv, dict) {
-					AddSuggestion(matches, word, pword + "." + kv.first);
-				}
-			}
-
-			Type::Ptr type = value.GetReflectionType();
-
-			for (int i = 0; i < type->GetFieldCount(); i++) {
-				Field field = type->GetFieldInfo(i);
-
-				AddSuggestion(matches, word, pword + "." + field.Name);
-			}
-
-			while (type) {
-				Object::Ptr prototype = type->GetPrototype();
-				Dictionary::Ptr dict = dynamic_pointer_cast<Dictionary>(prototype);
-
-				if (dict) {
-					ObjectLock olock(dict);
-					BOOST_FOREACH(const Dictionary::Pair& kv, dict) {
-						AddSuggestion(matches, word, pword + "." + kv.first);
-					}
-				}
-
-				type = type->GetBaseType();
-			}
 		} catch (...) { /* Ignore the exception */ }
 	}
 
