@@ -24,7 +24,6 @@
 #include "base/json.hpp"
 #include "base/configtype.hpp"
 #include "base/logger.hpp"
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string.hpp>
 
 using namespace icinga;
@@ -34,21 +33,12 @@ Type::Ptr FilterUtility::TypeFromPluralName(const String& pluralName)
 	String uname = pluralName;
 	boost::algorithm::to_lower(uname);
 
-	{
-		Dictionary::Ptr globals = ScriptGlobal::GetGlobals();
-		ObjectLock olock(globals);
-		BOOST_FOREACH(const Dictionary::Pair& kv, globals) {
-			if (!kv.second.IsObjectType<Type>())
-				continue;
+	for (const Type::Ptr&type : Type::GetAllTypes()) {
+		String pname = type->GetPluralName();
+		boost::algorithm::to_lower(pname);
 
-			Type::Ptr type = kv.second;
-
-			String pname = type->GetPluralName();
-			boost::algorithm::to_lower(pname);
-
-			if (uname == pname)
-				return type;
-		}
+		if (uname == pname)
+			return type;
 	}
 
 	return Type::Ptr();
@@ -56,10 +46,11 @@ Type::Ptr FilterUtility::TypeFromPluralName(const String& pluralName)
 
 void ConfigObjectTargetProvider::FindTargets(const String& type, const boost::function<void (const Value&)>& addTarget) const
 {
-	ConfigType::Ptr dtype = ConfigType::GetByName(type);
+	Type::Ptr ptype = Type::GetByName(type);
+	ConfigType *ctype = dynamic_cast<ConfigType *>(ptype.get());
 
-	if (dtype) {
-		BOOST_FOREACH(const ConfigObject::Ptr& object, dtype->GetObjects()) {
+	if (ctype) {
+		for (const ConfigObject::Ptr& object : ctype->GetObjects()) {
 			addTarget(object);
 		}
 	}
@@ -133,9 +124,9 @@ bool FilterUtility::EvaluateFilter(ScriptFrame& frame, Expression *filter,
 }
 
 static void FilteredAddTarget(ScriptFrame& permissionFrame, Expression *permissionFilter,
-    ScriptFrame& frame, Expression *ufilter, std::vector<Value>& result, const Object::Ptr& target)
+    ScriptFrame& frame, Expression *ufilter, std::vector<Value>& result, const String& variableName, const Object::Ptr& target)
 {
-	if (FilterUtility::EvaluateFilter(permissionFrame, permissionFilter, target) && FilterUtility::EvaluateFilter(frame, ufilter, target))
+	if (FilterUtility::EvaluateFilter(permissionFrame, permissionFilter, target, variableName) && FilterUtility::EvaluateFilter(frame, ufilter, target, variableName))
 		result.push_back(target);
 }
 
@@ -153,7 +144,7 @@ void FilterUtility::CheckPermission(const ApiUser::Ptr& user, const String& perm
 	Array::Ptr permissions = user->GetPermissions();
 	if (permissions) {
 		ObjectLock olock(permissions);
-		BOOST_FOREACH(const Value& item, permissions) {
+		for (const Value& item : permissions) {
 			String permission;
 			Function::Ptr filter;
 			if (item.IsObjectType<Dictionary>()) {
@@ -187,7 +178,7 @@ void FilterUtility::CheckPermission(const ApiUser::Ptr& user, const String& perm
 		BOOST_THROW_EXCEPTION(ScriptError("Missing permission: " + requiredPermission));
 }
 
-std::vector<Value> FilterUtility::GetFilterTargets(const QueryDescription& qd, const Dictionary::Ptr& query, const ApiUser::Ptr& user)
+std::vector<Value> FilterUtility::GetFilterTargets(const QueryDescription& qd, const Dictionary::Ptr& query, const ApiUser::Ptr& user, const String& variableName)
 {
 	std::vector<Value> result;
 
@@ -203,7 +194,7 @@ std::vector<Value> FilterUtility::GetFilterTargets(const QueryDescription& qd, c
 
 	ScriptFrame permissionFrame;
 
-	BOOST_FOREACH(const String& type, qd.Types) {
+	for (const String& type : qd.Types) {
 		String attr = type;
 		boost::algorithm::to_lower(attr);
 
@@ -211,10 +202,13 @@ std::vector<Value> FilterUtility::GetFilterTargets(const QueryDescription& qd, c
 			attr = "name";
 
 		if (query->Contains(attr)) {
-			Object::Ptr target = provider->GetTargetByName(type, HttpUtility::GetLastParameter(query, attr));
+			String name = HttpUtility::GetLastParameter(query, attr);
+			Object::Ptr target = provider->GetTargetByName(type, name);
 
-			if (FilterUtility::EvaluateFilter(permissionFrame, permissionFilter, target))
-				result.push_back(target);
+			if (!FilterUtility::EvaluateFilter(permissionFrame, permissionFilter, target, variableName))
+				BOOST_THROW_EXCEPTION(ScriptError("Access denied to object '" + name + "' of type '" + type + "'"));
+
+			result.push_back(target);
 		}
 
 		attr = provider->GetPluralName(type);
@@ -224,11 +218,13 @@ std::vector<Value> FilterUtility::GetFilterTargets(const QueryDescription& qd, c
 			Array::Ptr names = query->Get(attr);
 			if (names) {
 				ObjectLock olock(names);
-				BOOST_FOREACH(const String& name, names) {
+				for (const String& name : names) {
 					Object::Ptr target = provider->GetTargetByName(type, name);
 
-					if (FilterUtility::EvaluateFilter(permissionFrame, permissionFilter, target))
-						result.push_back(target);
+					if (!FilterUtility::EvaluateFilter(permissionFrame, permissionFilter, target, variableName))
+						BOOST_THROW_EXCEPTION(ScriptError("Access denied to object '" + name + "' of type '" + type + "'"));
+
+					result.push_back(target);
 				}
 			}
 		}
@@ -260,7 +256,7 @@ std::vector<Value> FilterUtility::GetFilterTargets(const QueryDescription& qd, c
 		Dictionary::Ptr filter_vars = query->Get("filter_vars");
 		if (filter_vars) {
 			ObjectLock olock(filter_vars);
-			BOOST_FOREACH(const Dictionary::Pair& kv, filter_vars) {
+			for (const Dictionary::Pair& kv : filter_vars) {
 				uvars->Set(kv.first, kv.second);
 			}
 		}
@@ -270,7 +266,7 @@ std::vector<Value> FilterUtility::GetFilterTargets(const QueryDescription& qd, c
 		try {
 			provider->FindTargets(type, boost::bind(&FilteredAddTarget,
 			    boost::ref(permissionFrame), permissionFilter,
-			    boost::ref(frame), ufilter, boost::ref(result), _1));
+			    boost::ref(frame), ufilter, boost::ref(result), variableName, _1));
 		} catch (const std::exception& ex) {
 			delete ufilter;
 			throw;
