@@ -37,10 +37,12 @@ static boost::once_flag l_HttpServerConnectionOnceFlag = BOOST_ONCE_INIT;
 static Timer::Ptr l_HttpServerConnectionTimeoutTimer;
 
 HttpServerConnection::HttpServerConnection(const String& identity, bool authenticated, const TlsStream::Ptr& stream)
-	: m_Stream(stream), m_CurrentRequest(stream), m_Seen(Utility::GetTime()), m_PendingRequests(0)
+	: m_Stream(stream), m_Seen(Utility::GetTime()), m_CurrentRequest(stream), m_PendingRequests(0)
 {
 	boost::call_once(l_HttpServerConnectionOnceFlag, &HttpServerConnection::StaticInitialize);
 
+	m_RequestQueue.SetName("HttpServerConnection");
+
 	if (authenticated)
 		m_ApiUser = ApiUser::GetByClientCN(identity);
 }
@@ -78,7 +80,10 @@ void HttpServerConnection::Disconnect(void)
 	ApiListener::Ptr listener = ApiListener::GetInstance();
 	listener->RemoveHttpClient(this);
 
-	m_Stream->Shutdown();
+	m_CurrentRequest.~HttpRequest();
+	new (&m_CurrentRequest) HttpRequest(Stream::Ptr());
+
+	m_Stream->Close();
 }
 
 bool HttpServerConnection::ProcessMessage(void)
@@ -161,7 +166,7 @@ void HttpServerConnection::ProcessMessageAsync(HttpRequest& request)
 
 	Log(LogInformation, "HttpServerConnection")
 	    << "Request: " << request.RequestMethod << " " << requestUrl
-	    << " (" << (user ? user->GetName() : "<unauthenticated>") << ")";
+	    << " (from " << m_Stream->GetSocket()->GetPeerAddress() << ", user: " << (user ? user->GetName() : "<unauthenticated>") << ")";
 
 	HttpResponse response(m_Stream, request);
 
@@ -234,7 +239,7 @@ void HttpServerConnection::TimeoutTimerHandler(void)
 {
 	ApiListener::Ptr listener = ApiListener::GetInstance();
 
-	BOOST_FOREACH(const HttpServerConnection::Ptr& client, listener->GetHttpClients()) {
+	for (const HttpServerConnection::Ptr& client : listener->GetHttpClients()) {
 		client->CheckLiveness();
 	}
 }
