@@ -24,7 +24,6 @@
 #include "base/configtype.hpp"
 #include "base/logger.hpp"
 #include "base/utility.hpp"
-#include <boost/foreach.hpp>
 
 using namespace icinga;
 
@@ -32,20 +31,17 @@ REGISTER_APIFUNCTION(Heartbeat, event, &JsonRpcConnection::HeartbeatAPIHandler);
 
 static Timer::Ptr l_HeartbeatTimer;
 
-static void StartHeartbeatTimer(void)
-{
+INITIALIZE_ONCE([]() {
 	l_HeartbeatTimer = new Timer();
 	l_HeartbeatTimer->OnTimerExpired.connect(boost::bind(&JsonRpcConnection::HeartbeatTimerHandler));
 	l_HeartbeatTimer->SetInterval(10);
 	l_HeartbeatTimer->Start();
-}
-
-INITIALIZE_ONCE(StartHeartbeatTimer);
+});
 
 void JsonRpcConnection::HeartbeatTimerHandler(void)
 {
-	BOOST_FOREACH(const Endpoint::Ptr& endpoint, ConfigType::GetObjectsByType<Endpoint>()) {
-		BOOST_FOREACH(const JsonRpcConnection::Ptr& client, endpoint->GetClients()) {
+	for (const Endpoint::Ptr& endpoint : ConfigType::GetObjectsByType<Endpoint>()) {
+		for (const JsonRpcConnection::Ptr& client : endpoint->GetClients()) {
 			if (client->m_NextHeartbeat != 0 && client->m_NextHeartbeat < Utility::GetTime()) {
 				Log(LogWarning, "JsonRpcConnection")
 				    << "Client for endpoint '" << endpoint->GetName() << "' has requested "
