@@ -26,6 +26,7 @@
 #include "base/utility.hpp"
 #include "base/logger.hpp"
 #include "base/exception.hpp"
+#include "base/convert.hpp"
 #include <boost/thread/once.hpp>
 
 using namespace icinga;
@@ -62,6 +63,10 @@ void JsonRpcConnection::StaticInitialize(void)
 
 	l_JsonRpcConnectionWorkQueueCount = Application::GetConcurrency();
 	l_JsonRpcConnectionWorkQueues = new WorkQueue[l_JsonRpcConnectionWorkQueueCount];
+
+	for (size_t i = 0; i < l_JsonRpcConnectionWorkQueueCount; i++) {
+		l_JsonRpcConnectionWorkQueues[i].SetName("JsonRpcConnection, #" + Convert::ToString(i));
+	}
 }
 
 void JsonRpcConnection::Start(void)
@@ -229,6 +234,9 @@ void JsonRpcConnection::DataAvailableHandler(void)
 {
 	bool close = false;
 
+	if (!m_Stream)
+		return;
+
 	if (!m_Stream->IsEof()) {
 		boost::mutex::scoped_lock lock(m_DataHandlerMutex);
 
@@ -321,12 +329,12 @@ void JsonRpcConnection::TimeoutTimerHandler(void)
 {
 	ApiListener::Ptr listener = ApiListener::GetInstance();
 
-	BOOST_FOREACH(const JsonRpcConnection::Ptr& client, listener->GetAnonymousClients()) {
+	for (const JsonRpcConnection::Ptr& client : listener->GetAnonymousClients()) {
 		client->CheckLiveness();
 	}
 
-	BOOST_FOREACH(const Endpoint::Ptr& endpoint, ConfigType::GetObjectsByType<Endpoint>()) {
-		BOOST_FOREACH(const JsonRpcConnection::Ptr& client, endpoint->GetClients()) {
+	for (const Endpoint::Ptr& endpoint : ConfigType::GetObjectsByType<Endpoint>()) {
+		for (const JsonRpcConnection::Ptr& client : endpoint->GetClients()) {
 			client->CheckLiveness();
 		}
 	}
