@@ -38,18 +38,7 @@ public:
 	virtual void FindTargets(const String& type,
 	    const boost::function<void (const Value&)>& addTarget) const override
 	{
-		std::vector<Type::Ptr> targets;
-
-		{
-			Dictionary::Ptr globals = ScriptGlobal::GetGlobals();
-			ObjectLock olock(globals);
-			BOOST_FOREACH(const Dictionary::Pair& kv, globals) {
-				if (kv.second.IsObjectType<Type>())
-					targets.push_back(kv.second);
-			}
-		}
-
-		BOOST_FOREACH(const Type::Ptr& target, targets) {
+		for (const Type::Ptr& target : Type::GetAllTypes()) {
 			addTarget(target);
 		}
 	}
@@ -85,6 +74,7 @@ bool TypeQueryHandler::HandleRequest(const ApiUser::Ptr& user, HttpRequest& requ
 
 	QueryDescription qd;
 	qd.Types.insert("Type");
+	qd.Permission = "types";
 	qd.Provider = new TypeTargetProvider();
 
 	if (params->Contains("type"))
@@ -108,7 +98,7 @@ bool TypeQueryHandler::HandleRequest(const ApiUser::Ptr& user, HttpRequest& requ
 
 	Array::Ptr results = new Array();
 
-	BOOST_FOREACH(const Type::Ptr& obj, objs) {
+	for (const Type::Ptr& obj : objs) {
 		Dictionary::Ptr result1 = new Dictionary();
 		results->Add(result1);
 
@@ -126,7 +116,7 @@ bool TypeQueryHandler::HandleRequest(const ApiUser::Ptr& user, HttpRequest& requ
 
 		if (prototype) {
 			ObjectLock olock(prototype);
-			BOOST_FOREACH(const Dictionary::Pair& kv, prototype) {
+			for (const Dictionary::Pair& kv : prototype) {
 				prototypeKeys->Add(kv.first);
 			}
 		}
