@@ -21,7 +21,7 @@
 #include "remote/zone.tcpp"
 #include "remote/jsonrpcconnection.hpp"
 #include "base/objectlock.hpp"
-#include <boost/foreach.hpp>
+#include "base/logger.hpp"
 
 using namespace icinga;
 
@@ -29,14 +29,33 @@ REGISTER_TYPE(Zone);
 
 void Zone::OnAllConfigLoaded(void)
 {
+	ObjectImpl<Zone>::OnAllConfigLoaded();
+
 	m_Parent = Zone::GetByName(GetParentRaw());
 
 	Zone::Ptr zone = m_Parent;
+	int levels = 0;
+
+	Array::Ptr endpoints = GetEndpointsRaw();
+
+	if (endpoints) {
+		ObjectLock olock(endpoints);
+		for (const String& endpoint : endpoints) {
+			Endpoint::Ptr ep = Endpoint::GetByName(endpoint);
+
+			if (ep)
+				ep->SetCachedZone(this);
+		}
+	}
 
 	while (zone) {
 		m_AllParents.push_back(zone);
 
 		zone = Zone::GetByName(zone->GetParentRaw());
+		levels++;
+
+		if (levels > 32)
+			BOOST_THROW_EXCEPTION(ScriptError("Infinite recursion detected while resolving zone graph. Check your zone hierarchy.", GetDebugInfo()));
 	}
 }
 
@@ -54,7 +73,7 @@ std::set<Endpoint::Ptr> Zone::GetEndpoints(void) const
 	if (endpoints) {
 		ObjectLock olock(endpoints);
 
-		BOOST_FOREACH(const String& name, endpoints) {
+		for (const String& name : endpoints) {
 			Endpoint::Ptr endpoint = Endpoint::GetByName(name);
 
 			if (!endpoint)
@@ -84,6 +103,9 @@ bool Zone::CanAccessObject(const ConfigObject::Ptr& object)
 	if (!object_zone)
 		object_zone = Zone::GetLocalZone();
 
+	if (object_zone->GetGlobal())
+		return true;
+
 	return object_zone->IsChildOf(this);
 }
 
@@ -106,6 +128,12 @@ bool Zone::IsGlobal(void) const
 	return GetGlobal();
 }
 
+bool Zone::IsSingleInstance(void) const
+{
+	Array::Ptr endpoints = GetEndpointsRaw();
+	return !endpoints || endpoints->GetLength() < 2;
+}
+
 Zone::Ptr Zone::GetLocalZone(void)
 {
 	Endpoint::Ptr local = Endpoint::GetLocalEndpoint();
@@ -116,3 +144,14 @@ Zone::Ptr Zone::GetLocalZone(void)
 	return local->GetZone();
 }
 
+void Zone::ValidateEndpointsRaw(const Array::Ptr& value, const ValidationUtils& utils)
+{
+	ObjectImpl<Zone>::ValidateEndpointsRaw(value, utils);
+
+	if (value && value->GetLength() > 2) {
+		Log(LogWarning, "Zone")
+		    << "The Zone object '" << GetName() << "' has more than two endpoints."
+		    << " Due to a known issue this type of configuration is strongly"
+		    << " discouraged and may cause Icinga to use excessive amounts of CPU time.";
+	}
+}
