@@ -19,7 +19,7 @@
 
 #include "base/convert.hpp"
 #include "base/object.hpp"
-#include <boost/test/unit_test.hpp>
+#include <BoostTestTargetConfig.h>
 #include <iostream>
 
 using namespace icinga;
@@ -46,14 +46,14 @@ BOOST_AUTO_TEST_CASE(todouble)
 BOOST_AUTO_TEST_CASE(tostring)
 {
 	BOOST_CHECK(Convert::ToString(7) == "7");
-	BOOST_CHECK(Convert::ToString(7.5) == "7.5");
+	BOOST_CHECK(Convert::ToString(7.5) == "7.500000");
 	BOOST_CHECK(Convert::ToString("hello") == "hello");
 
 	String str = "hello";
 	BOOST_CHECK(Convert::ToString(str) == "hello");
 
 	BOOST_CHECK(Convert::ToString(Value(7)) == "7");
-	BOOST_CHECK(Convert::ToString(Value(7.5)) == "7.5");
+	BOOST_CHECK(Convert::ToString(Value(7.5)) == "7.500000");
 	BOOST_CHECK(Convert::ToString(Value("hello")) == "hello");
 	BOOST_CHECK(Convert::ToString(Value("hello hello")) == "hello hello");
 }
