@@ -20,8 +20,7 @@
 #include "base/dictionary.hpp"
 #include "base/objectlock.hpp"
 #include "base/json.hpp"
-#include <boost/test/unit_test.hpp>
-#include <boost/foreach.hpp>
+#include <BoostTestTargetConfig.h>
 #include <boost/tuple/tuple.hpp>
 
 using namespace icinga;
@@ -82,7 +81,7 @@ BOOST_AUTO_TEST_CASE(foreach)
 
 	bool seen_test1 = false, seen_test2 = false;
 
-	BOOST_FOREACH(const Dictionary::Pair& kv, dictionary) {
+	for (const Dictionary::Pair& kv : dictionary) {
 		BOOST_CHECK(kv.first == "test1" || kv.first == "test2");
 
 		if (kv.first == "test1") {
