@@ -22,8 +22,7 @@
 #include "base/objectlock.hpp"
 #include "base/application.hpp"
 #include "base/type.hpp"
-#include <boost/test/unit_test.hpp>
-#include <boost/foreach.hpp>
+#include <BoostTestTargetConfig.h>
 #include <boost/tuple/tuple.hpp>
 
 using namespace icinga;
@@ -58,7 +57,7 @@ BOOST_AUTO_TEST_CASE(instantiate)
 {
 	Type::Ptr t = Type::GetByName("PerfdataValue");
 
-	Object::Ptr p = t->Instantiate();
+	Object::Ptr p = t->Instantiate(std::vector<Value>());
 
 	BOOST_CHECK(p);
 }
