@@ -19,7 +19,7 @@
 
 #include "config/configcompiler.hpp"
 #include "base/exception.hpp"
-#include <boost/test/unit_test.hpp>
+#include <BoostTestTargetConfig.h>
 
 using namespace icinga;
 
