@@ -19,7 +19,7 @@
 
 #include "icinga/perfdatavalue.hpp"
 #include "icinga/pluginutility.hpp"
-#include <boost/test/unit_test.hpp>
+#include <BoostTestTargetConfig.h>
 
 using namespace icinga;
 
