@@ -19,8 +19,7 @@
 
 #include "base/array.hpp"
 #include "remote/url.hpp"
-#include <boost/test/unit_test.hpp>
-#include <boost/foreach.hpp>
+#include <BoostTestTargetConfig.h>
 #include <boost/assign/list_of.hpp>
 
 using namespace icinga;
@@ -92,6 +91,7 @@ BOOST_AUTO_TEST_CASE(format)
 
 	url = new Url("/foo/bar/index.php?blaka");
 	BOOST_CHECK(new Url(url->Format()));
+	BOOST_CHECK(url->Format() == "/foo/bar/index.php?blaka");
 
 	url = new Url("/");
 	BOOST_CHECK(url->Format() == "/");
