@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -159,7 +159,8 @@ struct Field
 
 enum TypeAttribute
 {
-	TAAbstract = 1
+	TAAbstract = 1,
+	TAVarArgConstructor = 2
 };
 
 struct Klass
